/*                                                               28.Mar.2000 v2.1
  ================================================================================

                          U    U   GGG    SSSS  TTTTT
                          U    U  G       S       T
                          U    U  G  GG   SSSS    T
                          U    U  G   G       S   T
                           UUU     GG     SSS     T

                   ========================================
                    ITU-T - USER'S GROUP ON SOFTWARE TOOLS
                   ========================================

       =============================================================
       COPYRIGHT NOTE: This source code, and all of its derivations,
       is subject to the "ITU-T General Public License". Please have
       it  read  in    the  distribution  disk,   or  in  the  ITU-T
       Recommendation G.191 on "SOFTWARE TOOLS FOR SPEECH AND  AUDIO
       CODING STANDARDS".
       =============================================================

MODULE:         FIRFLT, HIGH QUALITY FIR UP/DOWN-SAMPLING FILTER
                Sub-unit: 
		Wideband speech filtering functions:
		- P.341 send-part characteristic
		- 50 Hz to 5 kHz bandpass filter (for wideband qual.tests)

ORIGINAL BY:
                Simao Ferraz de Campos Neto (C code)
		Comsat Laboratories             Tel:    +1-301-428-4516
		22300 Comsat Drive              Fax:    +1-301-428-9287
		Clarksburg MD 20871 - USA       E-mail: simao@ctd.comsat.com

		Rosario Drogo de Iacovo (filter coefficients)
		CSELT
		Audio Coding and Transmission Quality  Tel: +39-11-228-5697
		Via G.Reiss Romoli, 274                Fax: +39-11-228-6207
		10148 Torino - ITALY     
		Email: rosario.drogodeiacovo@cselt.stet.it

DESCRIPTION:
        This file contains procedures for FIR filtering according to the 
	mask in ITU-T Recommendation P.341 for wideband telephones.

FUNCTIONS:
  Global (have prototype in firflt.h)
  Local (should be used only here -- prototypes only in this file)

  Global (have prototype in firflt.h)
         = p341_16khz_init(): initialize P.341 send-part weighting filter
	 = bp5k_16khz_init(): initialize 5kHz bandpass filter

  Local (should be used only here -- prototypes only in this file)
         = fill_p341_16khz(...): filling filter coefficients into
                                 array for P.341 send-part weighting
         = fill_bp5k_16khz(...): idem, for 5kHz bandpass filtering


HISTORY:
    31.Jul.1995 v1.0 First beta-version <simao@ctd.comsat.com>
    13.May.1997 v2.0 Added 50-5000 Hz bandpass filter
    28.Mar.2000 v2.1 Added type casting to float in fill_p341_16khz() 
                     and fill_bp5k_16khz() to eliminate compilation warnings in 
                     MS Visual C compiler <simao.campos@labs.comsat.com>

  =============================================================================
*/


/*
 * ......... INCLUDES .........
 */
#include <stdio.h>
#ifndef VMS
#include <stdlib.h>		/* General utility definitions */
#endif

#include "firflt.h"		/* Global definitions for FIR-FIR filter */

#define F float

/*
 * ......... Local function prototypes .........
 */
void fill_p341_16kHz ARGS((float **h0, long *lenh0));
void fill_bp5k_16khz ARGS((float **h0, long *lenh0));


/* 
 * ..... Private function prototypes defined in other sub-unit ..... 
 */
extern SCD_FIR *fir_initialization ARGS((long lenh0, float h0[], double gain, 
                                                 long idwnup, int hswitch));


/*
 * ...................... BEGIN OF FUNCTIONS .........................
 */

/*
  ============================================================================

        void fill_p341_16khz (float **h0, long *lenh0);
        ~~~~~~~~~~~~~~~~~~~~~

        Description:
        ~~~~~~~~~~~~
	Fill filter coefficients for the sending mask of ITU-T 
	Recommendation P.341 for samples at 16 kHz.

        Parameters:
        ~~~~~~~~~~~
        h0:    (Out) pointer to array with FIR coefficients
        lenh0: (Out) pointer to number of coefficients

        Return value:
        ~~~~~~~~~~~~~
        None.

        Author: 
        ~~~~~~~
        Filter coefficients: 
          Rosario Drogo de Iacovo
          CSELT
          Audio Coding and Transm.Quality  Tel:    +39-11-228-5697
          Via G.Reiss Romoli, 274	   Fax:    +39-11-228-6207
          10148 Torino	    E-mail: rosario.drogodeiacovo@cselt.stet.it
          ITALY
          *** Copyright Notice: ***
          " (C) 1995 CSELT - Centro Studi E Laboratori Telecomunicazioni SpA "
          " All rights reserved "
          
        Routine:
          Simao Ferraz de Campos Neto
	  Comsat Laboratories             Tel:    +1-301-428-4516
	  22300 Comsat Drive              Fax:    +1-301-428-9287
	  Clarksburg MD 20871 - USA       E-mail: simao@ctd.comsat.com

        History:
        ~~~~~~~~
        01.Jul.1995 v1.0 Release of 1st version <simao@ctd.comsat.com>
        28.Mar.2000 v1.1 Added type cast to float to avoid compilation 
                         warnings with MS Visual C compiler <simao>

 ============================================================================
*/
#undef P341_16K_LEN
#define P341_16K_LEN 592
void fill_p341_16khz (h0, lenh0)
float **h0;
long *lenh0;
{
  static float p341_16khz_coeff[P341_16K_LEN] =  {
    (F)4.3370554e-03,  (F) 4.8963084e-03,  (F)-1.0865770e-03,  (F)-2.2118514e-04,  
    (F)1.5783547e-03,  (F)-9.0371860e-04,  (F) 1.0025039e-03,  (F) 2.7932322e-04,  
    (F)-1.8063678e-04, (F) 1.1228519e-03,  (F)-4.7977326e-04,  (F) 9.2887905e-04,   
    (F)1.0086939e-04,  (F) 2.0354803e-04,  (F) 8.0081349e-04,  (F)-2.7302929e-04,   
    (F)1.0026686e-03,  (F)-1.5205656e-04,  (F) 6.5360301e-04,  (F) 3.6157206e-04,   
    (F)1.3088441e-04,  (F) 8.1871439e-04,  (F)-1.4804904e-04,  (F) 9.0526585e-04,  
    (F)-2.3134480e-05, (F) 6.2321132e-04,  (F) 3.6859087e-04,  (F) 2.2122380e-04,   
    (F)7.4408704e-04,  (F)-2.7046864e-05,  (F) 8.6265617e-04,  (F) 9.6974412e-06,   
    (F)6.9084036e-04,  (F) 2.9722760e-04,  (F) 3.6743357e-04,  (F) 6.3089112e-04,   
    (F)8.6009648e-05,  (F) 8.2634163e-04,  (F) 2.1135920e-05,  (F) 7.7420201e-04,   
    (F)1.8675999e-04,  (F) 5.2482294e-04,  (F) 4.9217224e-04,  (F) 2.1806727e-04,   
    (F)7.5394748e-04,  (F) 3.9345021e-05,  (F) 8.2948886e-04,  (F) 7.8974577e-05,   
    (F)6.7639004e-04,  (F) 3.2520622e-04,  (F) 3.7250845e-04,  (F) 6.3361567e-04,   
    (F)9.5321090e-05,  (F) 8.2844632e-04,  (F)-2.5454455e-06,  (F) 7.9803746e-04,   
    (F)1.4418025e-04,  (F) 5.3996243e-04,  (F) 4.5701966e-04,  (F) 1.9639338e-04,   
    (F)7.5403862e-04,  (F)-3.5994011e-05,  (F) 8.6171870e-04,  (F)-2.6544669e-05,   
    (F)6.9943078e-04,  (F) 2.3383425e-04,  (F) 3.4376122e-04,  (F) 5.9758270e-04,  
    (F)-9.6120972e-06, (F) 8.4465806e-04,  (F)-1.5563653e-04,  (F) 8.2137416e-04,  
    (F)-8.7407682e-06, (F) 5.1653962e-04,  (F) 3.5863004e-04,  (F) 8.7334681e-05,   
    (F)7.2954959e-04,  (F)-2.2555892e-04,  (F) 8.7318212e-04,  (F)-2.3707685e-04,   
    (F)6.8138182e-04,  (F) 6.2581832e-05,  (F) 2.4350658e-04,  (F) 5.0432825e-04,  
    (F)-2.1007442e-04, (F) 8.2198119e-04,  (F)-4.1934647e-04,  (F) 8.0356853e-04,  
    (F)-2.5747280e-04, (F) 4.3052777e-04,  (F) 1.8509211e-04,  (F)-1.0928392e-04,   
    (F)6.4392889e-04,  (F)-5.1589872e-04,  (F) 8.3364848e-04,  (F)-5.5830975e-04,   
    (F)6.1069081e-04,  (F)-2.0610123e-04,  (F) 6.5714839e-05,  (F) 3.3753302e-04,  
    (F)-5.1022240e-04, (F) 7.3685066e-04,  (F)-7.9250753e-04,  (F) 7.2985394e-04,  
    (F)-6.1741765e-04, (F) 2.7802304e-04,  (F)-8.5035597e-05,  (F)-3.9297408e-04,   
    (F)4.8598972e-04,  (F)-9.1773152e-04,  (F) 7.3717440e-04,  (F)-9.9775731e-04,   
    (F)4.7875226e-04,  (F)-5.8379125e-04,  (F)-1.8844324e-04,  (F) 8.1040442e-05,  
    (F)-9.0973035e-04, (F) 5.8654411e-04,  (F)-1.2840812e-03,  (F) 6.0043298e-04,  
    (F)-1.0983587e-03, (F) 6.2571417e-05,  (F)-4.5858369e-04,  (F)-7.6343601e-04,   
    (F)2.5067647e-04,  (F)-1.4301295e-03,  (F) 5.8401655e-04,  (F)-1.5604368e-03,   
    (F)2.9356460e-04,  (F)-1.0786265e-03,  (F)-5.0987919e-04,  (F)-2.6825842e-04,  
    (F)-1.4059932e-03, (F) 3.7403302e-04,  (F)-1.8974634e-03,  (F) 4.2591529e-04,  
    (F)-1.7042091e-03, (F)-2.0398233e-04,  (F)-9.3923499e-04,  (F)-1.2114668e-03,  
    (F)-5.8080630e-05, (F)-2.0523333e-03,  (F) 3.8627175e-04,  (F)-2.2515963e-03,   
    (F)7.2968802e-05,  (F)-1.6928681e-03,  (F)-8.8733736e-04,  (F)-7.0504743e-04,  
    (F)-1.9928540e-03, (F) 1.1102043e-04,  (F)-2.6316794e-03,  (F) 2.2534739e-04,  
    (F)-2.4382822e-03, (F)-5.0346667e-04,  (F)-1.5226739e-03,  (F)-1.7244598e-03,  
    (F)-4.2924708e-04, (F)-2.7776651e-03,  (F) 1.6369469e-04,  (F)-3.0692266e-03,  
    (F)-1.5983374e-04, (F)-2.4264440e-03,  (F)-1.2978141e-03,  (F)-1.2209540e-03,  
    (F)-2.6567776e-03, (F)-1.8115649e-04,  (F)-3.4821691e-03,  (F) 2.6753403e-05,  
    (F)-3.2989222e-03, (F)-8.0478000e-04,  (F)-2.2051339e-03,  (F)-2.2785787e-03,  
    (F)-8.4385502e-04, (F)-3.5967515e-03,  (F)-5.0173398e-05,  (F)-4.0131403e-03,  
    (F)-3.6705838e-04, (F)-3.2753517e-03,  (F)-1.7085687e-03,  (F)-1.8026956e-03,  
    (F)-3.3777654e-03, (F)-4.7183504e-04,  (F)-4.4452577e-03,  (F)-1.2428060e-04,  
    (F)-4.2890581e-03, (F)-1.0623451e-03,  (F)-2.9794960e-03,  (F)-2.8436367e-03,  
    (F)-1.2738972e-03, (F)-4.5010059e-03,  (F)-2.0775726e-04,  (F)-5.0879740e-03,  
    (F)-4.9182557e-04, (F)-4.2421463e-03,  (F)-2.0756444e-03,  (F)-2.4300192e-03,  
    (F)-4.1369631e-03, (F)-7.1497396e-04,  (F)-5.5241206e-03,  (F)-1.6451873e-04,  
    (F)-5.4186736e-03, (F)-1.2157506e-03,  (F)-3.8412127e-03,  (F)-3.3813839e-03,  
    (F)-1.6857146e-03, (F)-5.4837505e-03,  (F)-2.4198474e-04,  (F)-6.3135075e-03,  
    (F)-4.5275451e-04, (F)-5.3449802e-03,  (F)-2.3347566e-03,  (F)-3.0874491e-03,  
    (F)-4.9139526e-03, (F)-8.4630255e-04,  (F)-6.7421814e-03,  (F) 4.3080433e-06,  
    (F)-6.7284528e-03, (F)-1.1705590e-03,  (F)-4.8045274e-03,  (F)-3.8433771e-03,  
    (F)-2.0324242e-03, (F)-6.5617644e-03,  (F)-4.6088337e-05,  (F)-7.7515103e-03,  
    (F)-1.2058937e-04, (F)-6.6398327e-03,  (F)-2.3916210e-03,  (F)-3.7691379e-03,  
    (F)-5.6977852e-03, (F)-7.6554808e-04,  (F)-8.1781162e-03,  (F) 5.4994663e-04,  
    (F)-8.3300798e-03, (F)-7.6778376e-04,  (F)-5.9277142e-03,  (F)-4.1653326e-03,  
    (F)-2.2482199e-03, (F)-7.8081727e-03,  (F) 5.7356736e-04,  (F)-9.5723976e-03,   
    (F)7.4658049e-04,  (F)-8.2841868e-03,  (F)-2.0837611e-03,  (F)-4.4931875e-03,  
    (F)-6.5098454e-03, (F)-2.8797664e-04,  (F)-1.0049007e-02,  (F) 1.8130718e-03,  
    (F)-1.0526155e-02, (F) 3.2312204e-04,  (F)-7.3976945e-03,  (F)-4.2344785e-03,  
    (F)-2.2237199e-03, (F)-9.4489053e-03,  (F) 2.0567242e-03,  (F)-1.2269975e-02,   
    (F)2.7378903e-03,  (F)-1.0761805e-02,  (F)-1.0158328e-03,  (F)-5.3719963e-03,  
    (F)-7.4710863e-03, (F) 1.0692955e-03,  (F)-1.3064357e-02,  (F) 4.7583043e-03,  
    (F)-1.4306429e-02, (F) 3.0611092e-03,  (F)-9.8659778e-03,  (F)-3.7682029e-03,  
    (F)-1.6488584e-03, (F)-1.2344226e-02,  (F) 5.8795482e-03,  (F)-1.7704338e-02,   
    (F)7.9554449e-03,  (F)-1.6000280e-02,  (F) 2.2828656e-03,  (F)-6.9512174e-03,  
    (F)-9.1906329e-03, (F) 5.3517620e-03,  (F)-2.0621147e-02,  (F) 1.3976623e-02,  
    (F)-2.4857777e-02, (F) 1.2526417e-02,  (F)-1.7214612e-02,  (F)-1.1532203e-03,   
    (F)1.3568596e-03,  (F)-2.2756321e-02,  (F) 2.3310698e-02,  (F)-4.1958306e-02,   
    (F)3.6273020e-02,  (F)-4.5178953e-02,  (F) 2.6147879e-02,  (F)-1.7848425e-02,  
    (F)-2.3884241e-02, (F) 6.6509096e-02,  (F)-1.8213102e-01,  (F) 6.1948613e-01,   
    (F)6.1948613e-01,  (F)-1.8213102e-01,  (F) 6.6509096e-02,  (F)-2.3884241e-02,  
    (F)-1.7848425e-02, (F) 2.6147879e-02,  (F)-4.5178953e-02,  (F) 3.6273020e-02,  
    (F)-4.1958306e-02, (F) 2.3310698e-02,  (F)-2.2756321e-02,  (F) 1.3568596e-03,  
    (F)-1.1532203e-03, (F)-1.7214612e-02,  (F) 1.2526417e-02,  (F)-2.4857777e-02,   
    (F)1.3976623e-02,  (F)-2.0621147e-02,  (F) 5.3517620e-03,  (F)-9.1906329e-03,  
    (F)-6.9512174e-03, (F) 2.2828656e-03,  (F)-1.6000280e-02,  (F) 7.9554449e-03,  
    (F)-1.7704338e-02, (F) 5.8795482e-03,  (F)-1.2344226e-02,  (F)-1.6488584e-03,  
    (F)-3.7682029e-03, (F)-9.8659778e-03,  (F) 3.0611092e-03,  (F)-1.4306429e-02,   
    (F)4.7583043e-03,  (F)-1.3064357e-02,  (F) 1.0692955e-03,  (F)-7.4710863e-03,  
    (F)-5.3719963e-03, (F)-1.0158328e-03,  (F)-1.0761805e-02,  (F) 2.7378903e-03,  
    (F)-1.2269975e-02, (F) 2.0567242e-03,  (F)-9.4489053e-03,  (F)-2.2237199e-03,  
    (F)-4.2344785e-03, (F)-7.3976945e-03,  (F) 3.2312204e-04,  (F)-1.0526155e-02,   
    (F)1.8130718e-03,  (F)-1.0049007e-02,  (F)-2.8797664e-04,  (F)-6.5098454e-03,  
    (F)-4.4931875e-03, (F)-2.0837611e-03,  (F)-8.2841868e-03,  (F) 7.4658049e-04,  
    (F)-9.5723976e-03, (F) 5.7356736e-04,  (F)-7.8081727e-03,  (F)-2.2482199e-03,  
    (F)-4.1653326e-03, (F)-5.9277142e-03,  (F)-7.6778376e-04,  (F)-8.3300798e-03,   
    (F)5.4994663e-04,  (F)-8.1781162e-03,  (F)-7.6554808e-04,  (F)-5.6977852e-03,  
    (F)-3.7691379e-03, (F)-2.3916210e-03,  (F)-6.6398327e-03,  (F)-1.2058937e-04,  
    (F)-7.7515103e-03, (F)-4.6088337e-05,  (F)-6.5617644e-03,  (F)-2.0324242e-03,  
    (F)-3.8433771e-03, (F)-4.8045274e-03,  (F)-1.1705590e-03,  (F)-6.7284528e-03,   
    (F)4.3080433e-06,  (F)-6.7421814e-03,  (F)-8.4630255e-04,  (F)-4.9139526e-03,  
    (F)-3.0874491e-03, (F)-2.3347566e-03,  (F)-5.3449802e-03,  (F)-4.5275451e-04,  
    (F)-6.3135075e-03, (F)-2.4198474e-04,  (F)-5.4837505e-03,  (F)-1.6857146e-03,  
    (F)-3.3813839e-03, (F)-3.8412127e-03,  (F)-1.2157506e-03,  (F)-5.4186736e-03,  
    (F)-1.6451873e-04, (F)-5.5241206e-03,  (F)-7.1497396e-04,  (F)-4.1369631e-03,  
    (F)-2.4300192e-03, (F)-2.0756444e-03,  (F)-4.2421463e-03,  (F)-4.9182557e-04,  
    (F)-5.0879740e-03, (F)-2.0775726e-04,  (F)-4.5010059e-03,  (F)-1.2738972e-03,  
    (F)-2.8436367e-03, (F)-2.9794960e-03,  (F)-1.0623451e-03,  (F)-4.2890581e-03,  
    (F)-1.2428060e-04, (F)-4.4452577e-03,  (F)-4.7183504e-04,  (F)-3.3777654e-03,  
    (F)-1.8026956e-03, (F)-1.7085687e-03,  (F)-3.2753517e-03,  (F)-3.6705838e-04,  
    (F)-4.0131403e-03, (F)-5.0173398e-05,  (F)-3.5967515e-03,  (F)-8.4385502e-04,  
    (F)-2.2785787e-03, (F)-2.2051339e-03,  (F)-8.0478000e-04,  (F)-3.2989222e-03,   
    (F)2.6753403e-05,  (F)-3.4821691e-03,  (F)-1.8115649e-04,  (F)-2.6567776e-03,  
    (F)-1.2209540e-03, (F)-1.2978141e-03,  (F)-2.4264440e-03,  (F)-1.5983374e-04,  
    (F)-3.0692266e-03, (F) 1.6369469e-04,  (F)-2.7776651e-03,  (F)-4.2924708e-04,  
    (F)-1.7244598e-03, (F)-1.5226739e-03,  (F)-5.0346667e-04,  (F)-2.4382822e-03,   
    (F)2.2534739e-04,  (F)-2.6316794e-03,  (F) 1.1102043e-04,  (F)-1.9928540e-03,  
    (F)-7.0504743e-04, (F)-8.8733736e-04,  (F)-1.6928681e-03,  (F) 7.2968802e-05,  
    (F)-2.2515963e-03, (F) 3.8627175e-04,  (F)-2.0523333e-03,  (F)-5.8080630e-05,  
    (F)-1.2114668e-03, (F)-9.3923499e-04,  (F)-2.0398233e-04,  (F)-1.7042091e-03,   
    (F)4.2591529e-04,  (F)-1.8974634e-03,  (F) 3.7403302e-04,  (F)-1.4059932e-03,  
    (F)-2.6825842e-04, (F)-5.0987919e-04,  (F)-1.0786265e-03,  (F) 2.9356460e-04,  
    (F)-1.5604368e-03, (F) 5.8401655e-04,  (F)-1.4301295e-03,  (F) 2.5067647e-04,  
    (F)-7.6343601e-04, (F)-4.5858369e-04,  (F) 6.2571417e-05,  (F)-1.0983587e-03,   
    (F)6.0043298e-04,  (F)-1.2840812e-03,  (F) 5.8654411e-04,  (F)-9.0973035e-04,   
    (F)8.1040442e-05,  (F)-1.8844324e-04,  (F)-5.8379125e-04,  (F) 4.7875226e-04,  
    (F)-9.9775731e-04, (F) 7.3717440e-04,  (F)-9.1773152e-04,  (F) 4.8598972e-04,  
    (F)-3.9297408e-04, (F)-8.5035597e-05,  (F) 2.7802304e-04,  (F)-6.1741765e-04,   
    (F)7.2985394e-04,  (F)-7.9250753e-04,  (F) 7.3685066e-04,  (F)-5.1022240e-04,   
    (F)3.3753302e-04,  (F) 6.5714839e-05,  (F)-2.0610123e-04,  (F) 6.1069081e-04,  
    (F)-5.5830975e-04, (F) 8.3364848e-04,  (F)-5.1589872e-04,  (F) 6.4392889e-04,  
    (F)-1.0928392e-04, (F) 1.8509211e-04,  (F) 4.3052777e-04,  (F)-2.5747280e-04,   
    (F)8.0356853e-04,  (F)-4.1934647e-04,  (F) 8.2198119e-04,  (F)-2.1007442e-04,   
    (F)5.0432825e-04,  (F) 2.4350658e-04,  (F) 6.2581832e-05,  (F) 6.8138182e-04,  
    (F)-2.3707685e-04, (F) 8.7318212e-04,  (F)-2.2555892e-04,  (F) 7.2954959e-04,   
    (F)8.7334681e-05,  (F) 3.5863004e-04,  (F) 5.1653962e-04,  (F)-8.7407682e-06,   
    (F)8.2137416e-04,  (F)-1.5563653e-04,  (F) 8.4465806e-04,  (F)-9.6120972e-06,   
    (F)5.9758270e-04,  (F) 3.4376122e-04,  (F) 2.3383425e-04,  (F) 6.9943078e-04,  
    (F)-2.6544669e-05, (F) 8.6171870e-04,  (F)-3.5994011e-05,  (F) 7.5403862e-04,   
    (F)1.9639338e-04,  (F) 4.5701966e-04,  (F) 5.3996243e-04,  (F) 1.4418025e-04,   
    (F)7.9803746e-04,  (F)-2.5454455e-06,  (F) 8.2844632e-04,  (F) 9.5321090e-05,   
    (F)6.3361567e-04,  (F) 3.7250845e-04,  (F) 3.2520622e-04,  (F) 6.7639004e-04,   
    (F)7.8974577e-05,  (F) 8.2948886e-04,  (F) 3.9345021e-05,  (F) 7.5394748e-04,   
    (F)2.1806727e-04,  (F) 4.9217224e-04,  (F) 5.2482294e-04,  (F) 1.8675999e-04,   
    (F)7.7420201e-04,  (F) 2.1135920e-05,  (F) 8.2634163e-04,  (F) 8.6009648e-05,   
    (F)6.3089112e-04,  (F) 3.6743357e-04,  (F) 2.9722760e-04,  (F) 6.9084036e-04,   
    (F)9.6974412e-06,  (F) 8.6265617e-04,  (F)-2.7046864e-05,  (F) 7.4408704e-04,   
    (F)2.2122380e-04,  (F) 3.6859087e-04,  (F) 6.2321132e-04,  (F)-2.3134480e-05,   
    (F)9.0526585e-04,  (F)-1.4804904e-04,  (F) 8.1871439e-04,  (F) 1.3088441e-04,   
    (F)3.6157206e-04,  (F) 6.5360301e-04,  (F)-1.5205656e-04,  (F) 1.0026686e-03,  
    (F)-2.7302929e-04, (F) 8.0081349e-04,  (F) 2.0354803e-04,  (F) 1.0086939e-04,   
    (F)9.2887905e-04,  (F)-4.7977326e-04,  (F) 1.1228519e-03,  (F)-1.8063678e-04,   
    (F)2.7932322e-04,  (F) 1.0025039e-03,  (F)-9.0371860e-04,  (F) 1.5783547e-03,  
    (F)-2.2118514e-04, (F)-1.0865770e-03,  (F) 4.8963084e-03,  (F) 4.3370554e-03
  };

  *lenh0 = P341_16K_LEN;           /* store 'number of coefficients' */
  *h0 = p341_16khz_coeff;		/* store pointer to []-array */
}
#undef P341_16K_LEN
/* ...................... End of fill_p341_16khz() ..................... */


/*
  ============================================================================

        SCD_FIR *p341_16khz_init (void);
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        Description:
        ~~~~~~~~~~~~

        Initialization routine for ITU-T Recommendation send-part mask 
	filtering characteristic for data sampled at 16 kHz. No rate change 
	is implemented.

        Parameters:  none.
        ~~~~~~~~~~~

        Return value:
        ~~~~~~~~~~~~~
        Returns a pointer to struct SCD_FIR;

        Author: <simao@ctd.comsat.com>
        ~~~~~~~

        History:
        ~~~~~~~~
	01.Jul.95 v1.0 Release of 1st version <simao@ctd.comsat.com>

 ============================================================================
*/
SCD_FIR *p341_16khz_init ()
{
  float          *h0;		/* pointer to array with FIR coeff. */
  long            lenh0;	/* number of FIR coefficients */


  /* allocate array for FIR coeff. and fill with coefficients */
  fill_p341_16khz(&h0, &lenh0);

  return
    fir_initialization(		/* Returns: pointer to SCD_FIR-struct */
		       lenh0,	/* In: number of FIR-coefficients */
		       h0,	/* In: pointer to array with FIR-cof. */
		       1.0,	/* In: gain factor for FIR-coeffic. */
		       1l,	/* In: Down-sampling factor */
		       'D'	/* In: switch to down-sampling proc. */
    );				/* (works here as simple FIR-fil. */
}
/* .................... End of p341_16khz_init() .................... */


/*
  ============================================================================

        void fill_bp5k_16khz (float **h0, long *lenh0);
        ~~~~~~~~~~~~~~~~~~~~~

        Description:
        ~~~~~~~~~~~~
	Fill filter coefficients for limiting the bandwidth of the
	signals to 5 kHz for samples at 16 kHz.

        Parameters:
        ~~~~~~~~~~~
        h0:    (Out) pointer to array with FIR coefficients
        lenh0: (Out) pointer to number of coefficients

        Return value:
        ~~~~~~~~~~~~~
        None.

        Author: 
        ~~~~~~~
        Filter coefficients: 
          Rosario Drogo de Iacovo
          CSELT
          Audio Coding and Transm.Quality  Tel:    +39-11-228-5697
          Via G.Reiss Romoli, 274	   Fax:    +39-11-228-6207
          10148 Torino	    E-mail: rosario.drogodeiacovo@cselt.stet.it
          ITALY
          *** Copyright Notice: ***
          " (C) 1997 CSELT - Centro Studi E Laboratori Telecomunicazioni SpA "
          " All rights reserved "
          
        Routine:
          Simao Ferraz de Campos Neto
	  Comsat Laboratories             Tel:    +1-301-428-4516
	  22300 Comsat Drive              Fax:    +1-301-428-9287
	  Clarksburg MD 20871 - USA       E-mail: simao@ctd.comsat.com

        History:
        ~~~~~~~~
        13.May.97 v1.0 Release of 1st version <simao@ctd.comsat.com>
        28.Mar.2000 v1.1 Added type cast to float to avoid compilation 
                         warnings with MS Visual C compiler <simao>

 ============================================================================
*/
#undef BP5K_16K_LEN
#define BP5K_16K_LEN 592
void fill_bp5k_16khz (h0, lenh0)
float **h0;
long *lenh0;
{
  static float bp5k_16khz_coeff[BP5K_16K_LEN] =  {
   (F)2.0805499e-03,  (F) 3.9722215e-03,  (F) 3.2368711e-03,  (F) 6.3445872e-05,  
   (F)-1.3358539e-03, (F) 4.8035495e-04,  (F) 1.4837936e-03,  (F)-7.1432006e-05,  
   (F)-5.3197924e-04, (F) 9.4085515e-04,  (F) 8.3261061e-04,  (F)-4.2019012e-04,   
   (F)2.3758270e-04,  (F) 1.0771052e-03,  (F) 6.7905656e-05,  (F)-1.6575014e-04,   
   (F)9.0703407e-04,  (F) 5.2925119e-04,  (F)-2.4009910e-04,  (F) 5.8300451e-04,   
   (F)8.1158061e-04,  (F)-9.5331602e-05,  (F) 2.7649832e-04,  (F) 9.0170081e-04,  
   (F)1.4676282e-04,  (F) 6.6667703e-05,  (F) 8.4747506e-04,  (F) 3.9680262e-04,  
   (F)-2.9073419e-05, (F) 7.0953770e-04,  (F) 6.0362256e-04,  (F)-2.5249950e-05,   
   (F)5.3852836e-04,  (F) 7.4613466e-04,  (F) 5.2826418e-05,  (F) 3.6872795e-04,   
   (F)8.1936773e-04,  (F) 1.7834353e-04,  (F) 2.2038581e-04,  (F) 8.2492171e-04,   
   (F)3.2723682e-04,  (F) 1.0633143e-04,  (F) 7.7031646e-04,  (F) 4.8260334e-04,   
   (F)4.0287664e-05,  (F) 6.7170137e-04,  (F) 6.3253750e-04,  (F) 3.0359128e-05,   
   (F)5.3765695e-04,  (F) 7.5149712e-04,  (F) 6.7577648e-05,  (F) 3.7640856e-04,   
   (F)8.2555112e-04,  (F) 1.5900697e-04,  (F) 2.1942502e-04,  (F) 8.4946533e-04,   
   (F)2.9158385e-04,  (F) 7.4805292e-05,  (F) 8.0375966e-04,  (F) 4.4799518e-04,  
   (F)-2.6877102e-05, (F) 7.0140838e-04,  (F) 6.1047682e-04,  (F)-7.6935612e-05,
   (F)5.3898956e-04,  (F) 7.5079435e-04,  (F)-5.6697379e-05,  (F) 3.4489013e-04, 
   (F)8.4572319e-04,  (F) 2.5781549e-05,  (F) 1.3440062e-04,  (F) 8.7589585e-04,  
   (F)1.7022328e-04,  (F)-5.8804298e-05,  (F) 8.2351175e-04,  (F) 3.4868918e-04, 
   (F)-2.0786328e-04, (F) 6.9373941e-04,  (F) 5.3987170e-04,  (F)-2.9129914e-04,   
   (F)4.8968352e-04,  (F) 7.1065718e-04,  (F)-2.8792879e-04,  (F) 2.3721823e-04, 
   (F)8.2442752e-04,  (F)-2.0239452e-04,  (F)-3.3846228e-05,  (F) 8.5952215e-04, 
   (F)-4.2920075e-05, (F)-2.9124518e-04,  (F) 7.9570539e-04,  (F) 1.7050387e-04,  
   (F)-4.9268552e-04, (F) 6.2979774e-04,  (F) 3.9780715e-04,  (F)-6.1185209e-04,  
   (F)3.7506374e-04,  (F) 6.0138846e-04,  (F)-6.3116397e-04,  (F) 5.5801711e-05, 
   (F)7.4482882e-04,  (F)-5.4079543e-04,  (F)-2.9161075e-04,  (F) 7.8742167e-04, 
   (F)-3.5641282e-04, (F)-6.1987769e-04,  (F) 7.1061989e-04,  (F)-1.0650414e-04,  
   (F)-8.8526201e-04, (F) 5.1001679e-04,  (F) 1.6805066e-04,  (F)-1.0529198e-03,  
   (F)1.9525361e-04,  (F) 4.2103856e-04,  (F)-1.0919296e-03,  (F)-2.0205436e-04, 
   (F)5.9807594e-04,  (F)-9.9796908e-04,  (F)-6.3483778e-04,  (F) 6.5804381e-04, 
   (F)-7.8489062e-04, (F)-1.0488444e-03,  (F) 5.7331275e-04,  (F)-4.8723154e-04,  
   (F)-1.3916558e-03, (F) 3.3534474e-04,  (F)-1.5028960e-04,  (F)-1.6124011e-03, 
   (F)-4.4736478e-05, (F) 1.6210356e-04,  (F)-1.6781356e-03,  (F)-5.2768325e-04,  
   (F)3.8893919e-04,  (F)-1.5802909e-03,  (F)-1.0599137e-03,  (F) 4.7758328e-04,
   (F)-1.3322860e-03, (F)-1.5747678e-03,  (F) 3.9262526e-04,  (F)-9.7289874e-04,  
   (F)-2.0066034e-03, (F) 1.1906035e-04,  (F)-5.6017973e-04,  (F)-2.2933960e-03, 
   (F)-3.2942885e-04, (F)-1.6840325e-04,  (F)-2.3922045e-03,  (F)-9.0788612e-04,  
   (F)1.2766640e-04,  (F)-2.2900981e-03,  (F)-1.5537480e-03,  (F) 2.6244445e-04,
   (F)-1.9993089e-03, (F)-2.1869429e-03,  (F) 1.8869166e-04,  (F)-1.5643897e-03,  
   (F)-2.7257064e-03, (F)-1.1422064e-04,  (F)-1.0541415e-03,  (F)-3.0936849e-03, 
   (F)-6.3307831e-04, (F)-5.5588793e-04,  (F)-3.2350341e-03,  (F)-1.3201463e-03, 
   (F)-1.6402940e-04, (F)-3.1275771e-03,  (F)-2.0976384e-03,  (F) 4.0228664e-05, 
   (F)-2.7841155e-03, (F)-2.8720059e-03,  (F)-5.5816763e-06,  (F)-2.2532994e-03, 
   (F)-3.5399219e-03, (F)-3.2967385e-04,  (F)-1.6168623e-03,  (F)-4.0078194e-03, 
   (F)-9.2201973e-04, (F)-9.7976498e-04,  (F)-4.2043413e-03,  (F)-1.7302268e-03, 
   (F)-4.5514941e-04, (F)-4.0930890e-03,  (F)-2.6646442e-03,  (F)-1.4987760e-04, 
   (F)-3.6837365e-03, (F)-3.6082928e-03,  (F)-1.4287889e-04,  (F)-3.0323975e-03, 
   (F)-4.4379771e-03, (F)-4.7605215e-04,  (F)-2.2325424e-03,  (F)-5.0344117e-03, 
   (F)-1.1443298e-03, (F)-1.4099526e-03,  (F)-5.3043884e-03,  (F)-2.0916948e-03, 
   (F)-7.0417569e-04, (F)-5.1954846e-03,  (F)-3.2147444e-03,  (F)-2.4893559e-04, 
   (F)-4.7081688e-03, (F)-4.3749887e-03,  (F)-1.5195163e-04,  (F)-3.9004359e-03, 
   (F)-5.4167544e-03, (F)-4.7614384e-04,  (F)-2.8824934e-03,  (F)-6.1869107e-03, 
   (F)-1.2253861e-03, (F)-1.8070597e-03,  (F)-6.5630700e-03,  (F)-2.3413170e-03, 
   (F)-8.4537442e-04, (F)-6.4672154e-03,  (F)-3.7069390e-03,  (F)-1.7053360e-04, 
   (F)-5.8844936e-03, (F)-5.1548613e-03,  (F) 7.1659157e-05,  (F)-4.8718467e-03, 
   (F)-6.4891423e-03, (F)-2.1317928e-04,  (F)-3.5562787e-03,  (F)-7.5130954e-03, 
   (F)-1.0500512e-03, (F)-2.1205997e-03,  (F)-8.0554503e-03,  (F)-2.3841176e-03, 
   (F)-7.8340158e-04, (F)-7.9990811e-03,  (F)-4.0822857e-03,  (F) 2.2889582e-04, 
   (F)-7.3017179e-03, (F)-5.9428942e-03,  (F) 7.1401279e-04,  (F)-6.0110286e-03, 
   (F)-7.7197987e-03, (F) 5.2510592e-04,  (F)-4.2636499e-03,  (F)-9.1499310e-03, 
   (F)-4.0516251e-04, (F)-2.2793984e-03,  (F)-9.9880979e-03,  (F)-2.0435675e-03, 
   (F)-3.3909452e-04, (F)-1.0043874e-02,  (F)-4.2488535e-03,  (F) 1.2480797e-03, 
   (F)-9.2165441e-03, (F)-6.7817160e-03,  (F) 2.1846610e-03,  (F)-7.5147738e-03, 
   (F)-9.3238581e-03, (F) 2.2239974e-03,  (F)-5.0703559e-03,  (F)-1.1511062e-02,  
   (F)1.2078520e-03,  (F)-2.1353719e-03,  (F)-1.2972233e-02,  (F)-9.0273574e-04, 
   (F)9.3357826e-04,  (F)-1.3372948e-02,  (F)-4.0071332e-03,  (F) 3.7031099e-03, 
   (F)-1.2461197e-02, (F)-7.8598133e-03,  (F) 5.7045982e-03,  (F)-1.0100569e-02,  
   (F)-1.2080651e-02, (F) 6.4711243e-03,  (F)-6.2969332e-03,  (F)-1.6171275e-02,  
   (F)5.5803497e-03,  (F)-1.2083596e-03,  (F)-1.9544795e-02,  (F) 2.6766788e-03, 
   (F)4.8625376e-03,  (F)-2.1544087e-02,  (F)-2.5328532e-03,  (F) 1.1492770e-02, 
   (F)-2.1430688e-02, (F)-1.0346261e-02,  (F) 1.8175979e-02,  (F)-1.8278143e-02,  
   (F)-2.1279927e-02, (F) 2.4371904e-02,  (F)-1.0564866e-02,  (F)-3.6759368e-02,  
   (F)2.9560631e-02,  (F) 5.4891592e-03,  (F)-6.2027321e-02,  (F) 3.3295602e-02, 
   (F)4.6290827e-02,  (F)-1.2975737e-01,  (F) 3.5249028e-02,  (F) 5.2480944e-01,  
   (F)5.2480944e-01,  (F) 3.5249028e-02,  (F)-1.2975737e-01,  (F) 4.6290827e-02,  
   (F)3.3295602e-02,  (F)-6.2027321e-02,  (F) 5.4891592e-03,  (F) 2.9560631e-02, 
   (F)-3.6759368e-02, (F)-1.0564866e-02,  (F) 2.4371904e-02,  (F)-2.1279927e-02,  
   (F)-1.8278143e-02, (F) 1.8175979e-02,  (F)-1.0346261e-02,  (F)-2.1430688e-02,  
   (F)1.1492770e-02,  (F)-2.5328532e-03,  (F)-2.1544087e-02,  (F) 4.8625376e-03, 
   (F)2.6766788e-03,  (F)-1.9544795e-02,  (F)-1.2083596e-03,  (F) 5.5803497e-03, 
   (F)-1.6171275e-02, (F)-6.2969332e-03,  (F) 6.4711243e-03,  (F)-1.2080651e-02,  
   (F)-1.0100569e-02, (F) 5.7045982e-03,  (F)-7.8598133e-03,  (F)-1.2461197e-02,  
   (F)3.7031099e-03,  (F)-4.0071332e-03,  (F)-1.3372948e-02,  (F) 9.3357826e-04,
   (F)-9.0273574e-04, (F)-1.2972233e-02,  (F)-2.1353719e-03,  (F) 1.2078520e-03,  
   (F)-1.1511062e-02, (F)-5.0703559e-03,  (F) 2.2239974e-03,  (F)-9.3238581e-03, 
   (F)-7.5147738e-03, (F) 2.1846610e-03,  (F)-6.7817160e-03,  (F)-9.2165441e-03,  
   (F)1.2480797e-03,  (F)-4.2488535e-03,  (F)-1.0043874e-02,  (F)-3.3909452e-04,
   (F)-2.0435675e-03, (F)-9.9880979e-03,  (F)-2.2793984e-03,  (F)-4.0516251e-04,  
   (F)-9.1499310e-03, (F)-4.2636499e-03,  (F) 5.2510592e-04,  (F)-7.7197987e-03, 
   (F)-6.0110286e-03, (F) 7.1401279e-04,  (F)-5.9428942e-03,  (F)-7.3017179e-03,  
   (F)2.2889582e-04,  (F)-4.0822857e-03,  (F)-7.9990811e-03,  (F)-7.8340158e-04,
   (F)-2.3841176e-03, (F)-8.0554503e-03,  (F)-2.1205997e-03,  (F)-1.0500512e-03,  
   (F)-7.5130954e-03, (F)-3.5562787e-03,  (F)-2.1317928e-04,  (F)-6.4891423e-03, 
   (F)-4.8718467e-03, (F) 7.1659157e-05,  (F)-5.1548613e-03,  (F)-5.8844936e-03, 
   (F)-1.7053360e-04, (F)-3.7069390e-03,  (F)-6.4672154e-03,  (F)-8.4537442e-04, 
   (F)-2.3413170e-03, (F)-6.5630700e-03,  (F)-1.8070597e-03,  (F)-1.2253861e-03, 
   (F)-6.1869107e-03, (F)-2.8824934e-03,  (F)-4.7614384e-04,  (F)-5.4167544e-03, 
   (F)-3.9004359e-03, (F)-1.5195163e-04,  (F)-4.3749887e-03,  (F)-4.7081688e-03, 
   (F)-2.4893559e-04, (F)-3.2147444e-03,  (F)-5.1954846e-03,  (F)-7.0417569e-04, 
   (F)-2.0916948e-03, (F)-5.3043884e-03,  (F)-1.4099526e-03,  (F)-1.1443298e-03, 
   (F)-5.0344117e-03, (F)-2.2325424e-03,  (F)-4.7605215e-04,  (F)-4.4379771e-03, 
   (F)-3.0323975e-03, (F)-1.4287889e-04,  (F)-3.6082928e-03,  (F)-3.6837365e-03, 
   (F)-1.4987760e-04, (F)-2.6646442e-03,  (F)-4.0930890e-03,  (F)-4.5514941e-04, 
   (F)-1.7302268e-03, (F)-4.2043413e-03,  (F)-9.7976498e-04,  (F)-9.2201973e-04, 
   (F)-4.0078194e-03, (F)-1.6168623e-03,  (F)-3.2967385e-04,  (F)-3.5399219e-03, 
   (F)-2.2532994e-03, (F)-5.5816763e-06,  (F)-2.8720059e-03,  (F)-2.7841155e-03,  
   (F)4.0228664e-05,  (F)-2.0976384e-03,  (F)-3.1275771e-03,  (F)-1.6402940e-04,
   (F)-1.3201463e-03, (F)-3.2350341e-03,  (F)-5.5588793e-04,  (F)-6.3307831e-04,  
   (F)-3.0936849e-03, (F)-1.0541415e-03,  (F)-1.1422064e-04,  (F)-2.7257064e-03, 
   (F)-1.5643897e-03, (F) 1.8869166e-04,  (F)-2.1869429e-03,  (F)-1.9993089e-03,  
   (F)2.6244445e-04,  (F)-1.5537480e-03,  (F)-2.2900981e-03,  (F) 1.2766640e-04,
   (F)-9.0788612e-04, (F)-2.3922045e-03,  (F)-1.6840325e-04,  (F)-3.2942885e-04,  
   (F)-2.2933960e-03, (F)-5.6017973e-04,  (F) 1.1906035e-04,  (F)-2.0066034e-03, 
   (F)-9.7289874e-04, (F) 3.9262526e-04,  (F)-1.5747678e-03,  (F)-1.3322860e-03,  
   (F)4.7758328e-04,  (F)-1.0599137e-03,  (F)-1.5802909e-03,  (F) 3.8893919e-04,
   (F)-5.2768325e-04, (F)-1.6781356e-03,  (F) 1.6210356e-04,  (F)-4.4736478e-05,  
   (F)-1.6124011e-03, (F)-1.5028960e-04,  (F) 3.3534474e-04,  (F)-1.3916558e-03, 
   (F)-4.8723154e-04, (F) 5.7331275e-04,  (F)-1.0488444e-03,  (F)-7.8489062e-04,  
   (F)6.5804381e-04,  (F)-6.3483778e-04,  (F)-9.9796908e-04,  (F) 5.9807594e-04,
   (F)-2.0205436e-04, (F)-1.0919296e-03,  (F) 4.2103856e-04,  (F) 1.9525361e-04,  
   (F)-1.0529198e-03, (F) 1.6805066e-04,  (F) 5.1001679e-04,  (F)-8.8526201e-04, 
   (F)-1.0650414e-04, (F) 7.1061989e-04,  (F)-6.1987769e-04,  (F)-3.5641282e-04,  
   (F)7.8742167e-04,  (F)-2.9161075e-04,  (F)-5.4079543e-04,  (F) 7.4482882e-04, 
   (F)5.5801711e-05,  (F)-6.3116397e-04,  (F) 6.0138846e-04,  (F) 3.7506374e-04, 
   (F)-6.1185209e-04, (F) 3.9780715e-04,  (F) 6.2979774e-04,  (F)-4.9268552e-04,   
   (F)1.7050387e-04,  (F) 7.9570539e-04,  (F)-2.9124518e-04,  (F)-4.2920075e-05, 
   (F)8.5952215e-04,  (F)-3.3846228e-05,  (F)-2.0239452e-04,  (F) 8.2442752e-04,  
   (F)2.3721823e-04,  (F)-2.8792879e-04,  (F) 7.1065718e-04,  (F) 4.8968352e-04, 
   (F)-2.9129914e-04, (F) 5.3987170e-04,  (F) 6.9373941e-04,  (F)-2.0786328e-04,   
   (F)3.4868918e-04,  (F) 8.2351175e-04,  (F)-5.8804298e-05,  (F) 1.7022328e-04, 
   (F)8.7589585e-04,  (F) 1.3440062e-04,  (F) 2.5781549e-05,  (F) 8.4572319e-04,  
   (F)3.4489013e-04,  (F)-5.6697379e-05,  (F) 7.5079435e-04,  (F) 5.3898956e-04, 
   (F)-7.6935612e-05, (F) 6.1047682e-04,  (F) 7.0140838e-04,  (F)-2.6877102e-05,   
   (F)4.4799518e-04,  (F) 8.0375966e-04,  (F) 7.4805292e-05,  (F) 2.9158385e-04, 
   (F)8.4946533e-04,  (F) 2.1942502e-04,  (F) 1.5900697e-04,  (F) 8.2555112e-04,  
   (F)3.7640856e-04,  (F) 6.7577648e-05,  (F) 7.5149712e-04,  (F) 5.3765695e-04,  
   (F)3.0359128e-05,  (F) 6.3253750e-04,  (F) 6.7170137e-04,  (F) 4.0287664e-05,  
   (F)4.8260334e-04,  (F) 7.7031646e-04,  (F) 1.0633143e-04,  (F) 3.2723682e-04,  
   (F)8.2492171e-04,  (F) 2.2038581e-04,  (F) 1.7834353e-04,  (F) 8.1936773e-04,  
   (F)3.6872795e-04,  (F) 5.2826418e-05,  (F) 7.4613466e-04,  (F) 5.3852836e-04, 
   (F)-2.5249950e-05, (F) 6.0362256e-04,  (F) 7.0953770e-04,  (F)-2.9073419e-05,   
   (F)3.9680262e-04,  (F) 8.4747506e-04,  (F) 6.6667703e-05,  (F) 1.4676282e-04, 
   (F)9.0170081e-04,  (F) 2.7649832e-04,  (F)-9.5331602e-05,  (F) 8.1158061e-04,  
   (F)5.8300451e-04,  (F)-2.4009910e-04,  (F) 5.2925119e-04,  (F) 9.0703407e-04, 
   (F)-1.6575014e-04, (F) 6.7905656e-05,  (F) 1.0771052e-03,  (F) 2.3758270e-04,  
   (F)-4.2019012e-04, (F) 8.3261061e-04,  (F) 9.4085515e-04,  (F)-5.3197924e-04, 
   (F)-7.1432006e-05, (F) 1.4837936e-03,  (F) 4.8035495e-04,  (F)-1.3358539e-03,  
   (F)6.3445872e-05,  (F) 3.2368711e-03,  (F) 3.9722215e-03,  (F)2.0805499e-03  
  };

  *lenh0 = BP5K_16K_LEN;           /* store 'number of coefficients' */
  *h0 = bp5k_16khz_coeff;		/* store pointer to []-array */
}
#undef BP5K_16K_LEN
/* ...................... End of fill_bp5k_16khz() ..................... */


/*
  ============================================================================

        SCD_FIR *bp5k_16khz_init (void);
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        Description:
        ~~~~~~~~~~~~

        Initialization routine for a band-pass filter which limits the 
	bandwidth of the input signal from 50 Hz to 5000 Hz (3 dB points).
        No rate change is implemented.

        Parameters:  none.
        ~~~~~~~~~~~

        Return value:
        ~~~~~~~~~~~~~
        Returns a pointer to struct SCD_FIR;

        Author: <simao@ctd.comsat.com>
        ~~~~~~~

        History:
        ~~~~~~~~
        13.May.97 v1.0 Release of 1st version <simao@ctd.comsat.com>

 ============================================================================
*/
SCD_FIR *bp5k_16khz_init ()
{
  float          *h0;		/* pointer to array with FIR coeff. */
  long            lenh0;	/* number of FIR coefficients */


  /* allocate array for FIR coeff. and fill with coefficients */
  fill_bp5k_16khz(&h0, &lenh0);

  return
    fir_initialization(		/* Returns: pointer to SCD_FIR-struct */
		       lenh0,	/* In: number of FIR-coefficients */
		       h0,	/* In: pointer to array with FIR-cof. */
		       1.0,	/* In: gain factor for FIR-coeffic. */
		       1l,	/* In: Down-sampling factor */
		       'D'	/* In: switch to down-sampling proc. */
    );				/* (works here as simple FIR-fil. */
}
/* .................... End of bp5k_16khz_init() .................... */


/* ************************** END OF FIR-WB.C *************************** */
