////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    io_shell.c: input, output, and memory related algorithms            //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include "gsl/gsl_errno.h"
#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"
#include "io_shell.h"


const unsigned char wav_head_block[40] =
{
 0x52, 0x49, 0x46, 0x46, 0xF4, 0x2F, 0x02, 0x00,
 0x57, 0x41, 0x56, 0x45, 0x66, 0x6D, 0x74, 0x20,
 0x10, 0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x00,
 0x80, 0x3E, 0x00, 0x00, 0x00, 0x7D, 0x00, 0x00,
 0x02, 0x00, 0x10, 0x00, 0x64, 0x61, 0x74, 0x61
};

typedef int8_t _int8;

/*
short flip_short(short sin);
long flip_long(long lin);
float flip_float(float fin);
void flip(_int8 *data,int len);
*/

// 
char msg[SPA_MAX_STR_LEN];

int spa_io_saybye(char *msg)
{
	//fprintf(stderr,"\n *** Error around LINE %d at FILE \"%s\" !\n\n     Error Message: %s\n\n",__LINE__,__FILE__,msg);
	printf("Error Message: %s\n\n", msg);
	GSL_ERROR (msg, -1);
}
void spa_io_lookout(char *msg)
{
	printf("Warining  %s !\n", msg);
}


// swap byte order
void spa_io_swap16(short *s)
{
	*s = ((*s & 0x0000ffL) << 8) | ((*s & 0x00ff00L) >> 8);
}

void spa_io_swap32(int *l)
{
	*l = ((*l & 0x000000ffL) << 24) | \
		((*l & 0x0000ff00L) << 8) | \
		((*l & 0x00ff0000L) >> 8) | \
		((*l & 0xff000000L) >> 24);
}

/*
void spa_io_swap64(double *l)
{
	*l = (*l>>56) | 
		((*l<<40) & 0x00FF000000000000) |
		((*l<<24) & 0x0000FF0000000000) |
		((*l<<8)  & 0x000000FF00000000) |
		((*l>>8)  & 0x00000000FF000000) |
		((*l>>24) & 0x0000000000FF0000) |
		((*l>>40) & 0x000000000000FF00) |
		(*l<<56);
}
*/

FILE *spa_io_fopen(char *fn, char *tag)
{
	FILE *fp;	
	if ((fp = fopen(fn, tag)) == NULL) {
		printf("Can not open %s for %s purpose.\n", fn, tag);
		exit(GSL_FAILURE);
	}	
	return fp;
}

void spa_io_fclose(FILE *fp)
{
	fclose(fp);
}

// Get Next Tag from 'fp', tag stored in 'ctag'
// Return Value:
//      0 : Unresolved TAG; 1 : Success; -1 : End of file
int spa_io_get_next_tag(FILE *fp, char *ctag)
{
	int i;
	char ch;
	while (!feof(fp)) {
		ch = getc(fp);
		if (ch != '<')
			continue;
		// ch = '<'
		i = 0;
		while ((ctag[i] = getc(fp)) != '>' && i < SPA_MAX_TAG_LEN)
			i++;
		if (i >= SPA_MAX_TAG_LEN)
			return 0;
		ctag[i] = 0;
		return 1;
	}
	return -1;
}

// Read and write the HTK feature
int spa_io_read_htk_header(char *fn, int swap, spa_htk_header **ph)
{
	spa_htk_header *header = (spa_htk_header *) calloc(sizeof(spa_htk_header), 1);
	
	FILE *fp_in = spa_io_fopen(fn, "rb");
	if (!fread(header, sizeof(spa_htk_header), 1, fp_in))
		return GSL_EFAULT;
	if (swap) {
		spa_io_swap32(&(header->nSamples));
		spa_io_swap32(&(header->sampPeriod));
		spa_io_swap16(&(header->sampSize));
		spa_io_swap16(&(header->sampKind));
	}
	if (header->nSamples < 0 || header->sampPeriod < 0 ||
		   header->sampPeriod > 100000 || header->sampSize < 0)
		// || !ParseHTKParmKind(ph->sampKind, str))
		return GSL_ESANITY;
	
	*ph = header;
	fclose(fp_in);
	return GSL_SUCCESS;
}

// read htk feature into gsl_matrix
int spa_io_read_htk_mat(char *fn, int swap, gsl_matrix **pm)
{
	char str[20];
	int i, j, dim;
	float f;
	spa_htk_header *header;
	gsl_matrix *m;
	header = (spa_htk_header *) calloc(sizeof(spa_htk_header), 1);
	
	FILE *fp_in = spa_io_fopen(fn, "rb");
	if (!fread(header, sizeof(spa_htk_header), 1, fp_in))
		return GSL_EFAULT;
	if (swap) {
		spa_io_swap32(&(header->nSamples));
		spa_io_swap32(&(header->sampPeriod));
		spa_io_swap16(&(header->sampSize));
		spa_io_swap16(&(header->sampKind));
	}
	if (header->nSamples < 0 || header->sampPeriod < 0 ||
		   header->sampPeriod > 100000 || header->sampSize < 0)
		   // || !ParseHTKParmKind(ph->sampKind, str))
		return GSL_ESANITY;
	
	// alloc matrix for storing feature and read in data
	dim = header->sampSize / sizeof(float);
	m = gsl_matrix_calloc(header->nSamples, dim);
	for (i = 0; i < m->size1; i++) {
		for (j = 0; j < m->size2; j++) {
			if (!fread(&f, sizeof(float), 1, fp_in))
				return GSL_EFAULT;
			if (swap) {
				//spa_io_swap32(&f));
			}
			gsl_matrix_set(m, i, j, (double)f);
		}
	}
	spa_io_fclose(fp_in);
	*pm = m;
	free(header);
	return GSL_SUCCESS;
}

// read buffer
// a matrix with row:frm, column:dim is returned
// if *pm is not null, release *pm.
int spa_io_read_htk_buf(char *fn, int swap, double ***pm, int *frm, int *dim)
{
	char str[20];
	int i, j;
	float f;
	spa_htk_header *header;
	double **m;
	header = (spa_htk_header *) calloc(1, sizeof(spa_htk_header));
	
	FILE *fp_in = spa_io_fopen(fn, "rb");
	if (!fread(header, sizeof(spa_htk_header), 1, fp_in))
		return GSL_EFAULT;
	if (swap) {
		spa_io_swap32(&(header->nSamples));
		spa_io_swap32(&(header->sampPeriod));
		spa_io_swap16(&(header->sampSize));
		spa_io_swap16(&(header->sampKind));
	}
	if (header->nSamples < 0 || header->sampPeriod < 0 ||
		   header->sampPeriod > 100000 || header->sampSize < 0)
		// || !ParseHTKParmKind(ph->sampKind, str))
		return GSL_ESANITY;
	
	// alloc matrix for storing feature and read in data
	*frm = header->nSamples;
	*dim = header->sampSize / sizeof(float);
	m = (double **) alloc2d(*frm, *dim, sizeof(double));
	for (i = 0; i < *frm; i++) {
		for (j = 0; j < *dim; j++) {
			if (!fread(&f, sizeof(float), 1, fp_in))
				return GSL_EFAULT;
			if (swap) {
				//spa_io_swap32(&f));
			}
			m[i][j] = (double) f;
		}
	}
	spa_io_fclose(fp_in);
	*pm = m;
	return GSL_SUCCESS;
}


int spa_io_read_htk_feamat(char *fn, int swap, spa_htk_header **ph, gsl_matrix **pm)
{
	spa_io_read_htk_header(fn, swap, ph);
	spa_io_read_htk_mat(fn, swap, pm);
}

int spa_io_read_htk_feabuf(char *fn, int swap, spa_htk_header **ph, double ***pm, int *frm, int *dim)
{
	spa_io_read_htk_header(fn, swap, ph);
	spa_io_read_htk_buf(fn, swap, pm, frm, dim);
}

// free header, matrix, and buffer
void spa_io_release_htk_header(spa_htk_header *h)
{
	free(h);
}

void spa_io_release_htk_mat(gsl_matrix *m)
{
	gsl_matrix_free(m);
}

void spa_io_release_htk_buf(double **m)
{
	free2d((char **) m);
}

void spa_io_release_htk_feamat(spa_htk_header *h, gsl_matrix *m)
{
	spa_io_release_htk_header(h);
	spa_io_release_htk_mat(m);
}

void spa_io_release_htk_feabuf(spa_htk_header *h, double **m)
{
	spa_io_release_htk_header(h);
	spa_io_release_htk_buf(m);
}

// write down htk feature
int spa_io_write_htk_header(FILE *fp, spa_htk_header *header)
{
	fwrite(header, sizeof(spa_htk_header), 1, fp);
}

// a frame as a gsl vector
int spa_io_write_htk_vec(FILE *fp, gsl_vector *v)
{
	int j;
	float f;
	for (j = 0; j < v->size; j++) {
		f = (float) gsl_vector_get(v, j);
		fwrite(&f, sizeof(float), 1, fp);
	}
}

// a frame as an array
int spa_io_write_htk_arr(FILE *fp, double *v, int s)
{
	int j;
	float f;
	for (j = 0; j < s; j++) {
		f = (float) v[j];
		fwrite(&f, sizeof(float), 1, fp);
	}
}

// feature array
int spa_io_write_htk_feabuf(char *fn, spa_htk_header *header, double **m, int irow, int icol)
{
	int i, row, col;
	float f;
	gsl_vector_view gv;
	row = header->nSamples;
	col = header->sampSize / sizeof(float);
	if (row != irow || col != icol) {
		sprintf(msg, "spa_io_write_htk_arr: check the row and col size of m.\n");
		GSL_ERROR(msg, GSL_FAILURE);
	}
	
	FILE *fp_out = spa_io_fopen(fn, "wb");
	spa_io_write_htk_header(fp_out, header);
	
	for (i = 0; i < row; i++) {
		spa_io_write_htk_arr(fp_out, m[i], col);
	}
	/*
	if (flag_free) {
		spa_io_release_htk_header(header);
		spa_io_release_htk_buf(m);
	}
	*/
	spa_io_fclose(fp_out);
}

// feature matrix
int spa_io_write_htk_feamat(char *fn, spa_htk_header *header, gsl_matrix *m)
{
	int i, j;
	float f;
	gsl_vector_view gv;
	FILE *fp_out = spa_io_fopen(fn, "wb");
	spa_io_write_htk_header(fp_out, header);
	for (i = 0; i < m->size1; i++) {
		gv = gsl_matrix_row(m, i);
		spa_io_write_htk_vec(fp_out, &gv.vector);
	}
	/*
	if (flag_free) {
		spa_io_release_htk_header(header);
		spa_io_release_htk_mat(m);
	}
	*/
	spa_io_fclose(fp_out);
}

// Read and write wave file
// return sampling rate
int spa_io_read_wav(char *fn, gsl_vector **pw, int cfg_smp_rate)
{
	spa_wav_form u_WaveForm;
	spa_wav_format_chunk u_FormatChunk;
	short BlockAlign, sv;
	spa_wav_fact_chunk u_FactChunk;
	spa_wav_data_chunk u_DataChunk;
	FILE * fp;
	gsl_vector *w;
	int i;

	fp = spa_io_fopen(fn, "rb");
	if (fp == NULL) { printf("Error: Not able open WAV file. %s\n", fn); exit(-3001); }

	fread(&u_WaveForm, sizeof(spa_wav_form), 1, fp);		// Wave-Form followed
	if ((u_WaveForm.IDRiff != 0x46464952) || (u_WaveForm.IDWave != 0x45564157))
	{ printf("Error: Bad format WAV file. %s\n", fn); exit(-3002); }

	fread(&u_FormatChunk, sizeof(spa_wav_format_chunk), 1, fp);	// Format-Chunk followed
	if ((u_FormatChunk.IDfmt != 0x20746D66) || (u_FormatChunk.PCM != 0x0001) || (u_FormatChunk.ChannelNum != 1) || (u_FormatChunk.BytePerSample != 2) || (u_FormatChunk.BitPerSample != 16)  || (u_FormatChunk.TransRate != u_FormatChunk.SampleRate * 2)) {
		printf("Error: Bad format WAV file. %s\n", fn); exit(-3002);
	}

	if (u_FormatChunk.ckLen == 0x12) {
		fread(&BlockAlign, sizeof(short), 1, fp);
		if (BlockAlign != 0) { printf("Error: Bad format WAV file. %s\n", fn); exit(-3002); }
	}
	fread(&u_DataChunk.IDdata, sizeof(int), 1, fp);
	fseek(fp, -sizeof(int), SEEK_CUR);

	if (u_DataChunk.IDdata == 0x74636166)	//	Fact-Chunk followed
	{
		fread(&u_FactChunk, sizeof(spa_wav_fact_chunk), 1, fp);
		if (u_FactChunk.ckLen != 4) { printf("Error: Bad format WAV file. %s\n", fn); exit(-3002); }

		fread(&u_DataChunk, sizeof(spa_wav_data_chunk), 1, fp);	//	Data-Chunk followed
		if (u_DataChunk.IDdata != 0x61746164) { printf("Error: Bad format WAV file. %s\n", fn); exit(-3002); }
		if (u_DataChunk.ckLen != u_FactChunk.SampleNum * sizeof(short)) { printf("Error: Bad format WAV file. %s\n", fn); exit(-3002); }
	}
	else	//	Data-Chunk followed
	{
		fread(&u_DataChunk, sizeof(spa_wav_data_chunk), 1, fp);
		if (u_DataChunk.IDdata != 0x61746164) { printf("Error: Bad format WAV file. %s\n", fn); exit(-3002); }
	}

	// read in wave buf
	w = gsl_vector_calloc(u_DataChunk.ckLen/sizeof(short));
	for (i = 0; i < w->size; i++) {
		fread(&sv, sizeof(short), 1, fp);
		gsl_vector_set(w, i, (double)sv);
	}
	*pw = w;
	spa_io_fclose(fp);
	
	if (u_FormatChunk.SampleRate != cfg_smp_rate) {
		sprintf(msg, "The samplig rate of the file: %s - %d does not match the configured sampling rate %d",
			fn, u_FormatChunk.SampleRate, cfg_smp_rate );
		spa_io_saybye(msg);
	}
	return u_FormatChunk.SampleRate;
}

void spa_io_write_wav(char *fn, gsl_vector *w, int smp_rate, int free_flag)
{
	FILE *fp;
	spa_wav_head wh;
	int i, len_in_byte;
	short s;
	len_in_byte = w->size * sizeof(short);
	fp = spa_io_fopen(fn, "wb");

	fwrite(wav_head_block, sizeof(char), 40, fp);
	fwrite(&len_in_byte, sizeof(int), 1, fp);

	rewind(fp);
	fread(&wav_head_block, sizeof(spa_wav_head), 1, fp);
	wh.nBytes	= len_in_byte;
	wh.FileLength	= len_in_byte + 36;
	wh.SampleRate	= smp_rate;
	wh.TransRate	= smp_rate * 2;
	wh.Adjust	= 2;
	rewind(fp);
	fread(&wav_head_block, sizeof(spa_wav_head), 1, fp);
	
	for (i = 0; i < w->size; i++) {
		s = (short) gsl_vector_get(w, i);
		fwrite(&s, sizeof(short), 1, fp);
	}
	
	if (free_flag)
		gsl_vector_free(w);
	
	fclose(fp);
}

// lines in a text file
int spa_io_count_lines_of_textfile(char *file)
{
	FILE *fp;
	char line[MAX_CHAR_NUM_IN_A_LINE];
	double d;
	int count;
	if ((fp = fopen(file, "rt")) == NULL) {
		printf("Cannot open F0 reference file %s.\n", file);
		return -1;
	};
	
	// how many lines
	count = 0;
	while(fgets(line, MAX_CHAR_NUM_IN_A_LINE, fp) != NULL) {
		count++;
	}
	fclose(fp);
	
	return count;
}

char          **alloc2d ( int dim1,	// "x" dimension
                          int dim2,	// "y" dimension
                          int size	// number of bytes each entry takes
                        )
{
	int             i;		// loop control variable
	unsigned        nelem;		// total number of elements

	char	       *p,		// pointer to matrix memory
	**pp;		// pointer to matrix mem table

	nelem = ( unsigned ) dim1 * dim2;

	p = ( char * ) calloc ( nelem, ( unsigned ) size );
	if ( p == NULL ) return ( NULL );

	pp = ( char ** ) calloc ( ( unsigned ) dim1, ( unsigned ) sizeof ( char * ) );
	if ( pp == NULL )
	{
		free ( p );
		return ( NULL );
	}

	for ( i = 0; i < dim1; i++ )
		pp[i] = p + ( i * dim2 * size );

	return ( pp );
}

void free2d(char **p)
{
	if (p != NULL) {
		free(p[0]);
		free(p);
	}
	return;
}


char      ***alloc3d ( int       dim1,       // "x" dimension
                       int 	dim2,        // "y" dimension
                       int 	dim3,        // "z" dimension
                       int 	size         // number of bytes per entry
                     )
{
	int     j,i;     	// loop control variables

	char    *p;
	char    **pp;
	char    ***ppp;        // pointer to matrix mem table

	// Allocate for all the elements
	if ( ( p = ( char * ) calloc ( dim1*dim2*dim3,size ) ) == NULL )
		return ( NULL );

	// Allocate for first level pointers
	if ( ( pp = ( char ** ) calloc ( dim1*dim2,sizeof ( char * ) ) ) == NULL )
	{
		free ( p );
		return ( NULL );
	}

	// Allocate for the 3-D level pointers
	if ( ( ppp = ( char *** ) calloc ( dim1,sizeof ( char ** ) ) ) == NULL )
	{
		free ( pp ); free ( p );
		return ( NULL );
	}

	// Fill the table with locations to where each row begins
	for ( i = 0, j = 0; i < dim1*dim2; i++, j += dim3 )
	{
		pp[i] = p + j*size;
	}
	for ( i = 0, j = 0; i < dim1; i++, j += dim2 )
	{
		ppp[i] = pp + j;
	}

	return ( ppp );
}

void  free3d(char ***X)
{
    	// The alloc3d routine simplies the freeing to the following:
	int i, j, k;
	if (X != NULL) {
		free((char *) X[0][0]);
		free((char **) X[0]);
		free(X);
	}
}

char      ***alloc4d ( int      dim1,       // "x" dimension
		       int 	dim2,        // "y" dimension
	 	       int 	dim3,        // "z" dimension
	  	       int	dim4,	     // "w" dimension
  			int 	size         // number of bytes per entry
		     )
{
	int     j,i;     	// loop control variables

	char    *p;
	char    **pp;
	char    **ppp;
	char    ****pppp;        // pointer to matrix mem table

	// Allocate for all the elements
	if ( ( p = ( char * ) calloc ( dim1*dim2*dim3*dim4,size ) ) == NULL )
		return ( NULL );

	// Allocate for first level pointers
	if ( ( pp = ( char ** ) calloc ( dim1*dim2*dim3, sizeof ( char * ) ) ) == NULL )
	{
		free ( p );
		return ( NULL );
	}

	// Allocate for the 3-D level pointers
	if ( ( ppp = ( char *** ) calloc ( dim1*dim2, sizeof ( char ** ) ) ) == NULL )
	{
		free ( pp ); free ( p );
		return ( NULL );
	}
	
	// Allocate for the 4-D level pointers
	if ( ( pppp = ( char **** ) calloc ( dim1, sizeof ( char *** ) ) ) == NULL )
	{
		free(ppp); free ( pp ); free ( p );
		return ( NULL );
	}

	// Fill the table with locations to where each row begins
	for ( i = 0, j = 0; i < dim1*dim2*dim3; i++, j += dim4 )
	{
		pp[i] = p + j*size;
	}
	for ( i = 0, j = 0; i < dim1*dim2; i++, j += dim3 )
	{
		ppp[i] = pp + j;
	}
	for ( i = 0, j = 0; i < dim1; i++, j += dim2 )
	{
		pppp[i] = ppp + j;
	}

	return ( pppp );
}


void  free4d(char ****X)
{
    	// The alloc4d routine simplies the freeing to the following:
	if (X != NULL) {
		free((char *) X[0][0][0]);
		free((char **) X[0][0]);
		free((char ***) X[0]);
		free(X);
	}
}

// flip an array 000 -> 001, 011 -> 100
// s is the size of ipFlag
int spa_io_flip_flag(int *ipFlag, int s)
{
	int j = 0;
	if (ipFlag[0] == 0) {
		ipFlag[0] = 1;
		return 1;
	}
	else if (s > 1) {
		ipFlag[0] = 0;
		if (spa_io_flip_flag(&ipFlag[1], s-1) == 0)
			return 0;
		else
			return 1;
	}
	else {
		return 0;
	}
}
				
/*
long flip_long(long lin)
{
	long lout;
	_int8 *i8in,*i8out,ii;

	i8in = (_int8*)&lin;
	i8out = (_int8*)&lout;

	for (ii=0;ii<sizeof(long);ii++)
		i8out[ii] = i8in[sizeof(long)-1-ii];


	return lout;
}

void flip(_int8 *data,int len)
{
	_int8 temp;

	int ii;
	for (ii=0;ii<len/2;ii++)
{
		temp = data[len-1-ii];
		data[len-1-ii] = data[ii];
		data[ii] = temp;
}
}

float flip_float(float fin)
{
	float fout;
	_int8 *i8in,*i8out,ii;

	i8in = (_int8*)&fin;
	i8out = (_int8*)&fout;

	for (ii=0;ii<sizeof(float);ii++)
		i8out[ii] = i8in[sizeof(float)-1-ii];

	return fout;
}

short flip_short(short sin)
{
	short sout;
	_int8 *i8in,*i8out,ii;

	i8in = (_int8*)&sin;
	i8out = (_int8*)&sout;

	for (ii=0;ii<sizeof(short);ii++)
		i8out[ii] = i8in[sizeof(short)-1-ii];

	return sout;
}
*/

void ShowVec(double *v, int s)
{
	int i;
	for (i = 0; i < s; i++)
		printf("%.1e\t", v[i]);
	printf("\n");
}

void ShowMat(double **m, int r, int l)
{
	int i, j;
	for (i = 0; i < r; i++) {
		for (j = 0; j < l; j++)
			printf("%.1e\t", m[i][j]);
		printf("\n");
	}
	printf("\n");
}

// transform c-style mat to fortran-style vec
// return the pt to the vec
double * C2FMat(double **c, int row, int col)
{
	int i, j;
	double *f = (double *) calloc(row*col, sizeof(double));
	for (j = 0; j < col; j++)
		for (i = 0; i < row; i++)		
			f[j*row+i] = c[i][j];
	return f;
}

void spa_io_init_int_vec(int *v, int s, int a)
{
	int i;
	for (i = 0; i < s; i++)
		v[i] = a;
}

void spa_io_init_vec(double *v, int s, double d)
{
	int i;
	for (i = 0; i < s; i++)
		v[i] = d;
}

void spa_io_init_mat(double **m, int r, int l, double d)
{
	int i, j;
	for (i = 0; i < r; i++)
		for (j = 0; j < l; j++)
			m[i][j] = d;
}


void spa_io_zero_vec(double *v, int s)
{
	int i;
	for (i = 0; i < s; i++)
		v[i] = 0.0;
}

void spa_io_copy_int_vec(int *dst, int *src, int s)
{
	int i;
	for (i = 0; i < s; i++)
		dst[i] = src[i];
}

void spa_io_copy_vec(double *dst, double *src, int s)
{
	int i;
	for (i = 0; i < s; i++)
		dst[i] = src[i];
}

void spa_io_zero_mat(double **m, int r, int l)
{
	int i, j;
	for (i = 0; i < r; i++)
		for (j = 0; j < l; j++)
			m[i][j] = 0.0;
}

void spa_io_copy_mat(double **dst, const double **src, int r, int l)
{
	int i, j;
	for (i = 0; i < r; i++)
		for (j = 0; j < l; j++)
			dst[i][j] = src[i][j];
}


void spa_io_zero_cub(double ***m, int x, int y, int z)
{
	int i, j, k;
	for (i = 0; i < x; i++)
		for (j = 0; j < y; j++)
			for (k = 0; k < z; k++)
				m[i][j][k] = 0.0;
}

void spa_io_copy_cub(double ***dst, double ***src, int x, int y, int z)
{
	int i, j, k;
	for (i = 0; i < x; i++)
		for (j = 0; j < y; j++)
			for (k = 0; k < z; k++)
				dst[i][j][k] = src[i][j][k];
}

void spa_io_zero_qua(double ****m, int x, int y, int z, int w)
{
	int i, j, k, l;
	for (i = 0; i < x; i++)
		for (j = 0; j < y; j++)
			for (k = 0; k < z; k++)
				for (l = 0; l < w; l++)
					m[i][j][k][l] = 0.0;
}

void spa_io_copy_qua(double ****dst, double ****src, int x, int y, int z, int w)
{
	int i, j, k, l;
	for (i = 0; i < x; i++)
		for (j = 0; j < y; j++)
			for (k = 0; k < z; k++)
				for (l = 0; l < w; l++)
					dst[i][j][k][l] = src[i][j][k][l];
}

void spa_io_zero_int_vec(int *v, int s)
{
	int i;
	for (i = 0; i < s; i++)
		v[i] = 0;
}

void spa_io_zero_int_mat(int **m, int r, int l)
{
	int i, j;
	for (i = 0; i < r; i++)
		spa_io_zero_int_vec(m[i], l);
}

void spa_io_zero_int_cub(int ***m, int x, int y, int z)
{
	int i, j, k;
	for (i = 0; i < x; i++)
		spa_io_zero_int_mat(m[i], y, z);
}

void spa_io_zero_int_qua(int ****m, int x, int y, int z, int w)
{
	int i, j, k, l;
	for (i = 0; i < x; i++)
		spa_io_zero_int_cub(m[i], y, z, w);
}

// get the suffix of xxx : ie src.abc, the suffix is abc
void spa_io_get_suffix (char *src, char *suffix)
{
	char *pc;
	pc = strrchr(src, '.');
	pc++;
	strcpy(suffix, pc);
}


// replace the suffix of xxx : ie src.abc -> src.suffix
void spa_io_replace_suffix (char *dst, char *src, char *suffix)
{
	char *pc;
	strcpy(dst, src);
	pc = strrchr(dst, '.');
	pc++;
	if (strcmp(pc, suffix) == 0)
		spa_io_lookout("The destination file is the same as the source file.\n");
	strcpy(pc, suffix);
}

// insert a string between the name and the suffix of xxx : ie src.abc -> src.insert.abc
void spa_io_insert_suffix (char *dst, char *src, char *insert)
{
	char suffix[SPA_MAX_TAG_LEN];
	char *pc;
	spa_io_get_suffix (src, suffix);
	strcpy(dst, src);
	pc = strrchr(dst, '.');
	pc++;
	strcpy(pc, insert);
	strcat(pc, ".");
	strcat(pc, suffix);
}

// write the gsl_matrix in controlled format
void spa_io_fwrite_gsl_matrix(FILE *fp, gsl_matrix *m)
{
	int i, j;
	double d;
	fwrite(&m->size1, sizeof(int), 1, fp);
	fwrite(&m->size2, sizeof(int), 1, fp);
	for (i = 0; i < m->size1; i++) {
		for (j = 0; j < m->size2; j++) {
			d = gsl_matrix_get(m, i, j);
			fwrite(&d, sizeof(double), 1, fp);
		}
	}
}

int spa_io_fsave_gsl_matrix(char *fn, gsl_matrix *m)
{
	FILE *fp = spa_io_fopen(fn, "wb");
	spa_io_fwrite_gsl_matrix(fp, m);
	spa_io_fclose(fp);
	return GSL_SUCCESS;
}

// fread the gsl_matrix in controlled format
void spa_io_fread_gsl_matrix(FILE *fp, gsl_matrix **pm)
{
	int i, j, row, col;
	double d;
	gsl_matrix *m;
	
	fread(&row, sizeof(int), 1, fp);
	fread(&col, sizeof(int), 1, fp);
	m = gsl_matrix_calloc(row, col);
	for (i = 0; i < m->size1; i++) {
		for (j = 0; j < m->size2; j++) {
			fread(&d, sizeof(double), 1, fp);
			gsl_matrix_set(m, i, j, d);
		}
	}
	
	*pm = m;
}

int spa_io_fload_gsl_matrix(char *fn, gsl_matrix **pm)
{
	FILE *fp = spa_io_fopen(fn, "rb");
	spa_io_fread_gsl_matrix(fp, pm);
	spa_io_fclose(fp);
	return GSL_SUCCESS;
}

void spa_io_fwrite_gsl_vector(FILE *fp, gsl_vector *v)
{
	int i;
	double d;
	fwrite(&v->size, sizeof(int), 1, fp);
	for (i = 0; i < v->size; i++) {
		d = gsl_vector_get(v, i);
		fwrite(&d, sizeof(double), 1, fp);
	}
}