////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    pitch.h:                                                            //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#ifndef _SPA_PITCH_H
#define _SPA_PITCH_H

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "io_shell.h"
#include "spec_dom.h"
#include "pconf.h"

#define SPA_PITCH_ACCU_STATS (1)
#define SPA_PITCH_TRAIN_MODEL (2)

#define SPA_PITCH_STAGE_CAND (1)
#define SPA_PITCH_STAGE_DP (2)

#define SPA_PITCH_LESS_NOISY (0)
#define SPA_PITCH_MORE_NOISY (1)

#define SPA_PITCH_USE_NON_PROMINENT_PEAK (0)
#define SPA_PITCH_USE_PROMINENT_PEAK (1)
#define SPA_PITCH_USE_ALL_PEAK (2)

#define SPA_PITCH_DP_MATCH_SCALE_FACTOR (0.025)

// residual infomation of the peak give the F0 
typedef struct _spa_resi_info {
	int multi;		// freq = F0 * (multi + resi) 
	double resi;
	double abs_snr;		// absolute SNR 
} spa_resi_info;

// storing the info of residuals clustered together 
typedef struct _spa_clus_resi_info {
	spa_resi_info *resi_vec;
	int resi_num;			// number of residuals 
	double b;			// the 'b' (variance) of laplacian distribution 
	double mu;			// the mean of laplacian distribution 
	double p_snr;			// the P(snr | band) 
	gsl_vector *pdf_table;		// table of laplacian distribution 
} spa_clus_resi_info;

// pitch parameters
typedef struct {
	int status;		// 1 - learning the pmf of pitch model, 2 - calculate the F0 candidates, 3 - output F0 contour using dynamic programing 
	int cand_num;		// num of possible F0 values in an analysis frame (351) 
	int less_cand_num;	// reduced num of possible F0 values in an analysis frame (20) 
	int freq_resi_num;	// the residual frequency of harmonic frequency divided by F0 
	int diff_snr_num;	// num of levels for normalized SNR difference values at peaks 
	int harm_num;		// num of possible harmonics in an analysis frame (4097) 
	int freq_band_num;	// num of frequency bands from 0 to f_cut 
	double* cands;		// cand[i] = freq_resolution x (i - k_min) 
	double* harms;		// harm[i] = freq_resolution x i 
	int spec_smth_len;	// smoothing window length in spectrum smoothing () 
	int abs_snr_num;	// num of SNR levels in dB (5: <0, 0-5, 5-10, 10-20, >20) 
	int peak_flag;		// prominent peaks: 1, all peaks: 0 
	int contour_num;	// number of stored F0 countours in dynamic programing
	double frm_len_sec;	// frame length in seconds 
	double hz_per_smp;	// Hz between two samples 
	double *abs_snr_upbd;	// upper bound of an absolute snr level 
	double *abs_snr_lobd;	// lower bound of an absolute snr level 
	double *diff_snr_upbd;	// upper bound of a difference snr level 
	double *diff_snr_lobd;	// lower bound of a difference snr level 
	double *freq_band_upbd;	// upper bound of a frequency band 
	double *freq_band_lobd;	// lower bound of a frequency band 
	double f_min;		// min F0 in Hz (50) 
	double f_max;		// max F0 in Hz (400) 
	double f_cut;		// estimate the F0 on the range from 0 to f_cut Hz (4000) 
	double lr_sca;		// a in weighting functin 1.0 / (1.0 + a * exp-bx)
	double lr_exp;		// b in weighting functin 1.0 / (1.0 + a * exp-bx)
	double k_min;		// min F0 in bin 
	double k_max;		// max F0 in bin 
	double k_cut;		// estimate the F0 on the range from 0 to k_cut bin 
	double octave_cost;	// cost for octave F0 changes 
	double dp_ratio;	// the scaling factor of calucating the costing function*/
	double resi_space;	// the space between harmonics
	spa_pdf_param ***inter_trans_param;	// parameters of F0 inter transitions (has f_band_num x f_ledis_num spa_pdf_param *s)
	spa_pdf_param ***intra_trans_param;	// parameters of F0 intra transitions (has f_band_num x f_intra_mem_len params)
	spa_pdf_param *prior_param;		// parameters of F0 priors (has prior_num params)
	spa_pdf_container ****inter_trans_pdf;	// the pdf containers of F0 inter transitions (has f_band_num x f_ledis_num pdfs)
	spa_pdf_container ****intra_trans_pdf;	// the pdf containers of F0 intra transitions (has f_band_num x f_intra_mem_len pdfs)
	spa_pdf_container **prior_pdf;		// the pdf containers of F0 prior (has cand_num pdfs)
	
	spa_clus_resi_info **snr_band_resi;	// storing the information of clustered residuals 
	gsl_matrix *trans_cost_table;	// the trans cost table 
} spa_pitch_param;


// init and release pitch parameters 
int spa_spec_pch_init(char *fn, spa_spec_param *par, spa_pitch_param** ppch);
int spa_spec_pch_release(spa_pitch_param* pch);

// load reference F0 countour 
void spa_load_pch_ref_file(char *pch_file, gsl_vector **ppch_ref);

// accumulate the occurence of F0 candidates 
// now only look at the residual 
void spa_pitch_accu_cand_occurrence(spa_pitch_param *pch, spa_vector_spec_peak *peak_mat, int frm_num, gsl_vector *pch_ref, spa_clus_resi_info ***psb_resi);

// Search the local peaks in the smoothed SNR spectrum*/
void spa_spec_local_peak(gsl_vector *short_spec_smth, gsl_vector *long_spec_smth, spa_pitch_param *pch, spa_vector_spec_peak *peak_arr);

// extract pitch candidates or learn pitch model  
void spa_pitch_peak_cand (spa_spec_param *par, spa_pitch_param *pch, gsl_vector *sph, gsl_vector *uv_ref, spa_vector_spec_peak **ppeak_mat, int *pfrm_num);

// generate an lookup table transition cost 
// tab - matrix(cand_num x cand_num), tc_table(i, j) distance from freq i to freq j 
void spa_pitch_get_trans_cost_table(spa_pitch_param *pch);

// generate posterior score for each F0 candidates in a frame given spectral peaks 
void spa_pitch_cand_postprob (spa_pitch_param *pch, spa_vector_spec_peak *peak_mat, int frm_num, gsl_matrix **pcand_pp);

// accumulate F0 transition statistics for one F0 contour
int spa_pitch_accu_trans_stats(spa_pitch_param *pch, gsl_vector *pch_ref, gsl_matrix ***spbd_intra, gsl_matrix ***spbd_inter, gsl_vector **pspbd_tot);

// using dynamic programing to search a path 
void spa_pitch_cand_dp (spa_pitch_param *pch, int frm_num, gsl_matrix *cand_pp, gsl_vector *uv_ref, gsl_vector **ppch_fin);

#endif // _SPA_PITCH_H 
