////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    simp_math.c: math operations                                        //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <string.h>
#include <stdio.h>
#include <memory.h>
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_blas.h>

#include "io_shell.h"
#include "simp_math.h"
#include "sglib.h"

#define	 MINRCOND	1E-8
#define  MINSGVAL	1E-8

extern char msg[SPA_MAX_STR_LEN];

///////////////////////////////////////////////////////////////////////////
//
//              gsl vector, matrix and double array convertion
//
///////////////////////////////////////////////////////////////////////////
// gslvec to double array
void spa_io_vec_gsl2arr(const gsl_vector *vec, double **pdarr, int *pn)
{
	int i;
	double d;
	double *darr = calloc(vec->size, sizeof(double));
	for (i = 0; i < vec->size; i++) {
		darr[i] = gsl_vector_get(vec, i);
	}
	*pdarr = darr;
	*pn = vec->size;
}

void spa_io_vec_arr2gsl(const double *darr, int n, gsl_vector **pvec)
{
	int i;
	double d;
	gsl_vector *vec = gsl_vector_alloc(n);
	for (i = 0; i < n; i++) {
		gsl_vector_set(vec, i, darr[i]);
	}
	*pvec = vec;
}

void spa_io_mat_gsl2arr(gsl_matrix *mat, double ***pdarr, int *pr, int *pl)
{
	int i, j;
	double d;
	double **darr = (double **) alloc2d(mat->size1, mat->size2, sizeof(double));
	for (i = 0; i < mat->size1; i++)
		for (j = 0; j < mat->size2; j++)
			darr[i][j] = gsl_matrix_get(mat, i, j);
	
	*pdarr = darr;
	*pr = mat->size1;
	*pl = mat->size2;
}

void spa_io_mat_arr2gsl(const double **darr, int r, int l, gsl_matrix **pmat)
{
	int i, j;
	double d;
	gsl_matrix *mat = gsl_matrix_alloc(r, l);
	for (i = 0; i < r; i++)
		for (j = 0; j < l; j++)
			gsl_matrix_set(mat, i, j, darr[i][j]);
	*pmat = mat;
}


// A = exp (A)
void spa_io_gslmat_exp(gsl_matrix *mat)
{
	int i, j;
	double d;
	for (i = 0; i < mat->size1; i++) {
		for (j = 0; j < mat->size2; j++) {
			d = gsl_matrix_get(mat, i, j);
			gsl_matrix_set(mat, i, j, exp(d));
		}
	}
}

// returen the determinant of diag mat a, note that det a should be > 0
double DetDiagMat(double *a, int n)
{
	int i;
	double d = 1.0;
	for (i = 0; i < n; i++) {
		if (a[i] < SPA_DSMALL)
			a[i] = SPA_DSMALL;
		d *= a[i];
	}
	return d;
}

// avoids runtime error, for checking matrix for positive definiteness
static inline double spa_math_quiet_sqrt (double x)  
{
	return (x >= 0) ? sqrt(x) : SPA_DSMALL;
}

// return the determinant of mat a, note that det a should be > 0
// note a is changed at return
double spa_math_logdet_psdmat(double **a, int n, int bDiag)
{
	int i,j;
	double d = 0.0;
	gsl_matrix_view v;
	if (bDiag) {
		for (i = 0; i < n; i++) {
			if (a[i][i] < SPA_DSMALL)
				d += SPA_LOGZERO;
			else
				d += log(a[i][i]);
		}
	}
	else {
		
		v = gsl_matrix_view_array(a[0], n, n);
		gsl_linalg_cholesky_decomp(&v.matrix);	// A = LL'
		for (i = 0; i < n; i++) {
			if (a[i][i] < SPA_DSMALL)
				d += SPA_LOGZERO;
			else
				d += log(a[i][i]);
		}
		d *= 2.0;				// log det(A) = 2 log det(L)
	}
	return d;	
}


void CMat2FArray(double ** a, double * b, int nr, int nc)
{	int i, j;
	for (i = 0; i < nc; i++)
		for(j = 0; j < nr; j++) 
			b[j+nr*i]=a[j][i];

}

void FArray2CMat(double **a, double *b, int nr, int nc)
{	int i, j;
	for (i = 0; i < nc; i++)
		for(j = 0; j < nr; j++)
			a[j][i] = b[j+nr*i];
}	

// Inverse of a matrix, use SVD
// note that det a and det ia should all > 0
// bDiag: 1 - the a is diag
// return: log determinant of the matrix
double spa_math_inv_mat(const double **a, double **ia, int n, int bDiag)
{
	int i, j, k, r, flag;
	double d;
	double log_det = 0;
	spa_io_zero_mat(ia, 0, 0);
	if (bDiag) {
		for (i = 0; i < n; i++) {
			if (a[i][i] > SPA_DSMALL) {
				ia[i][i] = 1.0 / a[i][i];
				log_det -= log(d);
			}
			else if (a[i][i] >= 0) {
				ia[i][i] = SPA_DLARGE;
				log_det -= SPA_LOGZERO;
			}
			else {
				printf("spa_math_mat_inv_mat: matrix has %.2e on diag %d\n", a[i][i], i);
				return GSL_FAILURE;
			}
		}
	}
	else {
		double **ca = (double **) alloc2d(n, n, sizeof(double));
		gsl_matrix *u = gsl_matrix_calloc(n, n);
		gsl_matrix *v = gsl_matrix_calloc(n, n);
		gsl_vector *s = gsl_vector_calloc(n);
		gsl_vector *work = gsl_vector_calloc(n);
		spa_io_copy_mat(ca, a, n, n);
		gsl_matrix_view a_mv = gsl_matrix_view_array(ca[0], n, n);
		gsl_matrix_view a_inv_mv = gsl_matrix_view_array(ia[0], n, n);
		// A = U S V^T, U is stored in A, only diag(S) is kept in s
		flag = gsl_linalg_SV_decomp(&a_mv.matrix, v, s, work);
		for (i = 0; i < n; i++) {
			d = gsl_vector_get(s, i);
			if (d > SPA_DSMALL) {
				ia[i][i] = 1.0 / d;
				log_det -= log(d);
			}
			else {
				ia[i][i] = SPA_DLARGE;
				log_det -= SPA_LOGZERO;
			}
		}
		// A^-1 = V S^-1 U^T
		gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, v, &a_mv.matrix, 0.0, u);
		gsl_blas_dgemm(CblasNoTrans, CblasTrans, 1.0, u, &a_mv.matrix, 0.0, &a_inv_mv.matrix);
		free2d((char **) ca);
		gsl_matrix_free(u);
		gsl_matrix_free(v);
		gsl_vector_free(s);
		gsl_vector_free(work);
	}
	
	return log_det;
}

// entropy calculation
double spa_math_entropy (double *s, int n)
{
	int i;
	double d, t;
	d = 0.0;
	for (i = 0; i < n; i++) {
		t = s[i] < SPA_DSMALL ? SPA_LOGZERO : log(s[i]);
		d += s[i] * t;
	}
	return d;
}

// sign function
int spa_math_sgn(double x)
{
	double y;
	if (x > SPA_DSMALL) {
		y = 1;
	}
	else if (x < -SPA_DSMALL) {
		y = -1;
	}
	else {
		y = 0;
	}
	return y;
}

// weighted kmeans, data and their likelihoods are stored in input, the clustering resulting are stored in out
void spa_math_weight_kmeans(spa_item *input, int in_num, spa_item *out, int out_num)
{
	int i, j, k, min_j, loop;
	double d, min_d, wad, prv_wad;	// wad - weighted all distance
	spa_item *tmp = (spa_item *) calloc(out_num, sizeof(spa_item));
	
	loop = 0;
	do {
		wad = 0.0;
		for (j = 0; j < out_num; j++) {
			tmp[j].pt = 0.0;
			tmp[j].ll = 0.0;
		}
		for (i = 0; i < in_num; i++) {
			min_d = DBL_MAX;
			for (j = 0; j < out_num; j++) {
				d = fabs(out[j].pt - input[i].pt);
				if (d < min_d) {
					min_d = d;
					min_j = j;
				}
			}
			wad += input[i].ll * min_d;
			tmp[min_j].pt += input[i].pt * input[i].ll;
			tmp[min_j].ll += input[i].ll;
		}
		
		// whether is converged
		loop++;
		if (loop != 1) {
			if (wad < prv_wad) {
				d = fabs( (wad -prv_wad)/prv_wad );
				printf("   (KMeans) - Loop: %d ,  Weighted Overall Distance: %e,  Decrement : %.4f\%\n", loop, wad, 100*d);
				if (d < SPA_TR_LR_TH)
					break;
			}
			else
				break;
		}
		else {
			printf("   (KMeans) - Loop: %d ,  Weighted Overall Distance: %e\n", loop, wad);
		}		
		prv_wad = wad;
		
		// update
		d = 0;
		for (j = 0; j < out_num; j++) {
			out[j].pt = tmp[j].pt / tmp[j].ll;
			d += out[j].ll;
		}
		for (j = 0; j < out_num; j++) {
			out[j].ll = tmp[j].ll / d;
		}
		
	} while (loop <= 20);
	
	free(tmp);
}

// smoothing using a window function
// v - org, w - window, s - spectrum
void spa_math_smoothing (gsl_vector *v, gsl_vector *w, gsl_vector *s)
{
	int tot = v->size;
	int win = w->size;
	int half = (int) ((win - 1)/2);
	int i, j, st, et, len;
	double w_sum, dt, dc;
	if (tot != s->size) {
		printf("spa_math_smoothing: vector size should match.\n");
		exit(1001);
	}
	if (win > s->size) {
		printf("spa_math_smoothing: vector size should be larger than the window size.\n");
		exit(1001);
	}

	for (i = 0; i < tot; i++) {
		st = i - half;
		et = i + half;
		st = st < 0 ? 0 : st;
		et = et > tot - 1 ? tot - 1 : et;
		len = et - st + 1;
		dt = 0.0;
		w_sum = 0.0;
		for (j = st; j <= et; j++) {
			dc = gsl_vector_get(w, half + j - i);
			dt += gsl_vector_get(v, j) * dc;
			w_sum += dc;
		}
		dt /= w_sum;
		gsl_vector_set(s, i, dt);
	}
	
}


// dynamic programing
/*
input: cost - row x col 
output:tran - row x col x col, tran[i][k][j] - state k at i-1 to state j at i
out: trk - row
*/
int spa_math_dp (gsl_matrix *cost, double ***tran, gsl_vector *trk)
{
	int i, j, k, imax, prv;
	int row = cost->size1;
	int col = cost->size2;
	double cmax, cc, d;
	gsl_vector *old_acc = gsl_vector_calloc(col);
	gsl_vector *new_acc = gsl_vector_calloc(col);
	gsl_matrix_int *bkpt = gsl_matrix_int_alloc(row, col);
	
	gsl_matrix_get_row(old_acc, cost, 0);
	// forward i-1 -> i
	for (i = 1; i < row; i++) {
		for (j = 0; j < col; j++) {
			// for all possible states at time i-1, find the best path to state j at time i*/
			gsl_vector_set(new_acc, j, -DBL_MAX);
			for (k = 0; k < col; k++) {
				cc = gsl_vector_get(old_acc, k) + tran[i][k][j];
				if (cc > gsl_vector_get(new_acc, j)) {
					gsl_vector_set(new_acc, j, cc);
					gsl_matrix_int_set(bkpt, i, j, k);
				}
			}
			// update accumulative cost at state j
			d = gsl_vector_get(new_acc, j) + gsl_matrix_get(cost, i, j);
			gsl_vector_set(new_acc, j, d);
		}
		// store in old_acc
		gsl_vector_memcpy(old_acc, new_acc);
	}
	
	// backward i -> i-1
	cmax = -DBL_MAX;
	for (j = 0; j < col; j++) {
		d = gsl_vector_get(old_acc, j);
		if (d > cmax) {
			cmax = d;
			gsl_vector_set(trk, row - 1, j);
		}
	}

	for (i = row - 1; i > 0; i--) {
		prv = gsl_vector_get(trk, i);
		j = gsl_matrix_int_get(bkpt, i, prv);
		gsl_vector_set(trk, i - 1, j);
	}
	
	gsl_vector_free(new_acc);
	gsl_vector_free(old_acc);
	gsl_matrix_int_free(bkpt);
	
	return GSL_SUCCESS;
}


// search the matched bin
int spa_search_match_bin(double *lobd, double *upbd, int num, double d)
{
	int i = 0;
	int bin = -1;
	for (i = 0; i < num; i++) {
		if ( d >= lobd[i] && d < upbd[i]) {
			bin = i;
			break;
		}
	}
	if (bin == -1) {
		bin = 0;
		printf("spa_search_match_bin: No bin has this value: %e.\n", d);
		exit(1002);
	}
	return bin;
}

// get p(x) given the laplacian pdf
double spa_math_lap_pdf(double x, double mu, double b)
{
	double px;
	if (b < SPA_DSMALL) {
		//printf("warning: spa_math_lap_pdf: var of laplacian (%e) should be a positive num", b);
		b = SPA_DSMALL;
	}
	px = 0.5/b * exp(-fabs(x-mu)/b);
	return px;
}


///////////////////////////////////////////////////////////////////////////////////////
//
//                         Statistics
//
///////////////////////////////////////////////////////////////////////////////////////

// normalize a series of values: (x-u)/sigma
void spa_math_norm_vector (gsl_vector *v)
{
	double mu, std, ivs;
	mu = gsl_stats_mean(v->data, v->stride, v->size);
	std = gsl_stats_variance(v->data, v->stride, v->size);
	ivs = 1.0 / sqrt(std);
	gsl_vector_add_constant(v, -mu);
	gsl_vector_scale(v, ivs);
}

// change counts to frequency so that \sum fx = 1
void spa_math_occur_to_freq (gsl_vector *fx)
{
	int i = 0;
	double sum = 0;
	for (i = 0; i < fx->size; i++) {
		sum += gsl_vector_get(fx, i);
	}
	gsl_vector_scale(fx, 1.0/sum);
}


// suppose know the 'frequency' or probablities of occurrences of the data
// mean = \int xf(x)dx = \sum x_i f(x_i)
// note that \sum f(x_i) = 1
double spa_math_freq_mean (gsl_vector *x, gsl_vector *fx)
{
	int i, sz1, sz2;
	double sum = 0;
	sz1 = x->size;
	sz2 = x->size;
	if (sz1 != sz2) {
		printf ("spa_math_freq_mean: vector sizes mismatch.\n");
		exit(-1);
	}
	
	for (i = 0; i < sz1; i++) {
		sum += gsl_vector_get(x, i) * gsl_vector_get(fx, i);
	}
	return sum;
}

// suppose know the 'frequency' or probablities of occurrences of the data
// kth central moment = \int (x-\mu)^order f(x)dx = \sum (x_i- \hat mu)^2 f(x_i)
// note that \sum f(x_i) = 1
// if mu == 0, it is a moment, else it is a central moment
double spa_math_freq_moment (gsl_vector *x, gsl_vector *fx, double mu, int k)
{
	int i, sz1, sz2;
	double d, sum = 0;
	sz1 = x->size;
	sz2 = x->size;
	if (sz1 != sz2) {
		printf ("spa_math_freq_mean: vector sizes mismatch.\n");
		exit(-1);
	}
	
	for (i = 0; i < sz1; i++) {
		d = gsl_vector_get(x, i);
		sum += pow(d - mu, k) * gsl_vector_get(fx, i);
	}
	return sum;
}

// suppose know the 'frequency' or probablities of occurrences of the data
// kurtosis = cm4 / var^2 - 3.0
// note that \sum f(x_i) = 1
double spa_math_freq_kurtosis (gsl_vector *x, gsl_vector *fx)
{
	double mu, cm4, var, k;
	mu = spa_math_freq_mean(x, fx);
	var = spa_math_freq_moment(x, fx, mu, 2);
	cm4 = spa_math_freq_moment(x, fx, mu, 2);
	k = cm4 /(var*var) - 3.0;
	return k;
}

// kurtosis = cm4 / var^2 - 3.0
double spa_math_cm_kurtosis (double mu, double var, gsl_vector *x, gsl_vector *fx) 
{
	double cm4, k;
	cm4 = spa_math_freq_moment(x, fx, mu, 2);
	k = cm4 /(var*var) - 3.0;
	return k;
}

/***************************************************************************
 Autocorrelation, the max lag is the size of the vector
 sf - a frame of speech (should be hamming windowed)
 acorr - store the r(i)
 ***************************************************************************/
void spa_math_acorr(gsl_vector *x, gsl_vector *acorr)
{
	int i, j;
	double dsum;
	int lag = acorr->size;
	int num = x->size;
	for (i = 0; i < lag; i++) {
		// i is the curr lag
		dsum = 0.0;
		for (j = 0; j < num - i; j++) {
			dsum += gsl_vector_get(x, j) * gsl_vector_get(x, j+i);
		}
		gsl_vector_set(acorr, i, dsum);
	}
}

/*************************************************************************************
 *                                                                                   *
 *      Levison-Durbin's method                                                      *
 *      Note: the size of lpc is P, the size of acorr is P + 1, P is the LPC order   *
 *************************************************************************************/
void spa_math_lpc (gsl_vector *acorr, gsl_vector *lpc)
{
	int i, j, order;
	double en, pc, d;
	gsl_vector *cur_lpc, *new_lpc, *tmp_lpc;
	
	order = acorr->size - 1;
	en = gsl_vector_get(acorr, 0);
	cur_lpc = gsl_vector_calloc(lpc->size);
	new_lpc = gsl_vector_calloc(lpc->size);
	
	for (i = 1; i <= order; i++) {
		pc = gsl_vector_get(acorr, i);
		for (j = 1; j < i; j++) {
			pc -= gsl_vector_get(cur_lpc, j - 1) * gsl_vector_get(acorr, i - j);
		}
		pc /= en;
		
		gsl_vector_set(new_lpc, i - 1, pc);
		for (j = 1; j < i; j++) {
			d = gsl_vector_get(cur_lpc, j - 1) - pc * gsl_vector_get(cur_lpc, i - j - 1);
			gsl_vector_set(new_lpc, j - 1, d);
		}
		en *= 1.0 - pc * pc;
		tmp_lpc = cur_lpc; cur_lpc = new_lpc; new_lpc = tmp_lpc;
	}
	
	gsl_vector_memcpy(lpc, cur_lpc);
	gsl_vector_free(cur_lpc);
	gsl_vector_free(new_lpc);
}

///////////////////////////////////////////////////////////////////////////////////////
//
//      Distance Measure
//
///////////////////////////////////////////////////////////////////////////////////////
 double spa_math_ara (gsl_vector *vec_a, gsl_vector *cor_a)
{
	int i, j;
	double d, lsum;
	int dim = vec_a->size; // for speed reason, do not check the size match
	
	// a x R_b x a'
	d = 0.0;
	for (i = 0; i < dim; i++) {
		lsum = 0.0;
		for (j = 0; j < dim; j++) {
			lsum += gsl_vector_get(vec_a, j) * gsl_vector_get(cor_a, abs(i - j));
		}
		d += lsum * gsl_vector_get(vec_a, i);
	}
	return d;
}
 
 // D(a->b), b is the reference
double spa_math_vec_not_sym_dist(int dist_measure, gsl_vector *vec_a, gsl_vector *vec_b,
				 gsl_vector *cor_a, gsl_vector *cor_b)
{
	int i, j;
	int dim = vec_a->size; // for speed reason, do not check the size match
	double dba, daa, lsum, sum;
	
	if (dist_measure == SPA_DM_LPC_LR){
		// a x R_a x a'
		daa = spa_math_ara(vec_a, cor_a);
		// b x R_a x b'
		dba = spa_math_ara(vec_b, cor_a);		
		sum = dba/daa;
	}
	return sum;
}

double spa_math_vec_sym_dist(int dist_measure, gsl_vector *vec_a, gsl_vector *vec_b,
			     gsl_vector *cor_a, gsl_vector *cor_b)
{
	int i, j;
	int dim = vec_a->size; // for speed reason, do not check the size match
	double da2b, db2a, lsum, sum;
	
	if (dist_measure == SPA_DM_LPC_LR){
		// D(b->a) vec_b and cor_a
		da2b = spa_math_vec_not_sym_dist(dist_measure, vec_a, vec_b, cor_a, cor_b);
		// D(a->b), b is the reference
		db2a = spa_math_vec_not_sym_dist(dist_measure, vec_b, vec_a, cor_b, cor_a);
		sum = 0.5 * (da2b + db2a) - 1.0;
	}
	/*if (sum < 0) {
		sum = 0.0;
	}*/
	return sum;
}
 
///////////////////////////////////////////////////////////////////////////////////////
//
//      Clustering
//
///////////////////////////////////////////////////////////////////////////////////////


// load configuration for clustering
int spa_clus_load_clus_cfg(char *fn, spa_clus_param **pclus)
{
	char ctag[SPA_MAX_TAG_LEN];	// content in the <...>
	FILE *fp;

	spa_clus_param *clus = (spa_clus_param *) calloc(sizeof(spa_clus_param), 1);
	clus->dist_crit		= 1;
	clus->dist_th		= 0.0;
	
	fp = spa_io_fopen(fn, "rt");	
	while ( spa_io_get_next_tag(fp, ctag) == 1) {
		if ( strcmp(ctag, "DIST_CRIT") == 0 )
			fscanf(fp, "%d", &clus->dist_crit);
		else if ( strcmp(ctag, "DIST_TH") == 0 )
			fscanf(fp, "%lf", &clus->dist_th);
		else {
			sprintf(msg, "\"%s\" has no matching tag.\n", ctag);
			GSL_ERROR(msg, GSL_EINVAL);
		}
	}
	spa_io_fclose(fp);
	
	*pclus = clus;
	return GSL_SUCCESS;
}

// load individual list
// indi_arr[i].inst is the i_th instance list, where keeps the numbers of lists: e.g.: FILE:
// 3	// individual_num
// 20	// the number of instances of the 1st individual
// 15	// the number of instances of the 2nd individual
// 6	// the number of instances of the 3rd individual
// after load the individual list, the indi_arr stores the info:
// indi_arr[0]: 0 1 ... 19
// indi_arr[1]: 20 21 ... 
int spa_clus_load_individual_arr(char *fn, spa_clus_list **pindi_arr, int *pindi_num)
{
	int i, j, id, indi_num, inst_num;
	gsl_vector_int *indi;
	spa_inst_list *il;
	
	FILE *fp = spa_io_fopen(fn, "rt");
	fscanf(fp, "%d", &indi_num);
	
	// create an array of individuals pointers
	spa_clus_list *indi_arr = (spa_clus_list *) calloc(indi_num, sizeof(spa_clus_list));
	indi = gsl_vector_int_calloc(indi_num);
	id = 0;
	for (i = 0; i < indi_num; i++) {
		fscanf(fp, "%d", &inst_num);
		
		indi_arr[i].id = id;
		indi_arr[i].num = inst_num;
		indi_arr[i].next = NULL;
		for (j = id; j < id + inst_num; j++) {
			il = (spa_inst_list *) calloc(1, sizeof(spa_inst_list));
			il->id = j;
			il->org = i;
			SGLIB_LIST_ADD(spa_inst_list, indi_arr[i].inst, il, next);
		}
		id += inst_num;
	}
	spa_io_fclose(fp);
	
	*pindi_arr = indi_arr;
	*pindi_num = indi_num;
	return GSL_SUCCESS;
}

// release individual list
int spa_clus_release_individual_arr(spa_clus_list *indi_arr, int indi_num)
{
	int i;
	spa_inst_list *il, *it;
	for (i = 0; i < indi_num; i++) {
		il = indi_arr[i].inst;
		while (il != NULL) {
			it = il->next;
			free(il);
			il = it;
		}
	}
	free(indi_arr);
	return GSL_SUCCESS;
}

// create a '2-d' linked list: 
//   the column (only one) list is the list of clusters
//   the row list is a list of nodes in the cluster
//   char *fn is the contains which cluster this instance is from
int spa_clus_load_individual_list (gsl_matrix *dist_mat, spa_clus_list **pcoll, char *fn)
{
	int i, j, indi_num, inst_num, id;
	spa_inst_list *rowl, *il;
	spa_clus_list *coll, *cl;
	FILE *fp;
	coll = NULL;
	
	if (fn != NULL) {
		fp = spa_io_fopen(fn, "rt");
		fscanf(fp, "%d", &indi_num);
		id = 0;
		for (i = 0; i < indi_num; i++) {
			fscanf(fp, "%d", &inst_num);
			for (j = id; j < id + inst_num; j++) {
				il = (spa_inst_list *) calloc(1, sizeof(spa_inst_list));
				il->id = j;
				il->org = i;
				il->next = NULL;
				cl = calloc(1, sizeof(spa_clus_list));
				cl->id = i;
				cl->num = 1;
				cl->inst = il;
				cl->next = NULL;
				SGLIB_LIST_ADD(spa_clus_list, coll, cl, next);
			}
			id += inst_num;
		}
		*pcoll = coll;
		return indi_num;
	}
	else {	
		for (i = 0; i < dist_mat->size1; i++) {
			rowl = NULL;
			il = malloc(sizeof(spa_inst_list));
			il->id = i;
			SGLIB_LIST_ADD(spa_inst_list, rowl, il, next);
			cl = malloc(sizeof(spa_clus_list));
			cl->id = i;
			cl->num = 1;
			cl->inst = rowl;
			SGLIB_LIST_ADD(spa_clus_list, coll, cl, next);
		}
		*pcoll = coll;
		return 0;
	}
	
	
	if (fn != NULL) {
		spa_io_fclose(fp);
	}	
}

void spa_clus_release_individual_list (spa_clus_list *coll)
{
	spa_clus_list *cl, *ct;
	spa_inst_list *il, *it;
	cl = coll;
	while (cl != NULL) {
		il = cl->inst;
		while (il != NULL) {
			it = il->next;
			free(il);
			il = it;
		}
		ct = cl->next;
		free(cl);
		cl = ct;
	}
}

// distance from one instance id to a cluster cl
double spa_clus_inst_to_clus_dist (int dist_crit, gsl_matrix *orgi_dist_mat, spa_inst_list *rowl, int id)
{
	int i, inst_num;
	double di2c, d;
	spa_inst_list *il, *ol;
	
	if (rowl == NULL) {
		sprintf(msg, "spa_clus_inst_to_clus_dist: the cluster should not be empty.\n");
		GSL_ERROR(msg, GSL_EINVAL);
	}
	
	il = rowl;
	if (dist_crit == SPA_CLUS_DIST_CRIT_MIN) {
		di2c = DBL_MAX;
		while (il != NULL) {
			d = gsl_matrix_get(orgi_dist_mat, il->id, id);
			if (d < di2c) {
				di2c = d;
			}
			il = il->next;
		}
	}
	else if (dist_crit == SPA_CLUS_DIST_CRIT_MAX) {
		di2c = -DBL_MAX;
		while (il != NULL) {
			d = gsl_matrix_get(orgi_dist_mat, il->id, id);
			if (d > di2c) {
				di2c = d;
			}
			il = il->next;
		}
	}
	else if (dist_crit == SPA_CLUS_DIST_CRIT_AVG) {
		di2c = 0.0;
		inst_num = 0;
		while (il != NULL) {
			d = gsl_matrix_get(orgi_dist_mat, il->id, id);
			di2c += d;
			inst_num++;
			il = il->next;
		}
		di2c /= (double) inst_num;
	}
	return di2c;
}

// distance from one cluster to another cluster cl
double spa_clus_clus_to_clus_dist (int dist_crit, gsl_matrix *orgi_dist_mat, spa_clus_list *ca, spa_clus_list *cb)
{
	int i, inst_num;
	double dc2c, d, dia, dib;
	spa_inst_list *al, *bl;
	
	if (ca == NULL || cb == NULL) {
		sprintf(msg, "spa_clus_clus_to_clus_dist: the cluster should not be valid.\n");
		GSL_ERROR(msg, GSL_EINVAL);
	}
	al = ca->inst;
	bl = cb->inst;
	if (al == NULL || bl == NULL) {
		sprintf(msg, "spa_clus_clus_to_clus_dist: the cluster should not be empty.\n");
		GSL_ERROR(msg, GSL_EINVAL);
	}

	if (dist_crit == SPA_CLUS_DIST_CRIT_MIN) {
		dc2c = DBL_MAX;
		while (al != NULL) {
			while (bl != NULL) {
				d = gsl_matrix_get(orgi_dist_mat, al->id, bl->id);
				if (d < dc2c) {
					dc2c = d;
				}
				bl = bl->next;
			}
			al = al->next;
		}
	}
	else if (dist_crit == SPA_CLUS_DIST_CRIT_MAX) {
		dc2c = -DBL_MAX;
		while (al != NULL) {
			while (bl != NULL) {
				d = gsl_matrix_get(orgi_dist_mat, al->id, bl->id);
				if (d > dc2c) {
					dc2c = d;
				}
				bl = bl->next;
			}
			al = al->next;
		}
	}
	else if (dist_crit == SPA_CLUS_DIST_CRIT_AVG) {
		// two sum of intra distances, note a->b and b->a are counted as 1 time
		dc2c = 0.0;
		inst_num = 0;
		while (al != NULL) {
			while (bl != NULL) {
				dc2c += gsl_matrix_get(orgi_dist_mat, al->id, bl->id);
				inst_num++;
				bl = bl->next;
			}
			al = al->next;
		}
		dc2c /= (double) inst_num;
	}
	return dc2c;
}

// intra distance when two clusters are combined, not really combined
double spa_clus_combine_intra_dist (gsl_matrix *orgi_dist_mat, spa_clus_list *ca, spa_clus_list *cb)
{
	int i, inst_num;
	double dc2c, d, dia, dib;
	spa_inst_list *al, *bl;
	
	if (ca == NULL || cb == NULL) {
		sprintf(msg, "spa_clus_combine_intra_dist: the cluster should not be valid.\n");
		GSL_ERROR(msg, GSL_EINVAL);
	}
	al = ca->inst;
	bl = cb->inst;
	if (al == NULL || bl == NULL) {
		sprintf(msg, "spa_clus_combine_intra_dist: the cluster should not be empty.\n");
		GSL_ERROR(msg, GSL_EINVAL);
	}

	// two sum of intra distances, note a->b and b->a are counted as 1 time
	dia = (double) (ca->num * (ca->num - 1) / 2);
	dib = (double) (cb->num * (cb->num - 1) / 2);
	dc2c = ca->avg_intra_dist * dia + cb->avg_intra_dist * dib;
	inst_num = (ca->num * (ca->num - 1) + cb->num * (cb->num - 1)) / 2;
	while (al != NULL) {
		while (bl != NULL) {
			dc2c += gsl_matrix_get(orgi_dist_mat, al->id, bl->id);
			inst_num++;
			bl = bl->next;
		}
		al = al->next;
	}
	dc2c /= (double) inst_num;
	return dc2c;
}

// update the distance matrix of the clustering
//   the ib is merged into ia.
//   note the D(i,j) means the distance between the cluster i and j
void spa_clus_update_dist_mat (int dist_crit, gsl_matrix *dist_mat, spa_clus_list *coll, spa_clus_list *ca, spa_clus_list *cb)
{
	int i, j;
	double dka, dkb, dclus;
	spa_clus_list *ck;
	
	// traverse the cluster to update the D(k, ia)
	ck = coll;
	i = 0;
	while (ck != NULL) {
		if (ck->id != ca->id && ck->id != cb->id) {
			dka = gsl_matrix_get(dist_mat, ck->id, ca->id);
			dkb = gsl_matrix_get(dist_mat, ck->id, cb->id);
			if (dist_crit == SPA_CLUS_DIST_CRIT_MIN) {
				dclus = dka < dkb? dka : dkb;
			}
			else if (dist_crit == SPA_CLUS_DIST_CRIT_MAX) {
				dclus = dka > dkb? dka : dkb;
			}
			else {
				printf("%d distance criteria is not accepted.\n", dist_crit);
			}
			gsl_matrix_set(dist_mat, ck->id, ca->id, dclus);
			gsl_matrix_set(dist_mat, ca->id, ck->id, dclus);
		}
		ck = ck->next;
	}
	
	// need to update the D (k, ib) to be all zeros
	for (i = 0; i < dist_mat->size1; i++) {
		gsl_matrix_set(dist_mat, i, cb->id, 0.0);
		gsl_matrix_set(dist_mat, cb->id, i, 0.0);
	}
}

// given current clusters, decrease the number of the clusters by 1 
//   return the number of clusters, if the intra cluster distance exceed some threshold, stop
int spa_clus_hierachy_iter (spa_clus_param *clus, gsl_matrix *dist_mat, spa_clus_list *coll)
{
	int i, j, clus_num;
	double min_dist, d;
	spa_clus_list *ca, *cb, *cn, *min_ca, *min_cb, *tmp;
	
	if (coll->next == NULL) {
		return 1;
	}
	
	min_dist = DBL_MAX;
	ca = cb = coll;
	clus_num = 0;
	while (ca != NULL) {
		cb = ca;
		while (cb->next != NULL) {
			// find the two clusters with minimum distances
			// d = spa_clus_clus_to_clus_dist(clus->dist_crit, dist_mat, ca, cb->next);
			d = spa_clus_combine_intra_dist(dist_mat, ca, cb->next);
			if (d < min_dist) {
				min_dist = d;
				min_ca = ca;
				min_cb = cb;
			}
			cb = cb->next;
		}
		ca = ca->next;
		clus_num++;
	}
	clus->min_inter_dist = min_dist;
	// merge the two closest clusters min_ca and min_cb->next
	// 0. update the distance matrix
	// spa_clus_update_dist_mat (clus->dist_crit, dist_mat, coll, min_ca, min_cb->next);
	min_ca->avg_intra_dist = spa_clus_combine_intra_dist(dist_mat, min_ca, min_cb->next);
	if (min_ca->avg_intra_dist > clus->dist_th) {
		return -1;
	}
	
	// 1. append min_cb->next->inst to the end of min_ca->inst
	SGLIB_LIST_CONCAT(spa_inst_list, min_ca->inst, min_cb->next->inst, next);
	min_ca->num += min_cb->next->num;
	
	// 2. delete the node in the clustering list, note min_cb will always not be NULL
	tmp = min_cb->next;
	min_cb->next = tmp->next;
	free(tmp);
	
	// two clusters merged
	clus_num--;
	return clus_num;
}

// the average of intra cluster distances (the cluster is stored in an array)
// inst_arr holds the instances, indi_num is the size of it
double spa_clus_avg_intra_single_clus_dist_inarr (spa_clus_param *clus, gsl_matrix *orgi_dist_mat, int *inst_arr, int inst_num)
{
	int i, j, isum;
	double dsum, d;
	dsum = 0.0;
	for (i = 0; i < inst_num; i++) {
		for (j = i + 1; j < inst_num; j++) {
			d = gsl_matrix_get(orgi_dist_mat, inst_arr[i], inst_arr[j]);
			dsum += d;
		}
	}
	isum = inst_num * (inst_num - 1) / 2;
	return dsum / (double) isum;
}

// the average of intra cluster distances (the cluster is stored in a list)
double spa_clus_avg_intra_single_clus_dist (spa_clus_param *clus, gsl_matrix *orgi_dist_mat, spa_inst_list *rowl, double *pdsum, int *pisum)
{
	int inst_num, isum;
	spa_inst_list *ia, *ib;
	double d, dsum;
	
	ia = rowl;
	dsum = 0.0;
	inst_num = 0;
	while (ia != NULL) {
		ib = ia->next;
		while (ib != NULL) {
			d = gsl_matrix_get(orgi_dist_mat, ia->id, ib->id);
			dsum += d;
			ib = ib->next;
		}
		ia = ia->next;
		inst_num++;
	}
	
	isum = inst_num * (inst_num - 1) / 2;
	if (pdsum != NULL) {
		*pdsum = dsum;
	}
	if (pisum != NULL) {
		*pisum = isum;
	}
	return dsum / (double) isum;
}

// the average of intra cluster distances
double spa_clus_avg_intra_clus_dist (spa_clus_param *clus, gsl_matrix *orgi_dist_mat, spa_clus_list *coll)
{
	int si, ai;
	double sd, ad;
	spa_clus_list *ca, *cb, *min_ca, *min_cb, *tmp;
	
	if (coll->next == NULL) {
		return 0.0;
	}
	
	ca = coll;
	ad = 0.0;
	ai = 0;
	while (ca != NULL) {
		spa_clus_avg_intra_single_clus_dist(clus, orgi_dist_mat, ca->inst, &sd, &si);
		ad += sd;
		ai += si;
		ca = ca->next;
	}
	return ad / (double) ai;
}

// the average of inter cluster distances
double spa_clus_avg_inter_clus_dist (spa_clus_param *clus, gsl_matrix *dist_mat, spa_clus_list *coll)
{
	int i, j, clus_num;
	double d, davg, isum;
	spa_clus_list *ca, *cb, *min_ca, *min_cb, *tmp;
	
	if (coll->next == NULL) {
		return 0.0;
	}
	
	ca = coll;
	davg = 0.0;
	clus_num = 0;
	while (ca != NULL) {
		cb = ca->next;
		while (cb != NULL) {
			// calulate the distance between two clusters
			d = gsl_matrix_get(dist_mat, ca->id, cb->id);
			davg += d;
			cb = cb->next;
		}
		ca = ca->next;
		clus_num++;
	}
	
	davg *= 2.0;
	isum = clus_num * clus_num;
	davg /= (double) isum;
	return davg;
}

// output intermediate clustering output
// if flag_details is true, print the id of the node in each clusters
void spa_clus_intermediate_output(spa_clus_param *clus, char *fn, spa_clus_list *coll, int flag_details)
{
	int clus_num;
	// print the clustering situation
	FILE *fp = spa_io_fopen(fn, "at");
	
	spa_clus_list *cl;
	spa_inst_list *il;
	cl = coll;
	clus_num = 1;
	while (cl != NULL) {
		if (flag_details) {
			fprintf(fp, "Cluster %d has %d instances.\n", clus_num, cl->num);
			while (il != NULL) {
				fprintf(fp, "%d ", il->id);	
				il = il->next;	
			}
			fprintf(fp, "\n");
		}
		
		cl = cl->next;
		clus_num++;
	}
	
	fprintf(fp, "%d\t%e\t%e\t%e\t%e\n", clus_num, clus->intra_dist, clus->inter_dist, clus->dratio, clus->min_inter_dist);
	/*
	fprintf(fp, "CLUS_NUM %d\n", clus_num);
	fprintf(fp, "DIST_RATIO %e\n", clus->dratio);
	fprintf(fp, "MIN_INTER_DIST %e\n\n", clus->min_inter_dist);
	*/
	
	spa_io_fclose(fp);
}

// display the clusters
void spa_clus_result_display (spa_clus_list *coml, char *fn, int indi_num)
{
	int i, j, k;
	spa_clus_list *cl;
	spa_inst_list *il;
	int *indi_count = calloc(sizeof(int), indi_num);
	int *indi_item  = calloc(sizeof(int), indi_num);	// an example for the clustered samples
	FILE *fp = spa_io_fopen(fn, "wt");
	
	i = 0;
	cl = coml;
	while (cl != NULL) {
		i++;
		cl = cl->next;
	}
	fprintf(fp, "%d\n\n", i);
	
	i = 0;
	cl = coml;
	while (cl != NULL) {
		il = cl->inst;
		i++;
		fprintf(fp, "%d\t%d\t%e\n", i, cl->num, cl->avg_intra_dist);	// number of instances, avg_distance in this common syllable
		spa_io_zero_int_vec(indi_count, indi_num);
		spa_io_init_int_vec(indi_item, indi_num, -1);
		while (il != NULL) {
			fprintf(fp, "%d ", il->id);
			il = il->next;
		}
		fprintf(fp, "\n");
		while (il != NULL) {
			fprintf(fp, "%d ", il->org);
			il = il->next;
		}
		il = cl->inst;
		while (il != NULL) {
			if (indi_count[il->org] == 0) {
			    indi_item[il->org] = il->id;
			}
			indi_count[il->org]++;			
			il = il->next;
		}
		
		k = 0;
		for (j = 0; j < indi_num; j++) {
			k = indi_item[j] > 0 ? k+1 : k;
		}
		fprintf(fp, "%d\n", k);
		for (j = 0; j < indi_num; j++) {
			if (indi_item[j] > 0) {
				fprintf(fp, "%d %d\n", j, indi_count[j]);
				il = cl->inst;
				while (il != NULL) {
					if (il->org == j) {
						fprintf(fp, "%d ", il->id);
					}
					il = il->next;
				}
				fprintf(fp, "\n");
			}
		}
		fprintf(fp, "\n");
		cl = cl->next;
	}
	
	free(indi_count);
	free(indi_item);
	spa_io_fclose(fp);
}

// get 'len' values from 'src' starting from 'loc', then put them into 'dst'
int spa_math_get_prev(gsl_vector *src, int loc, gsl_vector *dst)
{
	int i, j;
	double d;
	
	if (loc < 0|| loc >= src->size) {
		sprintf(msg, "spa_pitch_get_prev: (%d) is out of source range (%d).\n", loc, src->size);
		GSL_ERROR(msg, GSL_EINVAL);
	}
	if (dst == NULL) {
		sprintf(msg, "spa_pitch_get_prev: destination vector is null.\n");
		GSL_ERROR(msg, GSL_EINVAL);
	}

	// fill into destinations
	for (i = 0, j = loc; i < dst->size; i++, j--) {
		j = j < 0 ? 0 : j;
		d = gsl_vector_get(src, j);
		gsl_vector_set(dst, i, d);
	}
	return GSL_SUCCESS;
}


///////////////////////////////////////////////////////////////////////////////////
//
//		Distributions
//
///////////////////////////////////////////////////////////////////////////////////
int spa_math_pdf_load_cfg (char *fn, spa_pdf_param **ppdf)
{
	char ctag[SPA_MAX_TAG_LEN];	// content in the <...>
	int i, j, dim_num1, dim_num2, dim_num3, lin_flag;
	double space, d, rstep, dlo, dhi;
	FILE *fp;

	spa_pdf_param *pdf = (spa_pdf_param *) calloc(1, sizeof(spa_pdf_param));
	pdf->dim_num = 0;
	pdf->flr_fac = 0;
	pdf->bin_num = NULL;
	pdf->base_num = NULL;
	pdf->smth_len = NULL;
	pdf->bin_lobd = NULL;
	pdf->bin_upbd = NULL;
	
	fp = spa_io_fopen(fn, "rt");	
	while ( spa_io_get_next_tag(fp, ctag) == 1) {
		if ( strcmp(ctag, "DIM_NUM") == 0 )
			fscanf(fp, "%d", &pdf->dim_num);
		else if ( strcmp(ctag, "MOD_NUM") == 0 )
			fscanf(fp, "%d", &pdf->mod_num);
		else if ( strcmp(ctag, "FLOOR_FAC") == 0 )
			fscanf(fp, "%lf", &pdf->flr_fac);
		else if ( strcmp(ctag, "BOUND_LOC") == 0) {
			// <BOUND_LOC> dim_num lin_flag
			fscanf(fp, "%d", &dim_num1);
			pdf->bin_num = calloc(dim_num1, sizeof(int));
			pdf->bin_lobd = calloc(dim_num1, sizeof(double *));
			pdf->bin_upbd = calloc(dim_num1, sizeof(double *));
			fscanf(fp, "%d", &lin_flag);
			for (i = 0; i < dim_num1; i++) {
				// bin_num[i] ...
				fscanf(fp, "%d", &pdf->bin_num[i]);
				pdf->bin_lobd[i] = calloc(pdf->bin_num[i], sizeof(double));
				pdf->bin_upbd[i] = calloc(pdf->bin_num[i], sizeof(double));
				pdf->bin_lobd[i][0] = -DBL_MAX;
				pdf->bin_upbd[i][pdf->bin_num[i] - 1] = DBL_MAX;
				if (lin_flag) {
					fscanf(fp, " %lf", &pdf->bin_upbd[i][0]);
					fscanf(fp, " %lf", &pdf->bin_lobd[i][pdf->bin_num[i] - 1]);
					if (pdf->bin_num[i] > 2) {
						space = ( pdf->bin_lobd[i][pdf->bin_num[i] - 1] - pdf->bin_upbd[i][0])
								/ (double) (pdf->bin_num[i] - 2);
						for (j = 0; j < pdf->bin_num[i] - 1; j++) {
							pdf->bin_upbd[i][j] = pdf->bin_upbd[i][0] + space * j;
							pdf->bin_lobd[i][j+1] = pdf->bin_upbd[i][j];
						}
					}
				}
				else {
					// log scale
					fscanf(fp, " %lf", &dlo);
					fscanf(fp, " %lf", &dhi);
					rstep = pow(dhi / dlo, 1.0/(double) (pdf->bin_num[i] - 1));
					d = dlo;
					for (j = 0; j < pdf->bin_num[i] - 1; j++) {
						pdf->bin_upbd[i][j] = d;
						d *= rstep;
						pdf->bin_upbd[i][j] = 0.5 * (pdf->bin_upbd[i][j] + d);
						pdf->bin_lobd[i][j+1] = pdf->bin_upbd[i][j];
					}
				}
			}
		}
		else if ( strcmp(ctag, "SMTH_LEN") == 0 ) {
			fscanf(fp, "%d", &dim_num2);
			pdf->smth_len = calloc(dim_num2, sizeof(int));
			for (i = 0; i < dim_num2; i++) {
				fscanf(fp, "%d", &pdf->smth_len[i]);
			}
		}
		else {
			sprintf(msg, "spa_math_pdf_load_cfg: \"%s\" has no matching tag.\n", ctag);
			GSL_ERROR(msg, GSL_EINVAL);
		}
	}
	spa_io_fclose(fp);
	
	if (dim_num1 != pdf->dim_num || dim_num2 != pdf->dim_num) {
		spa_math_pdf_release_cfg(pdf);
		sprintf(msg, "spa_math_pdf_load_cfg: dimension number does not match.\n", ctag);
		GSL_ERROR(msg, GSL_EINVAL);
	}
	
	pdf->base_num = calloc(pdf->dim_num, sizeof(double));
	pdf->base_num[pdf->dim_num - 1] = 1;
	for (i = pdf->dim_num - 2; i >= 0 ; i--) {
		pdf->base_num[i] = pdf->base_num[i+1] * pdf->bin_num[pdf->dim_num-i-2];
	} 
	
	*ppdf = pdf;
	return GSL_SUCCESS;
}

int spa_math_pdf_release_cfg(spa_pdf_param *pdf)
{
	int i;
	if (pdf == NULL)
		return GSL_SUCCESS;
	if (pdf->bin_num != NULL) {
		free(pdf->bin_num);
	}
	if (pdf->smth_len != NULL) {
		free(pdf->smth_len);
	}
	for (i = 0; i < pdf->dim_num; i++) {
		free(pdf->bin_lobd[i]);
		free(pdf->bin_upbd[i]);
	}
	free(pdf->bin_lobd);
	free(pdf->bin_upbd);
	free(pdf->base_num);
	return GSL_SUCCESS;
}

// Initialize and release pdf table
int spa_math_pdf_alloc (spa_pdf_container ***ppp, spa_pdf_param *pdf)
{
	int i, size;
	spa_pdf_container **pp, *p;
	
	for (i = 0, size = 1; i < pdf->dim_num; i++) {
		size *= pdf->bin_num[i];
	}
	pp = (spa_pdf_container **) calloc(pdf->mod_num, sizeof(spa_pdf_container *));
	for (i = 0; i < pdf->mod_num; i++) {
		pp[i] = (spa_pdf_container *) calloc(1, sizeof(spa_pdf_container));
		p = pp[i];
		//p->curr_bin = calloc(pdf->dim_num, sizeof(int));
		p->tab_size = size;
		p->pdf_tab = calloc(p->tab_size, sizeof(double));
		p->mod_loaded = false;
		p->fea_loaded = false;
	}
	
	*ppp = pp;
	return GSL_SUCCESS;
}

// load the all the features for model training purpose
// input: pdf, fea_lst
// in/output: pp
int spa_math_pdf_load_all_feature (spa_pdf_param *pdf, spa_pdf_container **pp, char *fea_lst)
{
	int m, r, file_num;
	spa_pdf_container *p;
	char str_tmp[SPA_MAX_STR_LEN];
	FILE *fp;
	
	// Load features, there are mod_num rounds
	fp = spa_io_fopen(fea_lst, "rt");
	for (m = 0; m < pdf->mod_num; m++) {
		p = pp[m];
		fscanf(fp, "%s %d\n", str_tmp, &p->utt_num);
		strcpy(p->mod_name, str_tmp);
		// alloc the array for storing the feature list;
		p->utt_name = (char **) calloc(p->utt_num, sizeof(char *));
		for (r = 0; r < p->utt_num; r++) {
			fscanf(fp, "%s\n", str_tmp);
			p->utt_name[r] = (char *) calloc(strlen(str_tmp), sizeof(char));
			strcpy(p->utt_name[r], str_tmp);
		}
		p->fea_loaded = true;
	}
	spa_io_fclose(fp);
	return GSL_SUCCESS;
}

// load all models
// input: pdf, fname 
// in/output: pp
int spa_math_pdf_load_all_model (spa_pdf_param *pdf, spa_pdf_container ***ppp, char *fname)
{
	int m, mod_num;
	spa_pdf_container **pp, *p;
	
	spa_math_pdf_alloc(&pp, pdf);
	
	FILE *fp = spa_io_fopen(fname, "rb");
	// read model nums
	fread(&mod_num, sizeof(int), 1, fp);
	if (mod_num != pdf->mod_num) {
		sprintf(msg, "spa_math_pdf_load_all_model: model number mismatch.\n");
		GSL_ERROR(msg, EINVAL);
	}
	// read model content
	for (m = 0; m < mod_num; m++) {
		p = pp[m];
		fread(p->mod_name, sizeof(char), SPA_MAX_STR_LEN, fp);
		fread(&p->tab_size, sizeof(int), 1, fp);
		fread(p->pdf_tab, sizeof(double), p->tab_size, fp);
		p->mod_loaded = true;
	}
	spa_io_fclose(fp);
	
	*ppp = pp;
	return GSL_SUCCESS;
}


int spa_math_pdf_release (spa_pdf_container **pp, int mod_num)
{
	int m, r;
	if (pp == NULL) {
		return GSL_SUCCESS;
	}
	spa_pdf_container *p;
	for (m = 0; m < mod_num; m++) {
		p = pp[m];
		// free(p->curr_bin);
		if (p->fea_loaded) {
			for (r = 0; r < p->utt_num; r++) {
				free(p->utt_name[r]);
			}
			free(p->utt_name);
		}
		if (p->mod_loaded) {
			free(p->pdf_tab);
		}
		free(p);
	}
	free(pp);
	return GSL_SUCCESS;
}

// find the bin
// input: 
//		loc_vec
// return:
//		location in pdf_tab
int spa_math_pdf_vec_to_lin (spa_pdf_param *pdf, int *loc_vec)
{
	int i, loc;
	loc = 0;
	for (i = 0; i < pdf->dim_num; i++) {
		loc += pdf->base_num[i] * loc_vec[i];
	}
	return loc;
}

int spa_math_pdf_lin_to_vec (spa_pdf_param *pdf, int lin_loc, int *loc_vec)
{
	int i, resi;
	resi = lin_loc;
	for (i = 0; i < pdf->dim_num; i++) {
		loc_vec[i] = resi / pdf->base_num[i];
		resi = resi % pdf->base_num[i];
	}
	return GSL_SUCCESS;
}

// quantize the feature and return the location in the pdf table
// input: 
//		fea_vec
// return:
//		location in pdf_tab
int spa_math_pdf_fea_in_lin (spa_pdf_param *pdf, double *fea_vec)
{
	int i, *loc_vec, loc;
	
	loc_vec = calloc(pdf->dim_num, sizeof(int));
	for (i = 0; i < pdf->dim_num; i++) {
		loc_vec[i] = spa_search_match_bin(pdf->bin_lobd[i], pdf->bin_upbd[i], pdf->bin_num[i], fea_vec[i]);
	}
	loc = spa_math_pdf_vec_to_lin(pdf, loc_vec);
	free(loc_vec);
	return loc;
}

// use features to learn a distribution
// input:
//		pdf: configuration
//		p: pdf container
int spa_math_pdf_accu_stats (spa_pdf_param *pdf, spa_pdf_container *p)
{
	int i = 0, r, frm_num, dim_num, loc;
	double sum, cur_prob, avg_prob, fac, **fea_mat;
	
	if (!p->fea_loaded) {
		return spa_io_saybye("The feature list is not loaded.\n");
	}
	
	p->tot_frm_num = 0;
	for (r = 0; r < p->utt_num; r++) {
		fea_mat = NULL;
		spa_io_read_htk_buf(p->utt_name[r], SPA_SWAP_NO, &fea_mat, &frm_num, &dim_num);
		for (i = 0; i < frm_num; i++) {
 			loc = spa_math_pdf_fea_in_lin(pdf, fea_mat[i]);
			p->pdf_tab[loc] += 1.0;
		}
		free2d((char **) fea_mat);
		p->tot_frm_num += frm_num;
	}
	
	fac = 1.0 / (double) p->tot_frm_num;
	pdf->prob_flr = pdf->flr_fac * fac;
	for (i = 0; i < p->tab_size; i++) {
		p->pdf_tab[i] *= fac;
		if (p->pdf_tab[i] < pdf->prob_flr) {
			p->pdf_tab[i] = pdf->prob_flr;
		}
	}
	return GSL_SUCCESS;
}

// search the neighborhoods
// input:
//		pdf: configuration
//		row_num: the size of lobd_vec and upbd_vec
//		lobd_vec: lower bound vector
//		upbd_vec: upper bound vector
//		cur_vec: current location vector, start from lobd_vec
//		cur_row: current row, starts from 0
//		neb_vec: place holding location in a linear memory 
//		cur_neb: current neighbor location in a linear memory, starts from 0

int spa_math_pdf_next_flip (spa_pdf_param *pdf, int row_num, int *lobd_vec, int *upbd_vec,
			    int *cur_vec, int cur_row, int **neb_vec, int cur_neb)
{
	int mem_cur;
	// starting point should be valid
	if (cur_vec[cur_row] < upbd_vec[cur_row]) {
		spa_io_copy_int_vec(neb_vec[cur_neb], cur_vec, row_num);
		cur_neb++;
		cur_row = 0;
		cur_vec[cur_row]++;
		spa_math_pdf_next_flip(pdf, row_num, lobd_vec, upbd_vec, cur_vec, cur_row, neb_vec, cur_neb);
	}
	else if (cur_row + 1 < row_num) {
		// need to set previous rows to lower bounder
		mem_cur = cur_row;
		while (cur_row >= 0) {
			cur_vec[cur_row] = lobd_vec[cur_row];
			cur_row--;
		}
		cur_row = mem_cur + 1;
		cur_vec[cur_row]++;
		spa_math_pdf_next_flip(pdf, row_num, lobd_vec, upbd_vec, cur_vec, cur_row, neb_vec, cur_neb);
	}
}

/*
int spa_math_pdf_next_flip (spa_pdf_param *pdf, int row_num, int *lobd_vec, int *upbd_vec,
			    int *cur_vec, int cur_row, int **neb_vec, int cur_neb)
{
	int mem_cur;
	// starting point should be valid
	if (cur_vec[cur_row] < upbd_vec[cur_row]) {
		spa_io_copy_int_vec(neb_vec[cur_neb], cur_vec, row_num);
		cur_neb++;
		cur_row = 0;
		cur_vec[cur_row]++;
		spa_math_pdf_next_flip(pdf, row_num, lobd_vec, upbd_vec, cur_vec, cur_row, neb_vec, cur_neb);
	}
	else if (cur_row + 1 < row_num) {
		// need to set current rows to lower bound
		cur_vec[cur_row] = lobd_vec[cur_row];
		cur_row++;
		cur_vec[cur_row]++;
		spa_math_pdf_next_flip(pdf, row_num, lobd_vec, upbd_vec, cur_vec, cur_row, neb_vec, cur_neb);
	}
}
*/

// generate a bin's neighborhoods and itself
// input:
//		loc_vec: target bin
//		span[i] is the span length on the target (loc_vec[i] +- span[i])
// output:
//		plin_vec: a list of neighborhoods of the target bin
int spa_math_pdf_bin_neighbor (spa_pdf_param *pdf, spa_pdf_container *p,
			       int *loc_vec, int *span_vec, int ***pneb_vec, int *pneb_num)
{
	int i, j, lo, hi, k, neb_num, **neb_vec, *cur_vec, *lobd_vec, *upbd_vec;
	neb_num = 1;
	cur_vec = calloc(pdf->dim_num, sizeof(int));
	upbd_vec = calloc(pdf->dim_num, sizeof(int));
	lobd_vec = calloc(pdf->dim_num, sizeof(int));
	for (i = 0; i < pdf->dim_num; i++) {
		lo = loc_vec[i] - span_vec[i];
		hi = loc_vec[i] + span_vec[i] + 1;
		lo = lo >= 0 ? lo : 0;
		hi = hi <= pdf->bin_num[i] ? hi : pdf->bin_num[i];
		lobd_vec[i] = lo;
		cur_vec[i] = lo;
		upbd_vec[i] = hi;
		neb_num *= hi - lo;
	}
	
	neb_vec = (int **) alloc2d(neb_num, pdf->dim_num, sizeof(int));
	
	spa_math_pdf_next_flip(pdf, pdf->dim_num, lobd_vec, upbd_vec, cur_vec, 0, neb_vec, 0);
	
	free(cur_vec);
	free(upbd_vec);
	free(lobd_vec);
	*pneb_vec = neb_vec;
	*pneb_num = neb_num;
	return GSL_SUCCESS;
}

// smoothing the distribution
int spa_math_pdf_smooth (spa_pdf_param *pdf, spa_pdf_container *p)
{
	int i, j, k, *loc_vec, **neb_vec, neb_num, lin_loc;
	double prob, sum, *new_pdf_tab, cur_ratio, max_ratio;
	
	loc_vec = calloc(pdf->dim_num, sizeof(int));
	new_pdf_tab = calloc(p->tab_size, sizeof(double));
	spa_io_zero_vec(new_pdf_tab, p->tab_size);
	sum = 0;
	printf("%d\n", p->tab_size);
	for (i = 0; i < p->tab_size; i++) {
		if (i % 10000 == 0) {
			printf("%.4f\n", (double) i*100 / (double) p->tab_size);
		}
		spa_math_pdf_lin_to_vec (pdf, i, loc_vec);
		spa_math_pdf_bin_neighbor (pdf, p, loc_vec, pdf->smth_len, &neb_vec, &neb_num);
		prob = 0;
		for (j = 0; j < neb_num; j++) {
			lin_loc = spa_math_pdf_vec_to_lin(pdf, neb_vec[j]);
			max_ratio = -DBL_MAX;
			for (k = 0; k < pdf->dim_num; k++) {
				cur_ratio = fabs(neb_vec[j][k] - loc_vec[k]) / (double) pdf->smth_len[k];
				if (cur_ratio > max_ratio) {
					max_ratio = cur_ratio;
				}
			}
			prob += p->pdf_tab[lin_loc] * (1 - max_ratio);		// times a weighting factor
		}
		prob /= (double) neb_num;
		new_pdf_tab[i] = prob > pdf->prob_flr ? prob : pdf->prob_flr;
		sum += new_pdf_tab[i];
		free2d((char **)neb_vec);
	}
	printf("\n\n");
	for (i = 0; i < p->tab_size; i++) {
		p->pdf_tab[i] = new_pdf_tab[i] / sum;
		if (p->pdf_tab[i] < pdf->prob_flr) {
			p->pdf_tab[i] = pdf->prob_flr;
		}
	}
	

	free(loc_vec);
	free(new_pdf_tab);
	p->mod_loaded = true;
	return GSL_SUCCESS;
}

// save model
int spa_math_pdf_save_all_model (spa_pdf_container **pp, int mod_num, char *fname)
{
	int m;
	spa_pdf_container *p;
	FILE *fp = spa_io_fopen(fname, "wb");
	
	// write down model nums
	fwrite(&mod_num, sizeof(int), 1, fp);
	
	// write down model content
	for (m = 0; m < mod_num; m++) {
		p = pp[m];
		fwrite(p->mod_name, sizeof(char), SPA_MAX_STR_LEN, fp);
		fwrite(&p->tab_size, sizeof(int), 1, fp);
		fwrite(p->pdf_tab, sizeof(double), p->tab_size, fp);
	}
	spa_io_fclose(fp);
	
	return GSL_SUCCESS;
}


int spa_math_pdf_train_all_model (char *mod_cfg_file, char *fea_lst, spa_pdf_param **ppdf, spa_pdf_container ***ppp)
{
	int m, mod_num;
	spa_pdf_param *pdf;
	spa_pdf_container **pp;
	
	spa_math_pdf_load_cfg(mod_cfg_file, &pdf);
	spa_math_pdf_alloc (&pp, pdf);
	spa_math_pdf_load_all_feature(pdf, pp, fea_lst);
	
	for (m = 0; m < pdf->mod_num; m++) {
		spa_math_pdf_accu_stats(pdf, pp[m]);
		spa_math_pdf_smooth(pdf, pp[m]);
	}
	
	*ppp = pp;
	*ppdf = pdf;
	return GSL_SUCCESS;
	
}