////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    simp_math.h:                                                        //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#ifndef _SPA_MATH_H
#define _SPA_MATH_H

#define SPA_LOGZERO	(-46.05)   		// ~log(0) log values < LSMALL are set to LOGZERO
#define SPA_DSMALL	(1E-20)   		// min value lower bound
#define SPA_ASMALL	(1E-10)   		// a small value
#define SPA_ALARGE	(1E+10)   		// a large value
#define SPA_DLARGE	(1E+20)   		// max value upper bound
#define SPA_EXPINF	(46.05)   		// if x > EXPINF, exp(x) = DLARGE
#define SPA_TR_LR_TH	(0.001)			// convergence threshhold

// Distance Measure
#define SPA_DM_PNORM		0	// L_p norm based measure*/
#define SPA_DM_LOG_PNORM	1	// L_p log norm based measure*/
#define SPA_DM_LPC_LR		2	// Likelihood ratio using LPC
#define SPA_DM_ITAKURA		3	// Itakura
#define SPA_DM_ITA_SAITO	4	// Itakura - Saito

// Clustering
#define SPA_CLUS_DIST_CRIT_MIN	0
#define SPA_CLUS_DIST_CRIT_MAX	1
#define SPA_CLUS_DIST_CRIT_AVG	2

#include "io_shell.h"

typedef struct _spa_item {
	double pt;	// location
	double ll;	// value
} spa_item;

typedef struct _spa_value_list {
	double val;
	struct _spa_value_list *next;
} spa_value_list;

typedef struct _spa_clus_param {
	int dist_crit;		// criteria for clustering
	double dist_th;		// threshold for clustering
	double intra_dist;	// average of the intra dist within each cluster
	double inter_dist;	// average inter dist between clusters
	double dratio;		// dist_intra / dist_inter
	double min_inter_dist;	// minimum distance between clusters
} spa_clus_param;

typedef struct _spa_inst_list {
	int id;				// node id
	int org;			// which individual it is from
	double acc;			// accumulative cost
	struct _spa_inst_list *bkpt;	// backtrack pointer
	struct _spa_inst_list *next;	// next possible candidates
} spa_inst_list;

typedef struct _spa_clus_list {
	int id;				// id assigned at the initialization
	int num;			// number of insts in the cluster
	double avg_intra_dist;		// average of the intra cluster distance
	struct _spa_clus_list *next;
	struct _spa_inst_list *inst;
} spa_clus_list;

// a table holding the a multidimensional 
typedef struct _spa_pdf_container {
	bool mod_loaded;			// whether the model is loaded
	bool fea_loaded;			// whether the feature files are loaded
	char mod_name[SPA_MAX_STR_LEN];		// model name
	char **utt_name;			// array of utterance names
	int utt_num;				// number of utterance
	int tot_frm_num;			// number of total frames
	double **fea_mat;			// Feature
	int *curr_bin;				// size: dim_num, current bin ids
	int tab_size;				// size of the tab array
	double *pdf_tab;			// size: bin_num[0] x ... x bin_num[dim_num-1], each item holds a probability on that region
} spa_pdf_container;
		
typedef struct _spa_pdf_param {
	int dim_num;			// dimension
	int mod_num;			// model number
	int *smth_len;			// size: dim, each one means the smooth len on that dimension
	int *bin_num;			// size: dim_num, each one holds the number of bins on that dimension
	int *base_num;			// size: dim_num, the basis
	double **bin_lobd;		// size: dim_num, each dimension holds the lower bounds
	double **bin_upbd;		// size: dim_num, each dimension holds the upper bounds
	double flr_fac;			// variance flooring factor;
	double prob_flr;		// probability floor
} spa_pdf_param;

void ShowVec(double *v, int s);
void ShowMat(double **m, int r, int l);
// set the elements of vector and matrix to be all 0s.
void ZeroVec(double *v, int s);
void ZeroMat(double **m, int r, int l);
void ZeroCub(double ***m, int x, int y, int z);

// gslvec to double array
void spa_io_vec_gsl2arr(const gsl_vector *vec, double **pdarr, int *pn);
void spa_io_vec_arr2gsl(const double *darr, int n, gsl_vector **pvec);
void spa_io_mat_arr2gsl(const double **darr, int r, int l, gsl_matrix **pmat);
void spa_io_mat_gsl2arr(gsl_matrix *mat, double ***pdarr, int *pr, int *pl);
// A = exp (A)
void spa_io_gslmat_exp(gsl_matrix *mat);

// transform c-style mat to fortran-style vec
// return the pt to the vec
double * C2FMat(double **c, int row, int col);
// diag(AMA^T), size s
void CovTrans(double **A, double **M, int s, int p, double *v, int bDiag);
// vector distance
double VecDist(float *f1, float *f2, int s);

int Cholesky(double **A, double **L, int size);

double InvertCov(double** c, double **invc, int n);

double LogCovDet(double** c, int n);
// returen the determinant of mat a
double spa_math_logdet_psdmat(double **a, int n, int bDiag);
// Inverse of the matrix
double spa_math_inv_mat(const double **a, double **ia, int n, int bDiag);
int IsMatPsd(double **ia, int s);
//int minvert(double **a, double **ainv, int len);

// smoothing using a window function
void spa_math_smoothing (gsl_vector *v, gsl_vector *w, gsl_vector *s);
// entropy calculation
double spa_math_entropy (double *s, int n);
// sign function
int spa_math_sgn(double x);
// get p(x) given the laplacian pdf
double spa_math_lap_pdf(double x, double mu, double b);

// dynamic programming
int spa_math_dp (gsl_matrix *cost, double ***tran, gsl_vector *trk);

// normalize a series of values: (x-u)/sigma
void spa_math_norm_vector (gsl_vector *v);

// search the matched bin
int spa_search_match_bin(double *lobd, double *upbd, int num, double d);

// generate an lookup table for the truncated laplacian distribution
// tab - matrix(abs_snr_num x sample points)
double spa_math_get_trclap_table (gsl_matrix *tab, gsl_vector *var, gsl_vector *vec);


// statistics
void spa_math_occur_to_freq (gsl_vector *fx);
double spa_math_freq_mean (gsl_vector *x, gsl_vector *fx);
double spa_math_freq_moment (gsl_vector *x, gsl_vector *fx, double mu, int k);
double spa_math_freq_kurtosis (gsl_vector *x, gsl_vector *fx);
double spa_math_cm_kurtosis (double mu, double var, gsl_vector *x, gsl_vector *fx);

// autocorr

void spa_math_acorr(gsl_vector *x, gsl_vector *acorr);
void spa_math_lpc (gsl_vector *acorr, gsl_vector *lpc);

double spa_math_ara (gsl_vector *vec_a, gsl_vector *cor_a);
double spa_math_vec_not_sym_dist(int dist_measure, gsl_vector *vec_a, gsl_vector *vec_b,
				 gsl_vector *cor_a, gsl_vector *cor_b);
double spa_math_vec_sym_dist(int dist_measure, gsl_vector *vec_a, gsl_vector *vec_b,
				 gsl_vector *cor_a, gsl_vector *cor_b);
				 
				 
// Clustering

int spa_clus_load_clus_cfg(char *fn, spa_clus_param **pclus);
int spa_clus_load_individual_list (gsl_matrix *dist_mat, spa_clus_list **pcoll, char *fn);
void spa_clus_release_individual_list (spa_clus_list *coll);
int spa_clus_hierachy_iter (spa_clus_param *clus, gsl_matrix *dist_mat, spa_clus_list *coll);
double spa_clus_avg_intra_single_clus_dist_inarr (spa_clus_param *clus, gsl_matrix *orgi_dist_mat, int *inst_arr, int inst_num);
void spa_clus_result_display (spa_clus_list *coml, char *fn, int indi_num);

double spa_clus_inst_to_clus_dist (int dist_crit, gsl_matrix *orgi_dist_mat, spa_inst_list *rowl, int id);
double spa_clus_clus_to_clus_dist (int dist_crit, gsl_matrix *orgi_dist_mat, spa_clus_list *ca, spa_clus_list *cb);
double spa_clus_combine_intra_dist (gsl_matrix *orgi_dist_mat, spa_clus_list *ca, spa_clus_list *cb);
double spa_clus_avg_intra_clus_dist (spa_clus_param *clus, gsl_matrix *orgi_dist_mat, spa_clus_list *coll);
double spa_clus_avg_inter_clus_dist (spa_clus_param *clus, gsl_matrix *dist_mat, spa_clus_list *coll);
void spa_clus_intermediate_output(spa_clus_param *clus, char *fn, spa_clus_list *coll, int flag_details);

int spa_clus_load_individual_arr(char *fn, spa_clus_list **pindi_arr, int *pindi_num);
int spa_clus_release_individual_arr(spa_clus_list *indi_arr, int indi_num);

// get data
int spa_math_get_prev(gsl_vector *src, int loc, gsl_vector *dst);

// distribution
int spa_math_pdf_load_cfg (char *fn, spa_pdf_param **ppdf);
int spa_math_pdf_load_all_feature (spa_pdf_param *pdf, spa_pdf_container **pp, char *fea_lst);
int spa_math_pdf_load_all_model (spa_pdf_param *pdf, spa_pdf_container ***ppp, char *fname);
int spa_math_pdf_release_cfg(spa_pdf_param *pdf);
// Initialize and release pdf table
int spa_math_pdf_alloc (spa_pdf_container ***ppp, spa_pdf_param *pdf);
int spa_math_pdf_release (spa_pdf_container **pp, int mod_num);
int spa_math_pdf_vec_to_lin (spa_pdf_param *pdf, int *loc_vec);
int spa_math_pdf_fea_in_lin (spa_pdf_param *pdf, double *fea_vec);
// save model
int spa_math_pdf_save_all_model (spa_pdf_container **pp, int mod_num, char *fname);
int spa_math_pdf_train_all_model (char *mod_cfg_file, char *fea_lst, spa_pdf_param **ppdf, spa_pdf_container ***ppp);
#endif //_SPA_MATH_H
