////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    spec_dom.c: signal processing, mainly in spectral domain            //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#define GSL_RANGEHECK_OFF
//#define SPA_SPEC_FEA_DEBUG

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <float.h>
#include "fftw3.h"
#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"
#include "gsl/gsl_errno.h"
#include "gsl/gsl_blas.h"

#include "io_shell.h"
#include "simp_math.h"
#include "spec_dom.h"

static double logzero;
static double logten;
static FILE *gfp;
extern char msg[SPA_MAX_STR_LEN];

static char *pre_fea_kind[] = {"RESERVE2", "LPC", "RESERVE2",
	"RESERVE3", "RESERVE4", "RESERVE5", 
	"MFCC", "FBE", "RESERVE010", "USER", "RESERVE012", "RESERVE013", "SPEC", "ANULL"};

// FFT
static fftw_plan fft_plan;	// fft plan
static int fft_num;		// fft size
static int fft_half_num;	// half fft_num size
static double *fft_in;		// real input
static fftw_complex *fft_out;	// complex output

// DCT & IDCT
static fftw_plan dct_plan;	// dct plan
static int dct_num;		// dct_size
static int dct_half_num;	// half fft_num size
static double *dct_in;		// real input
static double *dct_out;		// real output
static double dct_scale;	// scale factor sqrt(2/N)

#ifdef SPA_SPEC_FEA_DEBUG
static FILE *gfp;
#endif

int spa_spec_feakind_parser(char *str)
{
	int i = -1;
	char *c, label[SPA_MAX_TAG_LEN];
	int withE, withD, withA, withT, withZ, with0, found;
	int size;
   
	withE = withD = withA = withT = withZ = with0 = 0;
	strcpy(label, str);
	size = strlen(label);
	c = label + size - 2;
	
	// backward searching
	while (size > 2 && *c == '_') {
		switch(*(c+1)) {
			case 'E':
				withE = 1;
				break;
			case 'D':
				withD = 1;
				break;
			case 'A':
				withA = 1;
				break;
			case 'Z':
				withZ = 1;
				break;
			case '0':
				with0 = 1;
				break;
			default:
				sprintf(msg, "unknown tag <%s>\n", str);
				spa_io_saybye(msg);
				break;
		}
		*c = '\0';
		size -= 2;
		c -= 2;
	}
	
	found = 0;
	do {
		c = pre_fea_kind[++i]; 
		if (strcmp(label, c) == 0) {
			found = 1;
			break;
		}
	} while (strcmp("ANULL", c) != 0);
	if (!found)
		return -1;
	
	if (withE)
		i |= SPA_FEA_HASENERGY;
	if (withD)
		i |= SPA_FEA_HASDELTA;
	if (withA)
		i |= SPA_FEA_HASACCS;
	if (withZ)
		i |= SPA_FEA_HASZEROM;
	if (with0)
		i |= SPA_FEA_HASZEROC;
	
	return i;
}

int spa_spec_load_cfg(char *fn, spa_spec_param** ppar)
{
	char ctag[SPA_MAX_TAG_LEN], str[SPA_MAX_TAG_LEN];	// content in the <...>
	int i, j, m, c;
	FILE *fp;

	spa_spec_param *par = (spa_spec_param *) calloc(sizeof(spa_spec_param), 1);
	par->fft_num	= 512;				// fft size (512)
	par->win_shape	= SPA_WIN_SHAPE_HAMMING;	// tappering window shape
	par->fb_shape	= SPA_NONE_FB;			// filter bank shape (SPA_MEL_TRI_FB)
	par->fb_num	= 26;				// number of filter banks (26)
	par->cep_num	= 12;				// number of cepstral coefficients (12)
	par->cep_lift_num = 22;				// number of cepstral lifter coefficients (22)
	par->sta_fea_num= 13;				// number of static features (13)
	par->der_win_num= 2;				// the window length for calculating the derivative features (2)
	par->acc_win_num= 2;				// the window length for calculating the accelerate features (2)
	par->use_log	= 1;				// use log/non-log spectrum t/f (true)
	par->use_pow	= 1;				// use power/magnitude spectrum: t/f (true)
	par->use_norm_eng = 1;				// use normalized energy t/f (true)
	par->strict_frm_time = 0;			// use strict frame timing t/f (false)
	par->fea_kind	= -1;				// feature kind code in the htk paramer table (null)
	
	par->pre_emph	= 0.97;				// pre-emphasis factor (0.97)
	par->smp_rate	= 16000.0;			// sampling rate in Hz
	par->frm_len_t	= 25.0;				// window length in ms (25)
	par->frm_step_t	= 10.0;				// window step in ms (10)
	par->fb_fmin	= 0;				// max frequecy of the filter bank (0)
	par->fb_fmax	= 0.5*par->smp_rate;		// max frequecy of the filter bank (fsr/2)
	par->sil_floor	= 50.0;				// log energy max to min ratio in dB (50.0)
	par->eng_scale	= 0.1;				// log energy scaling ratio
	par->filt_ls	= 0;				// low stop frequecy of the filter
	par->filt_lp	= 0;				// low pass frequecy of the filter
	par->filt_hp	= par->fb_fmax;			// high pass frequecy of the filter
	par->filt_hs	= par->fb_fmax;			// high stop frequecy of the filter

	fp = spa_io_fopen(fn, "rt");	
	while ( spa_io_get_next_tag(fp, ctag) == 1) {
		if ( strcmp(ctag, "FFT_NUM") == 0 )
			fscanf(fp, "%d", &par->fft_num);
		else if ( strcmp(ctag, "WIN_SHAPE") == 0 )
			fscanf(fp, "%d", &par->win_shape);
		else if ( strcmp(ctag, "FB_SHAPE") == 0 )
			fscanf(fp, "%d", &par->fb_shape);
		else if ( strcmp(ctag, "FB_NUM") == 0 )
			fscanf(fp, "%d", &par->fb_num);
		else if ( strcmp(ctag, "USE_POWER") == 0 )
			fscanf(fp, "%d", &par->use_pow);
		else if ( strcmp(ctag, "USE_LOG") == 0 )
			fscanf(fp, "%d", &par->use_log);
		else if ( strcmp(ctag, "CEP_NUM") == 0 )
			fscanf(fp, "%d", &par->cep_num);
		else if ( strcmp(ctag, "CEP_LIFT_NUM") == 0 )
			fscanf(fp, "%d", &par->cep_lift_num);
		else if ( strcmp(ctag, "STA_FEA_NUM") == 0 )
			fscanf(fp, "%d", &par->sta_fea_num);
		else if ( strcmp(ctag, "DER_WIN_NUM") == 0 )
			fscanf(fp, "%d", &par->der_win_num);
		else if ( strcmp(ctag, "ACC_WIN_NUM") == 0 )
			fscanf(fp, "%d", &par->der_win_num);
		else if ( strcmp(ctag, "FEA_KIND") == 0 ) {
			fscanf(fp, "%s", str);
			par->fea_kind = spa_spec_feakind_parser(str);
		}
		else if ( strcmp(ctag, "STRCT_FRM_TIME") == 0 )
			fscanf(fp, "%d", &par->strict_frm_time);
		else if ( strcmp(ctag, "SAMP_RATE") == 0 )
			fscanf(fp, "%d", &par->smp_rate);
		else if ( strcmp(ctag, "PRE_EMPH") == 0 )
			fscanf(fp, "%lf", &par->pre_emph);
		else if ( strcmp(ctag, "FRAME_LEN") == 0 )
			fscanf(fp, "%lf", &par->frm_len_t);
		else if ( strcmp(ctag, "FRAME_STEP") == 0 )
			fscanf(fp, "%lf", &par->frm_step_t);
		else if ( strcmp(ctag, "FB_FMIN") == 0 )
			fscanf(fp, "%lf", &par->fb_fmin);
		else if ( strcmp(ctag, "FB_FMAX") == 0 )
			fscanf(fp, "%lf", &par->fb_fmax);
		else if ( strcmp(ctag, "FILT_LOW_STOP") == 0 )
			fscanf(fp, "%lf", &par->filt_ls);
		else if ( strcmp(ctag, "FILT_LOW_PASS") == 0 )
			fscanf(fp, "%lf", &par->filt_lp);
		else if ( strcmp(ctag, "FILT_HIGH_STOP") == 0 )
			fscanf(fp, "%lf", &par->filt_hs);
		else if ( strcmp(ctag, "FILT_HIGH_PASS") == 0 )
			fscanf(fp, "%lf", &par->filt_hp);
		else if ( strcmp(ctag, "FB_FMAX") == 0 )
			fscanf(fp, "%lf", &par->fb_fmax);
		else {
			sprintf(msg, "\"%s\" has no matching tag.\n", ctag);
			GSL_ERROR(msg, GSL_EINVAL);
		}
	}
	par->fft_half_num = par->fft_num/2 + 1;			// half fft size = fft_num/2 + 1
	par->frm_len	= par->frm_len_t*1e-3*par->smp_rate;	// window length in samples (frm_len_t * 1e-3 * fsr)
	par->frm_step	= par->frm_step_t*1e-3*par->smp_rate;	// window step in samples (frm_step_t * 1e-3 * fsr)
	par->fb = NULL;
	par->fea_num = par->sta_fea_num;
	if (par->fea_kind & SPA_FEA_HASDELTA) {
		par->fea_num += par->sta_fea_num;
		if (par->fea_kind & SPA_FEA_HASACCS) {
			par->fea_num += par->sta_fea_num;
		}
	}
	spa_io_fclose(fp);
	
	*ppar = par;
	return GSL_SUCCESS;
}

/////////////////////////////////////////////////////////////////////////////
//
//                      FFT
//
/////////////////////////////////////////////////////////////////////////////
void spa_spec_fft_release()
{
	if (fft_in != NULL) {
		fftw_destroy_plan(fft_plan);
		free(fft_in);
		fftw_free(fft_out);
		fft_in = NULL;
		fft_out = NULL;
	}
}

void spa_spec_fft_init(int in_fft)
{
	fft_num = in_fft;
	fft_half_num = in_fft/2 + 1;
	spa_spec_fft_release();
	
	fft_in = (double *) malloc(sizeof(double) * fft_num);
	fft_out = (fftw_complex *) malloc(sizeof(fftw_complex) * fft_num);
	fft_plan = fftw_plan_dft_r2c_1d(fft_num, fft_in, fft_out, FFTW_MEASURE);
}

// Calculate spectrum power, magnitude, log magnitude of the real signal
//   the input is in the fft_in_r
void spa_spec_fft_pow(gsl_vector *in, gsl_vector *out)
{
	int i, k;
	double v;
	for (i = 0; i < in->size; i++)
		fft_in[i] = gsl_vector_get(in, i);
	for (i = in->size; i < fft_num; i++)
		fft_in[i] = 0.0;
	
	fftw_execute(fft_plan);
	v = fft_out[0][0]*fft_out[0][0]; // DC component
	gsl_vector_set(out, 0, v);
	for (k = 1; k < fft_half_num; k++) {  // (k <= N/2 rounded up)
		v = fft_out[k][0]*fft_out[k][0] + fft_out[k][1]*fft_out[k][1];
		gsl_vector_set(out, k, v);
	}
}

void spa_spec_fft_mag(gsl_vector *in, gsl_vector *out)
{
	int i, k;
	double v;
	for (i = 0; i < in->size; i++)
		fft_in[i] = gsl_vector_get(in, i);
	for (i = in->size; i < fft_num; i++)
		fft_in[i] = 0.0;
	
	fftw_execute(fft_plan);
	v = fabs(fft_out[0][0]); // DC component
	gsl_vector_set(out, 0, v);
	for (k = 1; k < fft_half_num; k++) { // (k <= N/2 rounded up)
		v = sqrt(fft_out[k][0]*fft_out[k][0] + fft_out[k][1]*fft_out[k][1]);
		gsl_vector_set(out, k, v);
	}
}

// get spectrum
void spa_get_frm_spec (gsl_vector *sph_frm, spa_spec_param *par, gsl_vector *spec_amp, int *log_flag)
{
	// Hamming Windowing s = s*frm_win
	gsl_vector_mul(sph_frm, par->frm_win);
	
	// FFT
	if (par->use_pow)
		spa_spec_fft_pow(sph_frm, spec_amp);
	else
		spa_spec_fft_mag(sph_frm, spec_amp);
}

// linear -> log
void spa_spec_vector_linear2log(gsl_vector *s)
{
	int k;
	double d;
	for (k = 0; k < s->size; k++) {
		d = gsl_vector_get(s, k);
		d = d > SPA_DSMALL ? log(d) : logzero;
		gsl_vector_set(s, k, d);
	}
}

// suppose power spectrum is used
void spa_spec_vector_log2db(gsl_vector *s, int flag_power)
{
	int k;
	double d;
	for (k = 0; k < s->size; k++) {
		d = gsl_vector_get(s, k);
		if (flag_power)
			d = 10.0 * (d / logten);
		else
			d = 20.0 * (d / logten);
		gsl_vector_set(s, k, d);
	}
}

// as energy normalization in HTK
void spa_spec_norm_loge(gsl_vector *eng_fea, spa_spec_param *par)
{
	int i;
	double d, emax, emin;
	emax = gsl_vector_max(eng_fea);
	emin = emax - 0.1*(par->sil_floor * log(10.0));
	for (i = 0; i < eng_fea->size; i++) {
		d = gsl_vector_get(eng_fea, i);
		if (d <= emin)
			gsl_vector_set(eng_fea, i, emin);
		else {
			d = 1.0 - (emax -d)*par->eng_scale;
			gsl_vector_set(eng_fea, i, d);
		}
	}	
}

/////////////////////////////////////////////////////////////////////////////
//
//                      DCT - II
//
/////////////////////////////////////////////////////////////////////////////
void spa_spec_dct_release()
{
	if (dct_in != NULL) {
		fftw_destroy_plan(dct_plan);
		free(dct_in);
		free(dct_out);
		dct_in = NULL;
		dct_out = NULL;
	}
}

void spa_spec_dct_init(int in_dct)
{
	spa_spec_dct_release();
	dct_num = in_dct;
	dct_half_num = in_dct/2 + 1;	
	dct_in = (double *) malloc(sizeof(double) * dct_num);
	dct_out = (double *) malloc(sizeof(double) * dct_num);
	dct_plan = fftw_plan_r2r_1d(dct_num, dct_in, dct_out, FFTW_REDFT10, FFTW_MEASURE); // FFTW_MEASURE will take more time in initialization
	// the reason why there is a factor of 0.5 is that FFTW's DCT has a factor of 2 ( see FFTW manual 4.8.3)
	dct_scale = 0.5*sqrt(2.0/dct_num);
}

// results stored in dct_out
void spa_spec_dct(gsl_vector *in, gsl_vector *out)
{
	int i;
	for (i = 0; i < in->size; i++)
		dct_in[i] = gsl_vector_get(in, i);
	fftw_execute(dct_plan);
	for (i = 0; i < out->size; i++)
		gsl_vector_set(out, i, dct_scale*dct_out[i]);
}

/////////////////////////////////////////////////////////////////////////////
//
//                     Widowing
//
/////////////////////////////////////////////////////////////////////////////

gsl_vector *spa_spec_hamming_win(int win_num)
{
	int i;
	double d;
	int win_half_num = (win_num + 1)/2;
	gsl_vector *vec = gsl_vector_calloc(win_num);
	for (i = 0; i < win_half_num; i++) {
		d = 0.54 - 0.46*cos(2.0*M_PI*i/(win_num-1));
		gsl_vector_set(vec, i, d);
		gsl_vector_set(vec, win_num - 1 - i, d);
	}
	return vec;
}

void spa_spec_win_release(gsl_vector *frm_win)
{
	gsl_vector_free(frm_win);
}

/////////////////////////////////////////////////////////////////////////////
//
//                     Filterbank
//
/////////////////////////////////////////////////////////////////////////////
double spa_spec_mel (double f)
{
	return (2595.0 * log10(1.0 + f / 700.0));
}

double spa_spec_invmel (double f)
{
	return (700.0 * (pow(10, f / 2595.0) - 1));
}

///////////////////////////////////////////////////////////////////////////////
// Given fb_min, fb_max, and fb_num, obtain mel frequecy bounds
// input:
//	par	- params 
// output:
//	pfb_bound - the pointer to the filter bank boundaries
//			note the size of the *pfb_bound should be fb_num + 2
//			(including the fb_fmin and fb_fmax)
void spa_spec_mel_freq_bound (spa_spec_param *par, gsl_vector **pfb_bound)
{
	int i;
	if ( par->fb_fmin < 0 || par->fb_fmin >= par->fb_fmax || par->fb_fmax > par->smp_rate)
		spa_io_saybye ("spa_spec_mel_freq_bound: Check fb_fmin, fb_fmax, sampling rate");
	gsl_vector *fb_bound = gsl_vector_calloc(par->fb_num + 2);
	
	double mel_fmin, mel_fmax, mel_fspac, mel_f;
	// FB ranges
	mel_fmin = spa_spec_mel(par->fb_fmin); 
	mel_fmax = spa_spec_mel(par->fb_fmax);
	mel_fspac = (mel_fmax - mel_fmin)/(par->fb_num + 1);
	
	gsl_vector_set(fb_bound, 0, par->fb_fmin);
	for (i = 1; i < par->fb_num + 1; i++) {
		mel_f = mel_fmin + i*mel_fspac;
		gsl_vector_set(fb_bound, i, spa_spec_invmel(mel_f));
	}
	gsl_vector_set(fb_bound, par->fb_num+1, par->fb_fmax);
	*pfb_bound = fb_bound;
}

///////////////////////////////////////////////////////////////////////////////
// given filtbank frequecy bounds, obtain filtbank matrix
// input:
//	fb_bound - filtbank boundaries
// in/output:
//	par	- params (output to par->fb)
//		the row is the fb_num, col is the fft_half_size
int spa_spec_fb_matrix (spa_spec_param *par, gsl_vector *fb_bound)
{
	int i, j, begin_pt, l_mid_pt, r_mid_pt, end_pt;
	double f, pt_space, amp;
	
	gsl_matrix *fb = par->fb;
	
	if (fb_bound->size != par->fb_num + 2) {
		sprintf(msg, "fb_bound->size (%d) should be (%d)", fb_bound->size, par->fb_num + 2);
		spa_io_saybye(msg);
	}
		
	// set filter bank
	pt_space = (double)par->smp_rate/par->fft_num;
	gsl_matrix_set_zero(fb);
	for (i = 0; i < par->fb_num; i++) {
		begin_pt = ceil(gsl_vector_get(fb_bound, i) / pt_space);
		l_mid_pt = floor(gsl_vector_get(fb_bound, i+1) / pt_space);
		r_mid_pt = ceil(gsl_vector_get(fb_bound, i+1) / pt_space);
		end_pt   = floor(gsl_vector_get(fb_bound, i+2) / pt_space);
		if (end_pt >= par->fft_half_num)
			end_pt = par->fft_half_num - 1;
		for (j = begin_pt; j <= l_mid_pt; j++) {
			amp = (j * pt_space - gsl_vector_get(fb_bound, i))/(gsl_vector_get(fb_bound, i+1) - gsl_vector_get(fb_bound, i));
			gsl_matrix_set(fb, i, j, amp);
		}
		for (j = r_mid_pt; j <= end_pt; j++) {
			amp = (gsl_vector_get(fb_bound, i+2) - j * pt_space)/(gsl_vector_get(fb_bound, i+2) - gsl_vector_get(fb_bound, i+1));
			gsl_matrix_set(fb, i, j, amp);
		}
	}
	
	return GSL_SUCCESS;
}

// obtain mel frequecy filter bank
void spa_spec_melfb(spa_spec_param *par)
{
	gsl_vector *fb_bound;
	spa_spec_mel_freq_bound (par, &fb_bound);
	spa_spec_fb_matrix (par, fb_bound);
	gsl_vector_free(fb_bound);
}

// Spectral processing initialization
void spa_spec_init(char *cfg_fn, spa_spec_param **ppar)
{	
	int i, win_num, win_half_num;
	double d;
	logzero = SPA_LOGZERO;
	logten = log(10.0);
	spa_spec_param *par;
	
	// load in cfg
	spa_spec_load_cfg(cfg_fn, &par);
	*ppar = par;
	
	spa_spec_fft_init(par->fft_num);
	spa_spec_dct_init(par->fb_num);
	
	// Create win_num Hamming window
	win_num = par->frm_len;
	win_half_num = (win_num + 1)/2;
	par->frm_win = gsl_vector_calloc(win_num);
	for (i = 0; i < win_half_num; i++){
		switch (par->win_shape) {
			case SPA_WIN_SHAPE_HAMMING:
				d = 0.54 - 0.46*cos(2.0*M_PI*i/(win_num-1));
				break;
			case SPA_WIN_SHAPE_HANNING:
				d = 0.5 - 0.5*cos(2.0*M_PI*i/(win_num-1));
				break;
			default:
				break;
				// case SPA_WIN_SHAPE_BLACKMAN: ...				
		}
		gsl_vector_set(par->frm_win, i, d);
		gsl_vector_set(par->frm_win, win_num - 1 - i, d);
	}
	
	// init a M x nfft/2 filter bank matrix, M: # of FB. each row is a filter
	if (par->fb_shape != SPA_NONE_FB) {
		par->fb = gsl_matrix_calloc(par->fb_num, par->fft_half_num);
		if (par->fb_shape == SPA_MEL_TRI_FB) {
			spa_spec_melfb(par);
		}
	}
}

void spa_spec_release(spa_spec_param *par)
{
	spa_spec_fft_release();
	spa_spec_dct_release();
	gsl_vector_free(par->frm_win);	
	if (par->fb != NULL) {
		gsl_matrix_free(par->fb);
	}
	free(par);
}

/////////////////////////////////////////////////////////////////////////////
//
//                     Noise Processing
//
/////////////////////////////////////////////////////////////////////////////

// Framing. Two configuration for the ith frame
void spa_spec_get_frm (gsl_vector *sph, gsl_vector *sph_frm, int st)
{
	int i, j, et, seg;
	double d;
	
	seg = sph_frm->size;
	et = st + seg;
	if (st >= 0 && st < sph->size && et > 0 && et <= sph->size) {
		// inside range
		for (j = 0; j < seg; j++) {
			d = gsl_vector_get(sph, st + j);
			gsl_vector_set(sph_frm, j, d);
		}
	}
	else if (st < 0 && et > 0 && et <= sph->size){
		// part in part out
		for (j = 0; j < -st; j++) {
			gsl_vector_set(sph_frm, j, 0.0);
		}
		for (j = -st; j < seg; j++) {
			d = gsl_vector_get(sph, st + j);
			gsl_vector_set(sph_frm, j, d);
		}
	}
	else if (st >= 0 && st < sph->size && et > sph->size){
		// part in part out
		for (j = 0; j < sph->size - st; j++) {
			d = gsl_vector_get(sph, st + j);
			gsl_vector_set(sph_frm, j, d);
		}
		for (sph->size - st; j < seg; j++) {
			gsl_vector_set(sph_frm, j, 0.0);
		}
	}
	else {
		// outside range
		//if (st >= sph->size || et <= 0) {
		gsl_vector_set_zero(sph_frm);
	}
}

// estimate the spectrum noise, take the average of low energy frame (5%) as estimated noise frames
void spa_estimate_noise_spectrum (gsl_vector *sph, gsl_vector *sph_frm, int frm_num, int st_org, spa_spec_param *par, gsl_vector *enos_amp)
{
	int frm_throw = 1;		// throw initial and final frames because of the possible out of range starting index
	int frm_noise = (int) (0.05 * frm_num);		// num of frames for estimating noise
	double d;
	int *idx, i, j, st, et, smth_len, spec_smth_len = 500;
	int frm_left = frm_num - 2 * frm_throw;
	if (frm_left <= 0) {
		printf ("The waveform should have more than %d frames\n.", 2*frm_throw);
		exit(-2001);
	}
	idx = (int *) calloc(frm_noise, sizeof(int));
	gsl_vector *eng		= gsl_vector_calloc(frm_left);
	gsl_vector *spec_amp	= gsl_vector_calloc(par->fft_half_num); // spectrum of the framed speech
	gsl_vector *onos_amp	= gsl_vector_calloc(par->fft_half_num); // spectrum of the unsmoothed noise spectrum
	for (i = frm_throw; i < frm_num - frm_throw; i++) {
		st = st_org + i * par->frm_step;
		spa_spec_get_frm(sph, sph_frm, st);
		gsl_blas_ddot(sph_frm, sph_frm, &d);	// norm^2 = x^T x
		gsl_vector_set(eng, i - frm_throw, d);
	}
	
	// the frames with lowest energy
	gsl_sort_vector_smallest_index(idx, frm_noise, eng);
	
	gsl_vector_set_zero(onos_amp);
	for (i = 0; i < frm_noise; i++) {
		j = idx[i] + frm_throw;
		st = st_org + j * par->frm_step;
		spa_spec_get_frm(sph, sph_frm, st);
		spa_get_frm_spec(sph_frm, par, spec_amp, SPA_LINEAR_SPEC);
		if (i > 0) 
			gsl_vector_add(onos_amp, spec_amp);
	}
	gsl_vector_scale(onos_amp, 1.0 / (double) frm_noise);
	
	// smooth noise spectrum by using a window
	for (i = 0; i < par->fft_half_num; i++) {
		st = i - spec_smth_len;
		et = i + spec_smth_len;
		st = st < 0 ? 0 : st;
		et = et > par->fft_half_num - 1 ? par->fft_half_num - 1 : et;
		smth_len = et - st + 1;
		d = 0.0;
		for (j = st; j <= et; j++) {
			d += gsl_vector_get(onos_amp, j);
		}
		d /= smth_len;
		gsl_vector_set(enos_amp, i, d);
	}
	
	free(idx);
	gsl_vector_free(eng);
	gsl_vector_free(spec_amp);
	gsl_vector_free(onos_amp);	
	return;
}


/////////////////////////////////////////////////////////////////////////////
//
//                      Feature Extraction
//
/////////////////////////////////////////////////////////////////////////////

// Preemphasis: s := s - a * s-1
void spa_spec_preemphasis (gsl_vector *s, double a)
{
	int i;
	double d;
	for (i = s->size - 1; i > 0; i--) {
		d = gsl_vector_get(s, i);
		d -= gsl_vector_get(s, i-1) * a;
		gsl_vector_set(s, i, d);
	}
	d = gsl_vector_get(s, 0);
	gsl_vector_set(s, 0, d*(1 - a));
}

///////////////////////////////////////////////////////////////////////////////
// Spectral liftering and trucate to get cepstral features
// input
//    in - the DCT of the filter bank's output, note in->data[0] is C0
//    out - the lifted cepstral coefficients C1 - Cn is stored
void spa_spec_cep_lift (gsl_vector *in, gsl_vector *out, int cep_lift_num, int cep_num)
{
	int i;
	double d;
	// if out->size < in->size, the cep coefficients are truncated
	double amp = 0.5 * cep_lift_num;
	double fac = M_PI / (double) cep_lift_num;
	// lift c(1) - c(cep_num)
	for (i = 1; i <= cep_num; i++) {
		d = gsl_vector_get(in, i);
		d *= 1 + amp*sin(fac*i);
		gsl_vector_set(out, i-1, d);
	}
}

///////////////////////////////////////////////////////////////////////////////
// create a trapezoidal filter in frequecy domain
// max 1, min 0,  constant between two pass freqs linear between pass and stop freqs
void spa_spec_create_filt (spa_spec_param *par, gsl_vector *filt)
{
	int i;
	double d;
	double ls = par->filt_ls * par->fft_num / par->smp_rate;	// l - low, h - high, s - stop, p - pass
	double hs = par->filt_hs * par->fft_num / par->smp_rate;
	double lp = par->filt_lp * par->fft_num / par->smp_rate;
	double hp = par->filt_hp * par->fft_num / par->smp_rate;
	
	for (i = 0; i < filt->size; i++) {
		if ( i < ls) {
			d = 0.0;
		}
		else if ( i >= ls && i < lp  ) {
			d = (i - ls) / (lp - ls);			
		}
		else if ( i >= lp && i < hp ) {
			d = 1.0;
		}
		else if ( i >= hp && i < hs ) {
			d = (hs - i) / (hs - hp);
		}
		else {
			d = 0.0;
		}
		gsl_vector_set(filt, i, d);
	}
}

///////////////////////////////////////////////////////////////////////////////
// spectral substraction s = min{s-n, 0}
void spa_spec_spec_sub (gsl_vector *s, gsl_vector *n)
{
	int i;
	double d;
	if (s->size != n->size) {
		printf("spa_spec_spec_sub: vector size mismatch.\n");
		exit(-1);
	}
	gsl_vector_sub(s, n);
	for (i = 0; i < s->size; i++) {
		d = gsl_vector_get(s, i);
		if (d < 0) {
			gsl_vector_set(s, i, 0.0);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// get the feature frame number
void spa_spec_fea_frm_num (gsl_vector *sph, spa_spec_param *par, int *frm_num, int *st)
{
	// Framming
	if (!par->strict_frm_time) {
		// 1. normal: start at the (i-1)*d, duration: [(i-1)*d (i-1)*d+L-1]
		*frm_num = ceil( (sph->size - par->frm_len)/par->frm_step + 0.5 );
		*st = 0;
	}
	else {	
		// 2. strict timing: center on the (i-0.5)*d), duration: [(i-0.5)*d-L/2 (i-1)*d+L/2-1]
		*frm_num = ceil( sph->size/par->frm_step + 0.5 );
		*st = (int) (0.5*par->frm_step - 0.5*par->frm_len);
	}
}

///////////////////////////////////////////////////////////////////////////////
// speech in, spectral magnitude output
// input:
//	sph, par	- Preemphasised
// output:
//	pspec_mat	- the pointer to spec mag matrix (frm_num x fft_half_num)
void spa_spec_utt_sph2mag (gsl_vector *sph, spa_spec_param *par, gsl_matrix **pspec_mat)
{
	int i, j, frm_num, st;
	double d;
	gsl_vector_view spec_vec;
	gsl_matrix *spec_mat = NULL;
	gsl_vector *sph_frm = gsl_vector_calloc(par->frm_len);
	
	// Framming
	spa_spec_fea_frm_num (sph, par, &frm_num, &st);
	spec_mat = gsl_matrix_calloc(frm_num, par->fft_half_num);
	
	// spectral magnitude
	for (i = 0; i < frm_num; i++) {
		spa_spec_get_frm(sph, sph_frm, st);
		st += par->frm_step;
		
		// Hamming Windowing s = s*frm_win
		gsl_vector_mul(sph_frm, par->frm_win);
	
		// sph_frm as input, feature as output
		spec_vec = gsl_matrix_row(spec_mat, i);
		
		// FFT
		if (par->use_pow)
			spa_spec_fft_pow(sph_frm, &spec_vec.vector);
		else
			spa_spec_fft_mag(sph_frm, &spec_vec.vector);
#ifdef SPA_SPEC_FEA_DEBUG
		gfp = spa_io_fopen("debug.txt", "wt");
		for (j = 0; j < spec_vec.vector.size; j++)
			fprintf(gfp, "%e\n", spec_vec.vector.data[j]);
		spa_io_fclose(gfp);
#endif
	}

	gsl_vector_free(sph_frm);
	*pspec_mat = spec_mat;
}

///////////////////////////////////////////////////////////////////////////////
// speech in, raw energy output
// input:
//	sph, par	- preemphasised
// output:
//	pleng_mat	- the pointer to the log engergy matrix
void spa_spec_utt_sph2eng (gsl_vector *sph, spa_spec_param *par, gsl_matrix **pleng_mat)
{
	int i, j, frm_num, st;
	double d, raw_eng;
	gsl_vector_view eng_vec;
	gsl_matrix *leng_mat = NULL;
	gsl_vector *sph_frm = gsl_vector_calloc(par->frm_len);
	
	// Framming
	spa_spec_fea_frm_num (sph, par, &frm_num, &st);	
	leng_mat = gsl_matrix_calloc(frm_num, 1);
	
	for (i = 0; i < frm_num; i++) {
		spa_spec_get_frm(sph, sph_frm, st);
		st += par->frm_step;
		
		// log engergy
		raw_eng = gsl_blas_dnrm2(sph_frm);
		d = raw_eng > SPA_DSMALL ? log(raw_eng) : SPA_LOGZERO;
		gsl_matrix_set(leng_mat, i, 0, d);
	}
	
	// normalize energy, suppose the energy is store at the last dimension
	if (par->use_norm_eng && (par->fea_kind & SPA_FEA_HASENERGY)) {
		eng_vec = gsl_matrix_column(leng_mat, 1);
		spa_spec_norm_loge(&eng_vec.vector, par);
	}

	gsl_vector_free(sph_frm);
	*pleng_mat = leng_mat;
}

///////////////////////////////////////////////////////////////////////////////
// speech in, LPC analysis results output
// input:
//	sph, par
// output:
//	plpcc_mat	- the pointer to lpcc feature matrix (frm_num x sta_fea_num)
//	pcorr_mat	- the pointer to correlation matrix from LPC (frm_num x (sta_fea_num + 1))
//	presi_mat	- the pointer to residual matrix from LPC (frm_num x 1)
void spa_spec_utt_sph2lpc (gsl_vector *sph, spa_spec_param *par,
			   gsl_matrix **plpcc_mat, gsl_matrix **pcorr_mat,
			   gsl_matrix **presi_mat)
{
	int i, j, frm_num, st;
	double d, raw_eng;
	gsl_vector_view spec_vec, corr_vec, fea_vec;
	gsl_matrix *spec_mat = NULL, *corr_mat = NULL, *resi_mat = NULL,  *leng_mat = NULL, *lpcc_mat = NULL;
	spa_htk_header *header = calloc(1, sizeof(spa_htk_header));
	gsl_vector *sph_frm = gsl_vector_calloc(par->frm_len);
	gsl_vector *lpc_vec = gsl_vector_calloc(par->sta_fea_num + 1);
	
	// Framming
	spa_spec_fea_frm_num (sph, par, &frm_num, &st);
	
	// note the corr_mat is frm_num x p+1, p is the lpc_order, or sta_fea_num
	lpcc_mat = gsl_matrix_calloc(frm_num, par->sta_fea_num);	
	corr_mat = gsl_matrix_calloc(frm_num, par->sta_fea_num + 1);
	resi_mat = gsl_matrix_calloc(frm_num, 1);
	
	// LPC
	for (i = 0; i < frm_num; i++) {
		spa_spec_get_frm(sph, sph_frm, st);
		st += par->frm_step;
		fea_vec = gsl_matrix_row(lpcc_mat, i);
		corr_vec = gsl_matrix_row(corr_mat, i);
		
		// Hamming Windowing s = s*frm_win
		gsl_vector_mul(sph_frm, par->frm_win);
		
		// autocorrelation: R (i)
		spa_math_acorr(sph_frm, &corr_vec.vector);
		// LPC Analysis (Levinson - Durbin)
		spa_math_lpc (&corr_vec.vector, &fea_vec.vector);
		// calculate the aRa'
		// append LPC to [-1.0 a1 a2 ... ap]
		gsl_vector_set(lpc_vec, 0, -1.0);
		for (j = 0; j < par->sta_fea_num; j++) {
			d = gsl_vector_get(&fea_vec.vector, j);
			gsl_vector_set(lpc_vec, j+1, d);
		}
		d = spa_math_ara(lpc_vec, &corr_vec.vector);
		gsl_matrix_set(resi_mat, i, 0, d);
	}
	
	gsl_vector_free(sph_frm);
	gsl_vector_free(lpc_vec);
	
	*plpcc_mat = lpcc_mat;
	*pcorr_mat = corr_mat;
	*presi_mat = resi_mat;
}

///////////////////////////////////////////////////////////////////////////////
// spectral magnitude in, filter bank energy output
// input:
//	par
//	spec_mat	- spec mag matrix (frm_num x fft_half_num)
// output:
//	pfbe_mat	- filter bank engergy matrix (frm_num x par->fb_num)
void spa_spec_utt_mag2fbe (spa_spec_param *par, gsl_matrix *spec_mat, gsl_matrix **pfbe_mat)
{
	int i, j, st;
	double raw_eng, d;
	char cor_file[SPA_MAX_STR_LEN], res_file[SPA_MAX_STR_LEN], *pc;
	gsl_vector_view spec_vec, fbe_vec;
	
	int frm_num = spec_mat->size1;
	gsl_matrix *fbe_mat	= gsl_matrix_calloc(frm_num, par->fb_num);
	
	for (i = 0; i < frm_num; i++) {
		// spec mag as input, feature as output
		spec_vec = gsl_matrix_row(spec_mat, i);
		fbe_vec = gsl_matrix_row(fbe_mat, i);
		
		// Pass Filter Bank, e = fb x s
		gsl_blas_dgemv(CblasNoTrans, 1.0, par->fb, &spec_vec.vector, 0.0, &fbe_vec.vector);

		// Log FB output
		if (par->use_log)
			spa_spec_vector_linear2log(&fbe_vec.vector);

#ifdef SPA_SPEC_FEA_DEBUG
		gfp = spa_io_fopen("debug.txt", "wt");
		for (j = 0; j < spec_vec.vector.size; j++)
			fprintf(gfp, "%e\n", spec_vec.vector.data[j]);
		spa_io_fclose(gfp);
		
		gfp = spa_io_fopen("debug.txt", "wt");
		for (j = 0; j < fbe_vec.vector.size; j++)
			fprintf(gfp, "%e\n", fbe_vec.vector.data[j]);
		spa_io_fclose(gfp);
#endif
	}
	*pfbe_mat = fbe_mat;
}

///////////////////////////////////////////////////////////////////////////////
// filter bank engergy magnitude in, cepstral feature output
// input:
//	par
//	fbe_mat	- filter bank engergy
// output:
//	pcep_mat	- the pointer to the cepstral feature matrix
//	pczo_mat	- the pointer to the c0 feature matrix
void spa_spec_utt_fbe2cep (spa_spec_param *par, gsl_matrix *fbe_mat, 
			   gsl_matrix **pcep_mat, gsl_matrix **pczo_mat)
{
	int i, j, st;
	double raw_eng, d;
	char cor_file[SPA_MAX_STR_LEN], res_file[SPA_MAX_STR_LEN], *pc;
	gsl_vector *spec_amp, *fb_eng;
	gsl_vector_view spec_vec, cep_vec, fbe_vec;
	gsl_matrix *czo_mat;
	
	int frm_num = fbe_mat->size1;
	gsl_vector *fea_dct	= gsl_vector_calloc(par->fb_num);
	gsl_matrix *cep_mat	= gsl_matrix_calloc(frm_num, par->cep_num);
	if (par->fea_kind & SPA_FEA_HASZEROC) {
		czo_mat	= gsl_matrix_calloc(frm_num, 1);
	}
	
	for (i = 0; i < frm_num; i++) {
		fbe_vec = gsl_matrix_row(fbe_mat, i);
		cep_vec = gsl_matrix_row(cep_mat, i);		
		
		// DCT
		spa_spec_dct(&fbe_vec.vector, fea_dct);

		// Cepstral liftering and trucate to get cepstral features
		spa_spec_cep_lift(fea_dct, &cep_vec.vector, par->cep_lift_num, par->cep_num);
	
		// C0
		if (par->fea_kind & SPA_FEA_HASZEROC) {
			d = gsl_vector_get(fea_dct, 0);
			gsl_matrix_set(czo_mat, i, 0, d);
		}
	}
	gsl_vector_free(fea_dct);
	
	*pcep_mat = cep_mat;
	if (par->fea_kind & SPA_FEA_HASZEROC)
		*pczo_mat = czo_mat;
	else 
		*pczo_mat = NULL;
}


///////////////////////////////////////////////////////////////////////////////
// append feature with C0 or logE, static feature output
// input:
//	par
//	fea_mat		- the org feature matrix (either cep_mat or lpcc_mat)
//	czo_mat		- the c0 feature matrix
//	leng_mat	- the log engergy matrix
// output:
//	psta_mat	- the pointer to the static feature matrix
void spa_spec_append_stafea (spa_spec_param *par, gsl_matrix *fea_mat, gsl_matrix *czo_mat,
				gsl_matrix *leng_mat, gsl_matrix **psta_mat)
{
	int i, j;
	int frm_num = fea_mat->size1;
	int sub_num = fea_mat->size2;	// the old dim
	int sta_fea_num = sub_num;	// the new dim
	double *mean;
	double d;
	
	if (par->fea_kind & SPA_FEA_HASZEROC) {
		sta_fea_num++;
	}	
	if (par->fea_kind & SPA_FEA_HASENERGY) {
		sta_fea_num++;
	}
	if (sta_fea_num != par->sta_fea_num) {
		spa_io_saybye("Check the static feature dimension.\n");
	}
	
	gsl_matrix *sta_mat = gsl_matrix_alloc(frm_num, sta_fea_num);
	for (i = 0; i < frm_num; i++) {
		for (j = 0; j < sub_num; j++) {
			d = gsl_matrix_get(fea_mat, i, j);
			gsl_matrix_set(sta_mat, i, j, d);
		}
		if (par->fea_kind & SPA_FEA_HASZEROC) {
			d = gsl_matrix_get(czo_mat, i, 0);
			gsl_matrix_set(sta_mat, i, j++, d);
		}	
		if (par->fea_kind & SPA_FEA_HASENERGY) {
			d = gsl_matrix_get(leng_mat, i, 0);
			gsl_matrix_set(sta_mat, i, j++, d);
		}
	}
	
	if (par->fea_kind & SPA_FEA_HASZEROM) {
		mean = calloc(sta_fea_num, sizeof(double));
		spa_io_zero_vec(mean, sta_fea_num);
		for (j = 0; j < sta_fea_num; j++) {
			for (i = 0; i < frm_num; i++) {
				mean[j] +=  gsl_matrix_get(sta_mat, i, j);
			}
			mean[j] /= (double) frm_num;
		}
		for (i = 0; i < frm_num; i++) {
			for (j = 0; j < sta_fea_num; j++) {
				d = gsl_matrix_get(sta_mat, i, j) - mean[j];
				gsl_matrix_set(sta_mat, i, j, d);
			}
		}
		free(mean);
	}
	*psta_mat = sta_mat;
}

///////////////////////////////////////////////////////////////////////////////
// static feature to derivative features
// \sum_a a*[sta(i+a) - sta(i-a)] / 2 * \sum a^2
void spa_spec_sta2der (gsl_matrix *sta_mat, gsl_matrix **pder_mat, int der_fea_num)
{
	int i, j, k, l, r;
	int frm_num = sta_mat->size1;
	int sub_num = sta_mat->size2;
	double fac, sum, d;
	gsl_vector_view *sta_vec, *der_vec;
	
	gsl_matrix *der_mat = gsl_matrix_alloc(frm_num, sub_num);
	
	fac = 0.0;
	for (k = 1; k <= der_fea_num; k++) {
		fac += (double) k * (double) k;
	}
	fac *= 2.0;
	
	for (i = 0; i < frm_num; i++) {
		for (j = 0; j < sub_num; j++) {		
			sum = 0.0;
			for (k = 1; k <= der_fea_num; k++) {
				l = i - k < 0 ? 0 : i - k;
				r = i + k >= frm_num ? frm_num - 1: i + k;
				d = gsl_matrix_get(sta_mat, r, j) - gsl_matrix_get(sta_mat, l, j);
				sum += (double) k * d;
			}
			sum /= fac;
			gsl_matrix_set(der_mat, i, j, sum);
		}
	}
	*pder_mat = der_mat;
}

///////////////////////////////////////////////////////////////////////////////
// save lpc based features
void spa_spec_save_lpcfea (spa_spec_param *par, gsl_matrix *lpcc_mat, gsl_matrix *corr_mat,
			   gsl_matrix *resi_mat, char *fea_file)
{
	int i, j, frm_num, st;
	double raw_eng, d;
	char corr_file[SPA_MAX_STR_LEN], resi_file[SPA_MAX_STR_LEN], *pc;
	spa_htk_header htk_head, add_htk_head;
	
	frm_num = lpcc_mat->size1;
	if (lpcc_mat->size2 != par->sta_fea_num) {
		spa_io_saybye("The static feature's dimension does not match the configuration.\n");
	}
	
	htk_head.nSamples = frm_num;		// frame num
	htk_head.sampPeriod = par->frm_step_t * 1e4;	// in 100 ns
	htk_head.sampSize = sizeof(float) * par->sta_fea_num;
	htk_head.sampKind = par->fea_kind;
	
	// write down feature
	spa_io_write_htk_feamat(fea_file, &htk_head, lpcc_mat);	
	
	// create two same name file as the feature but the suffixes are .cor .res
	memcpy(&add_htk_head, &htk_head, sizeof(spa_htk_header));
	add_htk_head.sampKind = SPA_FEA_USER;
	
	if (corr_mat != NULL) {
		if (corr_mat->size1 != frm_num || corr_mat->size2 != par->sta_fea_num + 1) {
			spa_io_saybye("Check the correlation matrix's sizes.\n");
		}
		add_htk_head.sampSize = sizeof(float) * (par->sta_fea_num + 1);
		
		spa_io_replace_suffix(corr_file, fea_file, "cor");
		spa_io_write_htk_feamat(corr_file, &add_htk_head, corr_mat);
	}
	
	if (resi_mat != NULL) {
		if (resi_mat->size1 != frm_num || resi_mat->size2 != par->sta_fea_num + 1) {
			spa_io_saybye("Check the correlation matrix's sizes.\n");
		}
		add_htk_head.sampSize = sizeof(float);
		spa_io_replace_suffix(resi_file, fea_file, "res");
		spa_io_write_htk_feamat(resi_file, &add_htk_head, resi_mat);
	}
}

///////////////////////////////////////////////////////////////////////////////
// save static, derivative and accelerate features
void spa_spec_save_sdafea (spa_spec_param *par, gsl_matrix *sta_mat, gsl_matrix *der_mat,
			   gsl_matrix *acc_mat, char *fea_file)
{
	int i, j, frm_num, tot_fea_num;
	spa_htk_header htk_head;
	gsl_vector_view gv;
	
	frm_num = sta_mat->size1;
	if (sta_mat != NULL) {
		if (sta_mat->size2 != par->sta_fea_num)
			spa_io_saybye("The static feature's dimension does not match the configuration.\n");
		tot_fea_num = par->sta_fea_num;
	}
	else {
		spa_io_saybye("No static feature matrix.\n");
	}
	
	if (par->fea_kind & SPA_FEA_HASDELTA) {
		if (der_mat != NULL) {
			if (der_mat->size1 != frm_num)
				spa_io_saybye("The derivative feature's frame number does not match the static feature.\n");
			if (der_mat->size2 != par->sta_fea_num)
				spa_io_saybye("The derivative feature's dimension does not match the configuration.\n");
			tot_fea_num += par->sta_fea_num;
		}
		else {
			spa_io_saybye("No derivative feature matrix.\n");
		}
		
		if (par->fea_kind & SPA_FEA_HASACCS) {
			if (acc_mat != NULL) {
				if (acc_mat->size1 != frm_num) {
					spa_io_saybye("The accelerate feature's frame number does not match the static feature.\n");
				}
				if (acc_mat->size2 != par->sta_fea_num) {
					spa_io_saybye("The accelerate feature's dimension does not match the configuration.\n");
				}
				tot_fea_num += par->sta_fea_num;
			}
			else {
				spa_io_saybye("No accelerate feature matrix.\n");
			}
		}
	}
	
	htk_head.nSamples = frm_num;		// frame num
	htk_head.sampPeriod = par->frm_step_t * 1e4;	// in 100 ns
	htk_head.sampSize = sizeof(float) * tot_fea_num;
	htk_head.sampKind = par->fea_kind;
	
	// write down feature
	FILE *fp = spa_io_fopen(fea_file, "wb");
	spa_io_write_htk_header(fp, &htk_head);
	for (i = 0; i < frm_num; i++) {
		gv = gsl_matrix_row(sta_mat, i);
		spa_io_write_htk_vec(fp, &gv.vector);
		if (par->fea_kind & SPA_FEA_HASDELTA) {
			gv = gsl_matrix_row(der_mat, i);
			spa_io_write_htk_vec(fp, &gv.vector);
			if (par->fea_kind & SPA_FEA_HASACCS) {
				gv = gsl_matrix_row(acc_mat, i);
				spa_io_write_htk_vec(fp, &gv.vector);
			}
		}
	}
	fclose(fp);
}

///////////////////////////////////////////////////////////////////////////////
// extract and save feature for an utterance
void spa_spec_utt_fea (gsl_vector *sph, spa_spec_param *par, char *fea_file)
{
	int mfk;
	gsl_matrix *spec_mat, *fbe_mat, *fea_mat, *czo_mat, *leng_mat, *corr_mat, *resi_mat;
	gsl_matrix *sta_mat, *der_mat, *acc_mat;
	spa_htk_header *header;
	
	spec_mat = fbe_mat = fea_mat = czo_mat = leng_mat = corr_mat = resi_mat = NULL;
	sta_mat = der_mat = acc_mat = NULL;
	
	spa_spec_preemphasis(sph, par->pre_emph);
	
	if (par->fea_kind & SPA_FEA_HASENERGY) {
		spa_spec_utt_sph2eng (sph, par, &leng_mat);
	}
	
	mfk = par->fea_kind & SPA_FEA_MASK;
	if (mfk == SPA_FEA_LPC) {
		spa_spec_utt_sph2lpc(sph, par, &fea_mat, &corr_mat, &resi_mat);
	}
	else if (mfk == SPA_FEA_MFCC) {
		spa_spec_utt_sph2mag(sph, par, &spec_mat);
		spa_spec_utt_mag2fbe(par, spec_mat, &fbe_mat);
		spa_spec_utt_fbe2cep(par, fbe_mat, &fea_mat, &czo_mat);
	}
	else if (mfk == SPA_FEA_FBE) {
		spa_spec_utt_sph2mag(sph, par, &spec_mat);
		spa_spec_utt_mag2fbe(par, spec_mat, &fea_mat);
	}
	else if (mfk == SPA_FEA_SPEC) {
		spa_spec_utt_sph2mag(sph, par, &fea_mat);
	}
	
	else {
		spa_io_saybye("Do not have this feature option.\n");
	}
	
	if (mfk == SPA_FEA_LPC) { // only LPC
		spa_spec_save_lpcfea(par, fea_mat, corr_mat, resi_mat, fea_file);
	}
	else {
		spa_spec_append_stafea (par, fea_mat, czo_mat, leng_mat, &sta_mat);
		if (par->fea_kind & SPA_FEA_HASDELTA) {
			spa_spec_sta2der(sta_mat, &der_mat, par->der_win_num);
			if (par->fea_kind & SPA_FEA_HASACCS) {
				spa_spec_sta2der(der_mat, &acc_mat, par->acc_win_num);
			}
		}
		spa_spec_save_sdafea(par, sta_mat, der_mat, acc_mat, fea_file);
	}
	
	// release
	if (spec_mat != NULL)
		gsl_matrix_free(spec_mat);
	if (fbe_mat != NULL)
		gsl_matrix_free(fbe_mat);
	if (fea_mat != NULL)
		gsl_matrix_free(fea_mat);
	if (sta_mat != NULL)
		gsl_matrix_free(sta_mat);
	if (der_mat != NULL)
		gsl_matrix_free(der_mat);
	if (acc_mat != NULL)
		gsl_matrix_free(acc_mat);
	if (czo_mat != NULL)
		gsl_matrix_free(czo_mat);
	if (leng_mat != NULL)
		gsl_matrix_free(leng_mat);
	if (corr_mat != NULL)
		gsl_matrix_free(corr_mat);
	if (resi_mat != NULL)
		gsl_matrix_free(resi_mat);
}