////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                      Speech Processing Assitant                        //
//                           _____ ____  ___                              //
//                          / ___// __ \/   |                             //
//                          \__ \/ /_/ / /| |                             //
//                         ___/ / ____/ ___ |                             //
//                        /____/_/   /_/  |_|                             //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    spec_dom.h:                                                         //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#ifndef _SPA_SPEC_H
#define _SPA_SPEC_H

#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"
#include "simp_math.h"

//#define SPA_ZERO_FLOOR	(1.0E-20)	// floor for log(0)

#define SPA_WIN_SHAPE_HAMMING 4000
#define SPA_WIN_SHAPE_HANNING 4001

#define SPA_FEA_LPC		01		// LPC
#define SPA_FEA_MFCC		06		// MFCC
#define SPA_FEA_FBE		07		// FILTER BANK ENERGY
#define SPA_FEA_USER		011		// USER
#define SPA_FEA_SPEC		014		// SPECTRUM
#define SPA_FEA_MASK		077		// MASK BITS FOR FEATURE
//#define SPA_FEA_LPC_ACORR	021		// LPC WITH AUTOCORR

#define SPA_FEA_HASENERGY  0100       // _E log energy included
#define SPA_FEA_HASNULLE   0200       // _N absolute energy suppressed
#define SPA_FEA_HASDELTA   0400       // _D delta coef appended
#define SPA_FEA_HASACCS   01000       // _A acceleration coefs appended
#define SPA_FEA_HASCOMPX  02000       // _C is compressed
#define SPA_FEA_HASZEROM  04000       // _Z zero meaned
#define SPA_FEA_HASCRCC  010000       // _K has CRC check
#define SPA_FEA_HASZEROC 020000       // _0 0'th Cepstra included
#define SPA_FEA_HASVQ    040000       // _V has VQ index attached
#define SPA_FEA_HASTHIRD 0100000       // _T has Delta-Delta-Delta index attached

#define SPA_NONE_FB	-1		// no filter bank is used
#define SPA_MEL_TRI_FB	6000		// trianglar mel-freq filter bank
#define SPA_USR_FB	6001		// use user defined filter bank

#define SPA_LINEAR_SPEC 0		// use linear spectrum
#define SPA_LOG_SPEC 1			// use log spectrum
#define SPA_DB_SPEC 2			// use log spectrum in dB

typedef struct {
	int fft_num;		// fft size (512)
	int fft_half_num;	// half fft size = fft_num/2 + 1
	int win_shape;		// tappering window shape (SPA_WIN_SHAPE_HAMMING)
	int fb_shape;		// filter bank shape (SPA_MEL_TRI_FB)
	int fb_num;		// number of filter banks (26)
	int cep_num;		// number of cepstral coefficients (12)
	int cep_lift_num;	// number of cepstral lifter coefficients (22)
	int sta_fea_num;	// number of static features (13)
	int fea_num;		// sta + der + acc
	int der_win_num;	// the window length for calculating the derivative features (2)
	int acc_win_num;	// the window length for calculating the accelerate features (2)
	int fea_kind;		// feature kind code in the htk paramer table (MFCC_0)
	int use_log;		// use log/non-log spectrum t/f (true)
	int use_pow;		// use power/magnitude spectrum: t/f (true)
	int use_norm_eng;	// use normalized energy t/f (true)
	int strict_frm_time;	// use strict frame timing t/f (false)
	int frm_len;		// window length in samples (frm_len_t * 1e-3 * fsr)
	int frm_step;		// window step in samples (frm_step_t * 1e-3 * fsr)	
	int smp_rate;		// sampling rate in Hz
	double pre_emph;	// pre-emphasis factor (0.97)	
	double frm_len_t;	// window length in ms (10)
	double frm_step_t;	// window step in ms (25)	
	double fb_fmin;		// max frequecy of the filter bank (0)
	double fb_fmax;		// max frequecy of the filter bank (fsr/2)
	double sil_floor;	// log energy max to min ratio in dB (50.0)
	double eng_scale;	// log energy scaling ratio (0.1)
	// spa_htk_header *htk_header;	// HTK header
	double filt_ls;		// low stop frequecy of the filter
	double filt_lp;		// low stop frequecy of the filter
	double filt_hp;		// high pass frequecy of the filter
	double filt_hs;		// high stop frequecy of the filter
	gsl_vector *frm_win;	// tappering window for frame data before fft
	gsl_matrix *fb;		// filter bank
} spa_spec_param;

typedef struct _spa_spec_peak {
	int abs_sid;	// short term SNR index
	double abs_snr;	// short term SNR
	int diff_sid;	// diff SNR index
	double diff_snr;// normalized short term SNR - long term SNR
	int hid;	// peak location index
	int bid;	// id of the band which peak is in
	double harm;	// peak location in Hz
	int multi;	// freq = F0 * (multi + resi)
	double resi;	// residual: harm/pch - round(harm/pch)
	int fid;	// F0 index
	double pch;	// F0 in Hz
	double logwt;	// weight in deciding pitch value
} spa_spec_peak;

// for storing the peak info of a utterance, voiced/unvoiced frame: pk_num >0/=0
typedef struct _spa_vector_spec_peak {
	spa_spec_peak *pk_vec;	// array of peaks
	int pk_num;		// num of peaks
} spa_vector_spec_peak;

// Spectral processing initialization and release
gsl_vector *spa_spec_hamming_win(int win_num);
void spa_spec_init(char *cfg_fn, spa_spec_param **ppar);
void spa_spec_release(spa_spec_param *par);

void spa_spec_preemphasis (gsl_vector *s, double a);
void spa_spec_get_frm (gsl_vector *sph, gsl_vector *sph_frm, int st);
void spa_spec_fft_pow(gsl_vector *in, gsl_vector *out);
void spa_spec_fft_mag(gsl_vector *in, gsl_vector *out);
void spa_spec_dct(gsl_vector *in, gsl_vector *out);
void spa_spec_cep_lift (gsl_vector *in, gsl_vector *out, int cep_lift_num, int cep_num);

// get spectrum
void spa_get_frm_spec (gsl_vector *sph_frm, spa_spec_param *par, gsl_vector *spec_amp, int *log_flag);
void spa_spec_vector_linear2log(gsl_vector *s);			// linear -> log
void spa_spec_vector_log2db(gsl_vector *s, int flag_power);	// suppose power spectrum is used
void spa_spec_spec_sub (gsl_vector *s, gsl_vector *n);
void spa_spec_fea_frm_num (gsl_vector *sph, spa_spec_param *par, int *frm_num, int *st); // get the feature frame number
void spa_spec_utt_sph2mag (gsl_vector *sph, spa_spec_param *par, gsl_matrix **pspec_mat);
void spa_spec_utt_sph2eng (gsl_vector *sph, spa_spec_param *par, gsl_matrix **preng_mat);
void spa_spec_utt_sph2lpc (gsl_vector *sph, spa_spec_param *par, gsl_matrix **plpcc_mat,
			   gsl_matrix **pcorr_mat, gsl_matrix **presi_mat);
void spa_spec_utt_mag2fbe (spa_spec_param *par, gsl_matrix *spec_mat, gsl_matrix **pfbe_mat);
void spa_spec_utt_fbe2cep (spa_spec_param *par, gsl_matrix *fbe_mat, 
			   gsl_matrix **pcep_mat, gsl_matrix **pczo_mat);
			   
///////////////////////////////////////////////////////////////////////////////
// static feature to derivative features
// \sum_a a*[sta(i+a) - sta(i-a)] / 2 * \sum a^2
void spa_spec_sta2der (gsl_matrix *sta_mat, gsl_matrix **pder_mat, int der_fea_num);
///////////////////////////////////////////////////////////////////////////////
// save static, derivative and accelerate features
void spa_spec_save_sdafea (spa_spec_param *par, gsl_matrix *sta_mat, gsl_matrix *der_mat,
			   gsl_matrix *acc_mat, char *fea_file);
///////////////////////////////////////////////////////////////////////////////
// extract and save feature for an utterance
void spa_spec_utt_fea (gsl_vector *sph, spa_spec_param *par, char *fea_file);

#endif // SPA_SPEC_H