////////////////////////////////////////////////////////////////////////////
//                                                                        //
//           SAFE: a Statistically Algorithm for F0 Estimation            //
//                         _____ ___    ____________                      //
//                        / ___//   |  / ____/ ____/                      //
//                        \__ \/ /| | / /_  / __/                         //
//                       ___/ / ___ |/ __/ / /___                         //
//                      /____/_/  |_/_/   /_____/                         //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    PitchExt.c:                                                         //
//    This is the F0 estimator, which contains 2 stages:                  //
//    Stage 1: given a wave file, its voicing boundary file and trained   //
//        F0 estimation models, obtain posterior probabilities            //
//        of F0 candidates on each voiced frame and store it.             //
//    Stage 2: load the F0 candidates file from Stage 1 and F0 transition //
//        models trained from PitchTrain, then use a dynamic programming  //
//        to search the optimal F0 contour.                               //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

//
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "gsl/gsl_blas.h"

#include "io_shell.h"
#include "spec_dom.h"
#include "pconf.h"
#include "pitch.h"

extern char *msg;

// generate an lookup table for the truncated laplacian distribution
// tab - matrix(abs_snr_num x sample points)
void spa_get_trclap_table (spa_clus_resi_info *info, gsl_vector *vec)
{
	int i, j;
	double sum, x, p;
	double mu	= info->mu;
	double b	= info->b;
	gsl_vector *tab = info->pdf_table;
	sum = 0;
	for (i = 0; i < vec->size; i++) {
		x = gsl_vector_get(vec, i);
		p = spa_math_lap_pdf(x, mu, b);
		sum += p;
		gsl_vector_set(tab, i, p);
	}
	for (i = 0; i < vec->size; i++) {
		p = gsl_vector_get(tab, i) / sum;
		gsl_vector_set(tab, i, p);
	}
}


// load model P(f|snr, harm_loc) from file */
void spa_load_pch_mod(char *mod_file, spa_pitch_param *pch)
{
	int i, j, k, n, spkr_num ,cand_num, abs_snr_num, freq_band_num, resi_smp_num, resi_space;
	double d;
	gsl_vector *resi_smp_vec;
	FILE *fp = spa_io_fopen(mod_file, "rb");
	fread(&spkr_num, sizeof(int), 1, fp);
	fread(&cand_num, sizeof(int), 1, fp);
	fread(&abs_snr_num, sizeof(int), 1, fp);
	fread(&freq_band_num, sizeof(int), 1, fp);
	if (abs_snr_num != pch->abs_snr_num || freq_band_num != pch->freq_band_num) {
		printf("spa_load_pch_mod_file: the model mismatches with the configuration.\n");
		exit(GSL_FAILURE);
	}
	
	// load the variance of the laplacian distribution */
	pch->snr_band_resi = (spa_clus_resi_info **) alloc2d(abs_snr_num, freq_band_num, sizeof(spa_clus_resi_info));
	for (k = 0; k < abs_snr_num; k++) {
		for (n = 0; n < freq_band_num; n++) {
			fread(&pch->snr_band_resi[k][n].mu, 1, sizeof(double), fp);
			fread(&pch->snr_band_resi[k][n].b, 1, sizeof(double), fp);
			fread(&pch->snr_band_resi[k][n].p_snr, 1, sizeof(double), fp);
		}
	}
	spa_io_fclose(fp);
	
	// obtain the truncated laplacian pdf table */
	resi_smp_num = (int) (1.0 / pch->resi_space) + 1;
	resi_smp_vec = gsl_vector_calloc(resi_smp_num);
	for (i = 0; i < resi_smp_num; i++) {
		d = -0.5 + i * pch->resi_space;
		gsl_vector_set(resi_smp_vec, i, d);
	}
	
	for (k = 0; k < abs_snr_num; k++) {
		for (n = 0; n < freq_band_num; n++) {
			pch->snr_band_resi[k][n].pdf_table	= gsl_vector_calloc(resi_smp_num);
			spa_get_trclap_table (&pch->snr_band_resi[k][n], resi_smp_vec);
		}
	}
	gsl_vector_free(resi_smp_vec);
}

// load model P(f|snr, harm_loc) from file */
void spa_release_pch_mod(spa_pitch_param *pch)
{
	int k, n;
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			gsl_vector_free(pch->snr_band_resi[k][n].pdf_table);
		}
	}
	free2d((char **) pch->snr_band_resi);
}

int main ( int argc, char *argv[] )
{
	static char *spec_cfg	= "spec_cfg";
	static char *pch_cfg	= "pch_cfg";
	static char *mod_cfg_lst= "mod_cfg.lst";
	static char *in_file	= "in_file";
	static char *out_file 	= "cand_file";
	static char *pch_file	= "pch_file";
	static char *mod_file 	= "mod_file";
	static char *stage	= "1";
	
	int smp_rate, spkr_num, curr_num, frm_num, cand_num, i, j, k, l, m, mix_resi_num, istage, istatus;
	int nb_num, *nb_idx, row, col;
	double d, pp;
	char mod_cfg_file[SPA_MAX_STR_LEN], pro_path[SPA_MAX_STR_LEN];
	char ins_suffix[SPA_MAX_STR_LEN], fea_lst[SPA_MAX_STR_LEN], mod_path[SPA_MAX_STR_LEN];
	char **intra_mod_cfg_files, **inter_mod_cfg_files, **org_proto_path, *prior_mod_cfg_files;
	spa_spec_param *par;
	spa_pitch_param *pch;
	gsl_vector *frm_win, *sph, *pch_fin, *uv_ref;
	gsl_vector_view gv;
	spa_vector_spec_peak *peak_mat;
	gsl_matrix *cand_pp;
	spa_spec_peak *pv;
	FILE *fp_scp, *fp, *fp_m2f;

	static Confit params[] =
	{
		"i", "switch to stage i",
		"-s", cmuSTRING, ( caddr_t ) &stage,
	
		"scf", "Load signal processing config file scf",
		"-c1", cmuSTRING, ( caddr_t ) &spec_cfg,
	
		"pcf", "Load F0 estimation config file scf",
		"-c2", cmuSTRING, ( caddr_t ) &pch_cfg,
		
		"inf", "Load input from file inf",
		"-i", cmuSTRING, ( caddr_t ) &in_file,
	
		"uvf", "Load voicing information from file uvf",
		"-f", cmuSTRING, ( caddr_t ) &pch_file,
	
		"mod", "Load F0 estimatin models from file mod",
		"-m", cmuSTRING, ( caddr_t ) &mod_file,
	
		"ouf", "Save output to file ouf",
		"-o", cmuSTRING, ( caddr_t ) &out_file,
	
		0, 0, 0, NOTYPE, 0
	};

	if (argc < 13 || ppconf(argc, argv, params, true)) {
		printf("\nUsage: PitchExt [options]\n");
		pusage (argv[0], params);
		return -1;
	}
	
	// Initialization */
	spa_spec_init(spec_cfg, &par);
	spa_spec_pch_init(pch_cfg, par, &pch);
	istage = atoi(stage);
	
	if (SPA_PITCH_STAGE_CAND == istage) {
		if (argc < 15) {
			pusage (argv[0], params);
			return -1;
		}
		printf("File: %s\n", in_file);
		smp_rate = spa_io_read_wav(in_file, &sph, par->smp_rate);
		
		// obtain peak candidates
		spa_load_pch_mod(mod_file, pch);
		spa_load_pch_ref_file(pch_file, &uv_ref);
		spa_pitch_peak_cand(par, pch, sph, uv_ref, &peak_mat, &frm_num);
		
		// generate posterior score for each frame
		spa_pitch_cand_postprob (pch, peak_mat, frm_num, &cand_pp);
		
		// write down F0 cands
		fp = spa_io_fopen(out_file, "wt");
		fwrite(&frm_num, sizeof(int), 1, fp);
		fwrite(&pch->cand_num, sizeof(int), 1, fp);
		for(i = 0; i < frm_num; i++) {
			for(j = 0; j < pch->cand_num; j++) {
				d = gsl_matrix_get(cand_pp, i, j);
				fwrite(&d, 1, sizeof(double), fp);
			}
		}
		spa_io_fclose(fp);
		// printf("done.\n");
		
		//spa_release_pch_mod(pch);
		gsl_vector_free(sph);
		gsl_matrix_free(cand_pp);
		gsl_vector_free(uv_ref);
		for(i = 0; i < frm_num; i++) {
			if (peak_mat[i].pk_num > 0) {
				free(peak_mat[i].pk_vec);
			}
		}
		free(peak_mat);
	}
	else if (SPA_PITCH_STAGE_DP == istage) {
		// load F0 cands
		fp = spa_io_fopen(in_file, "rb");
		fread(&frm_num, sizeof(int), 1, fp);
		fread(&cand_num, sizeof(int), 1, fp);
		cand_pp = gsl_matrix_calloc(frm_num, cand_num);
		for(i = 0; i < frm_num; i++) {
			for(j = 0; j < cand_num; j++) {
				fread(&d, 1, sizeof(double), fp);
				gsl_matrix_set(cand_pp, i, j, d);
			}
		}
		spa_io_fclose(fp);
		// printf("\tF0 candidates are loaded.\n\n");
		
		// load voicing information
		spa_load_pch_ref_file(pch_file, &uv_ref);
		
		spa_pitch_get_trans_cost_table(pch);
		spa_pitch_cand_dp(pch, frm_num, cand_pp, uv_ref, &pch_fin);
		
		// write down F0 values
		// printf("%s\n", out_file);
		fp = spa_io_fopen(out_file, "wt");
		for(i = 0; i < pch_fin->size; i++) {
			fprintf(fp, "%.2f\n", gsl_vector_get(pch_fin, i));
		}
		spa_io_fclose(fp);
		gsl_vector_free(pch_fin);
		gsl_vector_free(uv_ref);
	}
	else {
		printf("An invalid stage flag (%d).\n", istage);
	}
	
	spa_spec_release(par);
	spa_spec_pch_release(pch);
	
	// printf("\tMemory is released.\n\n");
	
	return GSL_SUCCESS;
}

