////////////////////////////////////////////////////////////////////////////
//                                                                        //
//           SAFE: a Statistically Algorithm for F0 Estimation            //
//                         _____ ___    ____________                      //
//                        / ___//   |  / ____/ ____/                      //
//                        \__ \/ /| | / /_  / __/                         //
//                       ___/ / ___ |/ __/ / /___                         //
//                      /____/_/  |_/_/   /_____/                         //
//                                                                        //
//                   Copyright © 2011-2020 by Wei Chu                     //
//                          weichu@ucla.edu                               //
//                                                                        //
//               This software is released under the LGPL                 //
//         See the file COPYING.LESSER for the conditions of use          //
//                This banner notice must not be removed                  //
//                                                                        //
//    PitchTrain.c:                                                       //
//    This is the training of F0 estimation and transition models, which  //
//    has 3 stages:                                                       //
//    Stage 1:                                                            //
//        Accumulate statistics for F0 estimation model training          //
//    Stage 2:                                                            //
//        Train F0 estimation models using statistics from Stage 1        //
//    Stage 3:                                                            //
//        Train F0 transition models using F0 reference files             //
//                                                                        //
//    Created: Apr 1, 2011                                                //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#include "io_shell.h"
#include "spec_dom.h"
#include "pconf.h"
#include "pitch.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "gsl/gsl_blas.h"
#include <gsl/gsl_errno.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_roots.h>

#include "io_shell.h"
#include "spec_dom.h"
#include "pconf.h"
#include "pitch.h"

extern char *msg;

typedef struct _spa_laplacian_param {
	spa_resi_info *data;
	int num;
	double a;
} spa_laplacian_param;

spa_laplacian_param g_params;

// f(a) = 1/a - 1/(1-e^-a) + c
double func_laplacian_var (double x, void *params)
{
	double ex;
	if (x < SPA_DSMALL) {
		printf("func_laplacian_var: the alpha should be a positive value large enough.\n", x);
	}
	double a = g_params.a;

	ex = x > SPA_EXPINF ? SPA_DLARGE : exp(x);
	double d = 1.0 / x - 1.0 / (ex - 1.0) - 2.0 * a;
	return d;
}

double func_laplacian_mean (double x, void *params)
{
	int i;
	double sum;
	spa_resi_info *data = g_params.data;
	int num = g_params.num;
	sum = 0;
	for (i = 0; i < num; i++) {
		sum += spa_math_sgn(x - data[i].resi);
	}
	return sum;
}

// f'(a) = 1/(1-e^-a) - 1.0/x^2
double func_laplacian_related_derivative (double x, void *params)
{
	double ex;
	if (x < SPA_DSMALL) {
		printf("func_laplacian_related_derivative: the alpha should be a positive value large enough.\n", x);
	}
	
	ex = x > SPA_EXPINF ? SPA_DLARGE : exp(x);
	double d = ex / ((ex - 1.0) * (ex - 1.0)) - 1.0 / (x * x);
	return d;
}

void func_laplacian_related_fdf (double x, void *params, double *y, double *dy)
{
	double ex, ry, rdy;
	if (x < SPA_DSMALL) {
		printf("func_laplacian_related_derivative: the alpha should be a positive value large enough.\n", x);
	}
	
	//double a = ((spa_laplacian_param *) params)->a;
	double a = g_params.a;
	
	ex = x > SPA_EXPINF ? SPA_DLARGE : exp(x);
	*y = 1.0 / x - 1.0 / (ex - 1.0) - 2.0 * a;
	*dy = ex / ((ex - 1.0) * (ex - 1.0)) - 1.0 / (x * x);
	ry = *y;
	rdy = *dy;
}

int solve_resi_pdf_deriv(spa_resi_info *vec, int num, double *px)
{
	int i, j, k, status;
	int iter = 0, max_iter = 1000;
	double abs_sum, x0 = 10, x = 100, r_expected = 1;	
	const gsl_root_fdfsolver_type *T;
	gsl_root_fdfsolver *s;
	gsl_function_fdf FDF;
	spa_laplacian_param params;
	params.a = 0.0;

	// need to accumulated information: \sum |x|
	abs_sum = 0.0;
	for (i = 0; i < num; i++) {
		abs_sum += fabs(vec[i].resi) / num;
	}
	g_params.a = abs_sum;
	
	FDF.f = &func_laplacian_var;
	FDF.df = &func_laplacian_related_derivative;
	FDF.fdf = &func_laplacian_related_fdf;
	FDF.params = &params;
     
	T = gsl_root_fdfsolver_newton;
	s = gsl_root_fdfsolver_alloc (T);
	gsl_root_fdfsolver_set (s, &FDF, x0);
	//gsl_root_fsolver_set (s, &FDF, x_lo, x_hi);
	printf ("using %s method\n", gsl_root_fdfsolver_name (s));
     
	printf ("%-5s %10s %10s %10s\n",
		"iter", "root", "err", "err(est)");
	do
	{
		iter++;
		status = gsl_root_fdfsolver_iterate (s);
		x0 = x;
		x = gsl_root_fdfsolver_root (s);
		printf ("Current root: %e\n", s->root);
		status = gsl_root_test_delta (x, x0, 0, 1e-6);
     
		if (status == GSL_SUCCESS)
			printf ("Converged:\n");
     
		printf ("%5d %10.7f %+10.7f %10.7f\n",
			iter, x, x - r_expected, x - x0);
	}
	while (status == GSL_CONTINUE && iter < max_iter);
     
	gsl_root_fdfsolver_free (s);
	*px = x;
	return GSL_SUCCESS;
}

// emu_flag the flag of whether estimate mean
int solve_resi_pdf_bracket(spa_resi_info *vec, int num, double *pmu, double *pvar, int emu_flag)
{
	int i, j, k, status;
	int iter = 0, max_iter = 1000;
	double abs_sum, x, x_lo = SPA_DSMALL, x_hi = SPA_DLARGE, mu, var;	
	const gsl_root_fsolver_type *T;
	gsl_root_fsolver *s;
	gsl_function FDF;
	spa_laplacian_param params;

	g_params.data = vec;
	g_params.num = num;
	
	// solve mean first
	if (emu_flag) {
		FDF.function = &func_laplacian_mean;
		FDF.params = &params;
		x_lo = -0.5;
		x_hi = 0.5;
		T = gsl_root_fsolver_brent;
		s = gsl_root_fsolver_alloc (T);
		gsl_root_fsolver_set (s, &FDF, x_lo, x_hi);
		iter = 0;
		do {
			iter++;
			status = gsl_root_fsolver_iterate (s);
			mu = gsl_root_fsolver_root (s);
			x_lo = gsl_root_fsolver_x_lower (s);
			x_hi = gsl_root_fsolver_x_upper (s);
			status = gsl_root_test_interval (x_lo, x_hi,
					0, 1e-3);
		}
		while (status == GSL_CONTINUE && iter < max_iter);
		gsl_root_fsolver_free (s);
	}
	else {
		mu = 0.0;
	}
	
	// then solve the variance
	// need to accumulated information: \sum |x - mu|
	abs_sum = 0.0;
	for (i = 0; i < num; i++) {
		abs_sum += fabs(vec[i].resi - mu) / num;
	}
	g_params.a = abs_sum;
	
	FDF.function = &func_laplacian_var;
	FDF.params = &params;
	x_lo = SPA_ASMALL;
	x_hi = SPA_ALARGE;
	T = gsl_root_fsolver_brent;
	s = gsl_root_fsolver_alloc (T);
	gsl_root_fsolver_set (s, &FDF, x_lo, x_hi);
	iter = 0;
	do {
		iter++;
		status = gsl_root_fsolver_iterate (s);
		x = gsl_root_fsolver_root (s);
		x_lo = gsl_root_fsolver_x_lower (s);
		x_hi = gsl_root_fsolver_x_upper (s);
		status = gsl_root_test_interval (x_lo, x_hi,
				0, 1e-3);
	}
	while (status == GSL_CONTINUE && iter < max_iter);
	gsl_root_fsolver_free (s);
	
	*pmu = mu;
	*pvar = 0.5 / x;
	return GSL_SUCCESS;
}

void save_residual_model (char *mod_file, spa_pitch_param *pch, int utt_num, spa_clus_resi_info **snr_band_resi)
{
	int k, n;
	FILE *fp;
	
	fp = spa_io_fopen(mod_file, "wb");
	fwrite(&utt_num, sizeof(int), 1, fp);
	fwrite(&pch->cand_num, sizeof(int), 1, fp);
	fwrite(&pch->abs_snr_num, sizeof(int), 1, fp);
	fwrite(&pch->freq_band_num, sizeof(int), 1, fp);
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			fwrite(&snr_band_resi[k][n].mu, 1, sizeof(double), fp);
			fwrite(&snr_band_resi[k][n].b, 1, sizeof(double), fp);
			fwrite(&snr_band_resi[k][n].p_snr, 1, sizeof(double), fp);
		}
	}
	spa_io_fclose(fp);
}

// Train a model fo F0 estimation on each frame
// input:
//	par		spec params
//	pch		pitch params
//	w2f_lst		wave to F0 reference file list
// output:
//	mod_file	model files (*.acc statistics file, *.zm.pdf: model files with zero means)
void pitch_accu_stats_and_save (spa_spec_param *par, spa_pitch_param *pch, char *w2f_lst, char *mod_file)
{
	int smp_rate, utt_num, curr_num, frm_num, resi_num, total_num;	
	int i, j, k, l, m, n, mix_resi_num, next_num, inter_num;
	double d, sum = 0;

	char wav_file[SPA_MAX_STR_LEN];
	char pch_file[SPA_MAX_STR_LEN];
	char acc_file[SPA_MAX_STR_LEN];
	char suffix[SPA_MAX_STR_LEN];
	
	gsl_vector *frm_win, *sph, *pch_ref;
	spa_vector_spec_peak *peak_mat;
	spa_spec_peak *pv;
	FILE *fp_scp, *fp;
	
	// data storage
	spa_clus_resi_info ****spsb_resi;
	spa_clus_resi_info **snr_band_resi;
	
	spa_io_get_suffix(mod_file, suffix);
	spa_io_replace_suffix (acc_file, mod_file, "acc");

	fp_scp = spa_io_fopen(w2f_lst, "rt");
	utt_num = 0;
	while (1) {
		if(fscanf(fp_scp, "%s %s\n", wav_file, pch_file) != 2)
			break;
		utt_num++;
	}
	rewind(fp_scp);
	
	// init the storage spsb: speaker, pitch, absolute SNR, band
	spsb_resi	= (spa_clus_resi_info ****) alloc4d(utt_num, pch->cand_num, pch->abs_snr_num, pch->freq_band_num, sizeof(spa_clus_resi_info));
	curr_num = 0;
	
	printf("\n\taccumulating statistics ... \n");
	while (1) {
		if (fscanf(fp_scp, "%s %s\n", wav_file, pch_file) != 2)
			break;
		printf("    (Pitch Train) - File: %s\n", wav_file);
		smp_rate = spa_io_read_wav(wav_file, &sph, par->smp_rate);
		spa_load_pch_ref_file(pch_file, &pch_ref);
		// extract information from speech
		spa_pitch_peak_cand(par, pch, sph, pch_ref, &peak_mat, &frm_num);
		// learn statistics or generate pitch candidates
		spa_pitch_accu_cand_occurrence(pch, peak_mat, frm_num, pch_ref, spsb_resi[curr_num]);
		
		gsl_vector_free(sph);
		gsl_vector_free(pch_ref);
		for(i = 0; i < frm_num; i++) {
			if (peak_mat[i].pk_num > 0) {
				free(peak_mat[i].pk_vec);
			}
		}
		free(peak_mat);
		curr_num++;
	}
	spa_io_fclose(fp_scp);
	
	// write down statistics
	fp = spa_io_fopen(acc_file, "wb");
	fwrite(&utt_num, sizeof(int), 1, fp);
	fwrite(&pch->cand_num, sizeof(int), 1, fp);
	fwrite(&pch->abs_snr_num, sizeof(int), 1, fp);
	fwrite(&pch->freq_band_num, sizeof(int), 1, fp);
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			resi_num = 0;
			for (i = 0; i < utt_num; i++) {
				for (j = 0; j < pch->cand_num; j++) {
					resi_num += spsb_resi[i][j][k][n].resi_num;
				}
			}
			if (resi_num == 0) {
				printf("abs_snr bin:%d, band num:%d\n", k, n);
			}
			total_num += resi_num;
			fwrite(&resi_num, sizeof(int), 1, fp);
			for (i = 0; i < utt_num; i++) {
				for (j = 0; j < pch->cand_num; j++) {
					for (l = 0; l < spsb_resi[i][j][k][n].resi_num; l++) {
						fwrite(&spsb_resi[i][j][k][n].resi_vec[l].abs_snr, 1, sizeof(double), fp);
						fwrite(&spsb_resi[i][j][k][n].resi_vec[l].multi, 1, sizeof(int),	   fp);
						fwrite(&spsb_resi[i][j][k][n].resi_vec[l].resi,	1, sizeof(double), fp);
					}
				}
			}
		}
	}
	
	printf("\n\tReleasing memory ...\n");
	// release	
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			for (i = 0; i < utt_num; i++) {
				for (j = 0; j < pch->cand_num; j++) {
					free(spsb_resi[i][j][k][n].resi_vec);
				}
			}
		}
	}
	free4d((char ****) spsb_resi);
}

// Train a model fo F0 estimation
// input:
//	par		spec params
//	pch		pitch params
//	w2f_lst		wave to F0 reference file list
// output:
//	mod_file	model files (*.acc statistics file, *.zm.pdf: model files with zero means)
void pitch_load_stats_and_train_model (spa_spec_param *par, spa_pitch_param *pch, char *w2f_lst, char *mod_file)
{
	int smp_rate, utt_num, curr_num, frm_num, resi_num, total_num;	
	int i, j, k, l, m, n, mix_resi_num, next_num, inter_num;
	double d, sum = 0;

	char wav_file[SPA_MAX_STR_LEN];
	char pch_file[SPA_MAX_STR_LEN];
	char acc_file[SPA_MAX_STR_LEN];
	char mzm_file[SPA_MAX_STR_LEN];			// store a zero mean pdf
	char suffix[SPA_MAX_STR_LEN];
	
	gsl_vector *frm_win, *sph, *pch_ref;
	spa_vector_spec_peak *peak_mat;
	spa_spec_peak *pv;
	FILE *fp_scp, *fp;
	
	// data storage
	spa_clus_resi_info **snr_band_resi;
	
	spa_io_get_suffix(mod_file, suffix);
	spa_io_replace_suffix (acc_file, mod_file, "acc");
	spa_io_replace_suffix (mzm_file, mod_file, "zm.pdf");
	
	fp = spa_io_fopen(acc_file, "rb");
	fread(&utt_num, sizeof(int), 1, fp);
	fread(&pch->cand_num, sizeof(int), 1, fp);
	fread(&pch->abs_snr_num, sizeof(int), 1, fp);
	fread(&pch->freq_band_num, sizeof(int), 1, fp);
	snr_band_resi = (spa_clus_resi_info **) alloc2d(pch->abs_snr_num, pch->freq_band_num, sizeof(spa_clus_resi_info));
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			fread(&snr_band_resi[k][n].resi_num, sizeof(int), 1, fp);
			snr_band_resi[k][n].resi_vec = (spa_resi_info *) calloc(snr_band_resi[k][n].resi_num, sizeof(spa_resi_info));
			for (l = 0; l < snr_band_resi[k][n].resi_num; l++) {
				fread(&snr_band_resi[k][n].resi_vec[l].abs_snr, 1, sizeof(double), fp);
				fread(&snr_band_resi[k][n].resi_vec[l].multi, 1, sizeof(int), fp);
				i = fread(&snr_band_resi[k][n].resi_vec[l].resi, 1, sizeof(double), fp);
			}
		}
	}
	spa_io_fclose(fp);
	printf("Accumulated statistics loaded.\n");
	
	// estimate snr-band info
	for (n = 0; n < pch->freq_band_num; n++) {
		sum = 0;
		for (k = 0; k < pch->abs_snr_num; k++) {
			sum += snr_band_resi[k][n].resi_num;
		}
		for (k = 0; k < pch->abs_snr_num; k++) {
			snr_band_resi[k][n].p_snr = (double) snr_band_resi[k][n].resi_num / sum;
		}
	}
	
	// estimate residual distribution
	// 1. zero mean
	printf("\n\tEstimating zero mean residual models ...\n");
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			if (snr_band_resi[k][n].resi_num > 10) {
				solve_resi_pdf_bracket(snr_band_resi[k][n].resi_vec, snr_band_resi[k][n].resi_num,
						&snr_band_resi[k][n].mu, &snr_band_resi[k][n].b, 0);
				//solve_resi_pdf_deriv(snr_band_resi[k][n].resi_vec, snr_band_resi[k][n].resi_num, &snr_band_resi[k][n].b);
			}
			else {
				snr_band_resi[k][n].mu = 0.0;
				snr_band_resi[k][n].b = 0.01;
			}
		}
	}
	save_residual_model(mzm_file, pch, utt_num, snr_band_resi);
	
	// 2. non-zero mean
	printf("\n\tEstimating non-zero mean residual models ...\n");
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			if (snr_band_resi[k][n].resi_num > 10) {
				solve_resi_pdf_bracket(snr_band_resi[k][n].resi_vec, snr_band_resi[k][n].resi_num,
						&snr_band_resi[k][n].mu, &snr_band_resi[k][n].b, 1);
				//solve_resi_pdf_deriv(snr_band_resi[k][n].resi_vec, snr_band_resi[k][n].resi_num, &snr_band_resi[k][n].b);
			}
			else {
				snr_band_resi[k][n].mu = 0.0;
				snr_band_resi[k][n].b = 0.01;
			}
		}
	}
	save_residual_model(mod_file, pch, utt_num, snr_band_resi);
	
	
	// release	
	printf("\n\tReleasing memory ...\n");
	for (k = 0; k < pch->abs_snr_num; k++) {
		for (n = 0; n < pch->freq_band_num; n++) {
			free(snr_band_resi[k][n].resi_vec);
		}
	}
	free2d((char **) snr_band_resi);
}

// replace the model name in the proto
void pitch_train_create_proto_file (char *old_pro, char *new_pro, char *mod_name)
{
	char line[SPA_MAX_STR_LEN], *p;
	FILE *fp_in, *fp_out;
	fp_in = spa_io_fopen(old_pro, "rt");
	fp_out = spa_io_fopen(new_pro, "wt");
	
	int count = 0;
	while ( fgets(line, SPA_MAX_STR_LEN, fp_in) != NULL ) {
		if(strstr(line, "<NAME>")) {
			p = strtok (line, " ");
			while (p != NULL) {
				if (!strcmp(p, "<NAME>")) {
					p = strtok (NULL, " ");
					fprintf(fp_out, "<NAME> %s.%d", mod_name, count++);
				}
				else {
					p = strtok (NULL, " ");
					if (p != NULL) {
						fprintf(fp_out, " %s", p);
					}
				}
			}
		}
		else {
			fputs(line, fp_out);
		}
	}
	
	spa_io_fclose(fp_out);
	spa_io_fclose(fp_in);
}

int main ( int argc, char *argv[] )
{
	static char *str_switch = "1";
	static char *spec_cfg	= "spec_cfg";
	static char *pch_cfg	= "pch_cfg";
	static char *file_lst	= "file_lst";
	static char *cand_file 	= "cand_file";
	static char *mod_file 	= "mod_file";
	int switch_flag;
	spa_spec_param *par;
	spa_pitch_param *pch;	
	
	static Confit params[] =
	{
		"i", "Switch to stage i",
		"-s", cmuSTRING, ( caddr_t ) &str_switch,

		"scf", "Load signal processing config from file scf",
		"-c1", cmuSTRING, ( caddr_t ) &spec_cfg,
	
		"pcf", "Load F0 estimation config from file scf",
		"-c2", cmuSTRING, ( caddr_t ) &pch_cfg,
		
		"w2f", "Load wave-F0 grouth truth from file list w2f",
		"-l", cmuSTRING, ( caddr_t ) &file_lst,
	
		"mod", "Save F0 estimation models to file mod",
		"-m", cmuSTRING, ( caddr_t ) &mod_file,
	
		0, 0, 0, NOTYPE, 0
	};

	if (argc < 11 || ppconf(argc, argv, params, true)) {
		printf("\nUsage: PitchTrain [options]\n");
		pusage (argv[0], params);
		return -1;
	}
	
	// Initialization
	spa_spec_init(spec_cfg, &par);
	spa_spec_pch_init(pch_cfg, par, &pch);
	
	switch_flag = atoi(str_switch);
	switch (switch_flag)
	{
		case SPA_PITCH_ACCU_STATS:
			printf("\n\tstarting accumulating statistics for F0 estimation ...\n");
			pitch_accu_stats_and_save(par, pch, file_lst, mod_file);
			break;
		case SPA_PITCH_TRAIN_MODEL:
			printf("\n\tstarting training models for F0 estimation ...\n");
			pitch_load_stats_and_train_model(par, pch, file_lst, mod_file);
			break;
		default:
			printf("An invalid switch flag (%d).\n", switch_flag);
			break;
	}

	
	spa_spec_pch_release(pch);
	spa_spec_release(par);
	printf("\n\tDone.\n");
	return GSL_SUCCESS;
}

